Package: DEGreport
Version: 1.48.0
Date: 2025-10-11
Type: Package
Title: Report of DEG analysis
Description: Creation of ready-to-share figures of differential expression
    analyses of count data. It integrates some of the code
    mentioned in DESeq2 and edgeR vignettes, and report a ranked
    list of genes according to the fold changes mean and
    variability for each selected gene.
biocViews: DifferentialExpression, Visualization, RNASeq,
        ReportWriting, GeneExpression, ImmunoOncology
URL: http://lpantano.github.io/DEGreport/
BugReports: https://github.com/lpantano/DEGreport/issues
Suggests: BiocStyle, AnnotationDbi, limma, pheatmap, rmarkdown,
        statmod, testthat
Depends: R (>= 4.0.0)
Imports: utils, methods, Biobase, BiocGenerics, broom, circlize,
        ComplexHeatmap, cowplot, ConsensusClusterPlus, cluster,
        dendextend, DESeq2, dplyr, edgeR, ggplot2, ggdendro, grid,
        ggrepel, grDevices, knitr, logging, magrittr, psych,
        RColorBrewer, reshape, rlang, scales, stats, stringr, stringi,
        S4Vectors, SummarizedExperiment, tidyr, tibble
Authors@R: c(person("Lorena", "Pantano", email = "lorena.pantano@gmail.com", 
  role = c("aut", "cre")),
  person("John", "Hutchinson",  role = c("ctb")),
  person("Victor", "Barrera",  role = c("ctb")),
  person("Mary", "Piper",  role = c("ctb")),
  person("Radhika", "Khetani",  role = c("ctb")),
  person("Kenneth", "Daily", role=c("ctb")),
  person("Thanneer", "Malai Perumal", role=c("ctb")),
  person("Rory", "Kirchner",  role=c("ctb")),
  person("Michael", "Steinbaugh",  role=c("ctb")),
  person("Ivo", "Zeller",  role=c("ctb")))
Maintainer: Lorena Pantano <lorena.pantano@gmail.com>
License: MIT + file LICENSE
VignetteBuilder: knitr
RoxygenNote: 7.3.1
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/DEGreport
git_branch: RELEASE_3_23
git_last_commit: bc514a8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:39:54 UTC; biocbuild
Author: Lorena Pantano [aut, cre],
  John Hutchinson [ctb],
  Victor Barrera [ctb],
  Mary Piper [ctb],
  Radhika Khetani [ctb],
  Kenneth Daily [ctb],
  Thanneer Malai Perumal [ctb],
  Rory Kirchner [ctb],
  Michael Steinbaugh [ctb],
  Ivo Zeller [ctb]

# Generated by roxygen2: do not edit by hand

export(DEGSet)
export(as.DEGSet)
export(createReport)
export(deg)
export(degCheckFactors)
export(degColors)
export(degComps)
export(degCorCov)
export(degCorrect)
export(degCovariates)
export(degDefault)
export(degFilter)
export(degMA)
export(degMB)
export(degMDS)
export(degMV)
export(degMean)
export(degObj)
export(degPCA)
export(degPatterns)
export(degPlot)
export(degPlotCluster)
export(degPlotWide)
export(degQC)
export(degResults)
export(degSignature)
export(degSummary)
export(degVB)
export(degVar)
export(degVolcano)
export(geom_cor)
export(significants)
exportMethods(as.DEGSet)
exportMethods(deg)
exportMethods(degCorrect)
exportMethods(degDefault)
exportMethods(significants)
import(cluster)
import(edgeR)
import(ggplot2)
import(ggrepel)
import(logging)
import(utils)
importFrom("grDevices","colorRampPalette")
importFrom("grDevices","dev.off")
importFrom("grDevices","jpeg")
importFrom("grDevices","pdf")
importFrom("grDevices","rgb")
importFrom("methods","slotNames")
importFrom("stats","aov")
importFrom("stats","as.dendrogram")
importFrom("stats","as.dist")
importFrom("stats","as.hclust")
importFrom("stats","cmdscale")
importFrom("stats","cor")
importFrom("stats","cor.test")
importFrom("stats","cutree")
importFrom("stats","density")
importFrom("stats","dist")
importFrom("stats","hclust")
importFrom("stats","median")
importFrom("stats","median.default")
importFrom("stats","na.omit")
importFrom("stats","p.adjust")
importFrom("stats","prcomp")
importFrom("stats","quantile")
importFrom("stats","sd")
importFrom("stats","time")
importFrom("stats","var")
importFrom("stats","xtabs")
importFrom(Biobase,rowMax)
importFrom(Biobase,rowMin)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,HeatmapAnnotation)
importFrom(ConsensusClusterPlus,ConsensusClusterPlus)
importFrom(ConsensusClusterPlus,calcICL)
importFrom(DESeq2,lfcShrink)
importFrom(DESeq2,plotCounts)
importFrom(DESeq2,results)
importFrom(DESeq2,resultsNames)
importFrom(DESeq2,rlog)
importFrom(RColorBrewer,brewer.pal)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,SimpleList)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(broom,tidy)
importFrom(circlize,colorRamp2)
importFrom(cowplot,draw_plot)
importFrom(cowplot,ggdraw)
importFrom(cowplot,plot_grid)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,rename)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,select_if)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(ggdendro,dendro_data)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,layer)
importFrom(grid,textGrob)
importFrom(knitr,kable)
importFrom(magrittr,set_colnames)
importFrom(magrittr,set_rownames)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(psych,corr.test)
importFrom(reshape,melt)
importFrom(reshape,melt.data.frame)
importFrom(rlang,sym)
importFrom(scales,math_format)
importFrom(scales,trans_breaks)
importFrom(scales,trans_format)
importFrom(stringr,str_split)
importFrom(tibble,as_tibble)
importFrom(tibble,column_to_rownames)
importFrom(tibble,remove_rownames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(tidyr,unite)
importMethodsFrom(BiocGenerics,plotMA)

# DEGreport

<img src="https://raw.githubusercontent.com/lpantano/DEGreport/main/inst/sticker/degreport.png" width="150" height="150" align="right"/>


[![Build status](https://secure.travis-ci.org/lpantano/DEGreport.png)](https://travis-ci.org/lpantano/DEGreport)
[![Project Status: Active - The project has reached a stable, usable state and is being actively developed.](http://www.repostatus.org/badges/latest/active.svg)](http://www.repostatus.org/#active)
[![BioC Downloads](http://bioconductor.org//shields/downloads/DEGreport.svg)](http://bioconductor.org/packages/3.6/bioc/html/DEGreport.html)
[![coverage](https://img.shields.io/codecov/c/github/lpantano/DEGreport/master.svg)](https://codecov.io/github/lpantano/DEGreport?branch=master)
[![Support this project by running your production jobs at BatchX](https://images.batchx.io/gh-badge-logo.svg)](https://platform.batchx.io/lpantano-team/tools/degreport%2Fdeg-patterns "Support this project by running your production jobs at BatchX")

QCs, figures and analyses after differential expression with DESeq2 or other similar tool.

Pantano L (2017). DEGreport: Report of DEG analysis. R package version 1.13.8. 

DOI: [10.18129/B9.bioc.DEGreport](https://doi.org/doi:10.18129/B9.bioc.DEGreport)


## Installation

This is an [R][] package.

### [Bioconductor][] stable version

```r
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("DEGreport")
```

### [Bioconductor][] latest version

```r
devtools::install_git("https://git@git.bioconductor.org/packages/DEGreport")
```

### [devtools][] unstable version

```r
install.packages("devtools")
devtools::install_github("lpantano/DEGreport")
```

Load the package and learn what to do:

`browseVignettes("DEGreport")`

Here is the last devel [vignette](https://bioconductor.org/packages/devel/bioc/vignettes/DEGreport/inst/doc/DEGreport.pdf).

[Bioconductor]: https://bioconductor.org
[bcbio-nextgen]: https://github.com/chapmanb/bcbio-nextgen
[devtools]: https://cran.r-project.org/package=devtools
[R]: https://www.r-project.org

Identifies clusters in a DESeq2 object using the `degPatterns` function from [DEGreport](https://github.com/lpantano/DEGreport).

# Context 
This tool clusters elements that show a similar pattern or behavior accross group of samples using the [`degPatterns`](https://rdrr.io/github/lpantano/DEGreport/man/degPatterns.html) function. Two different approaches to cluster elements are supported: [`diana`](https://www.rdocumentation.org/packages/cluster/versions/2.1.2/topics/diana) and [`ConsensusClusterPlus`](https://bioconductor.org/packages/release/bioc/html/ConsensusClusterPlus.html). 

The most common use case for `degPatterns` is to cluster genes in an [RNA-Seq](https://en.wikipedia.org/wiki/RNA-Seq) differential expression analysis; however, it can also be used in [ChIP-seq](https://en.wikipedia.org/wiki/ChIP_sequencing) for [differential peak binding](https://github.com/crazyhottommy/ChIP-seq-analysis/blob/master/part3_Differential_binding_by_DESeq2.md) as well as in [Metagenomics](https://www.genome.gov/genetics-glossary/Metagenomics), for [differential abundance](https://joey711.github.io/phyloseq-extensions/DESeq2.html) studies.
 
# Inputs
## Required inputs
This tool has the following **required** inputs:

1. **`deseq2Object`** 

	DESeq2 object in [RDS](https://www.r-bloggers.com/2016/12/remember-to-use-the-rds-format/) format. This object is generated after running the [`DESeq`](https://www.rdocumentation.org/packages/DESeq2/versions/1.12.3/topics/DESeq) function from DESeq2.   

2. **`explanatoryVar`** 

	Explanatory variable in the DESeq2 object. In an experimental study, it is the variable that is manipulated by the researcher (e.g. `time`). It will be used on the x-axis of the plots. This variable must exist in the DESeq2 object. 
   
	If the `designFile` shown below was used to run a differential expression analysis with DESeq2. The variable `time` could then be used as `explanatoryVar` when running `degPatterns`.

```console
sample	tissue	sex	time
sample_1	bone	M	1st
sample_2	bone	F	2nd
sample_3	bone	M	3rd
sample_4	bone	M	1st
sample_5	bone	F	2nd
sample_6	bone	M	3rd
sample_7	liver	F	1st
sample_8	liver	M	2nd
sample_9	liver	F	3rd
sample_10	liver	F	1st
sample_11	liver	M	2nd
sample_12	liver	F	3rd
```

## Optional inputs
The tool provides additional configuration through the following **optional** inputs:

1. **`qValue`**

	[Q-value](https://www.statisticshowto.com/q-value/) threshold to filter elements from the DESeq2 object (default: `0.05`).

2. **`log2FoldChange`**

	Minimum absolute log2 fold-change threshold to filter elements from the DESeq2 object (default: `0`).

3. **`minElements`**

	Minimum number of elements to form a cluster (default: `15`).

4. **`groupVar`**   

	Group variable in the DESeq2 object. It will determine how samples are grouped (eg. male/female, control/mutant). This variable must exist in the DESeq2 object. 

	If the `designFile` shown below was used to run a differential expression analysis with DESeq2. The variables `tissue` or `sex` could then be used as `groupVar` when running `degPatterns`.

```console
sample	tissue	sex	time
sample_1	bone	M	1st
sample_2	bone	F	2nd
sample_3	bone	M	3rd
sample_4	bone	M	1st
sample_5	bone	F	2nd
sample_6	bone	M	3rd
sample_7	liver	F	1st
sample_8	liver	M	2nd
sample_9	liver	F	3rd
sample_10	liver	F	1st
sample_11	liver	M	2nd
sample_12	liver	F	3rd
```

5. **`clusterMethod`**

	Clustering method to use, either [`diana`](https://www.rdocumentation.org/packages/cluster/versions/2.1.2/topics/diana) or [`ConsensusClusterPlus`](https://bioconductor.org/packages/release/bioc/html/ConsensusClusterPlus.html) funtion (default: `diana`).

6. **`removeOutliers`**

	Remove outliers from the cluster distribution (default: `true`).

7. **`scale`**

	Scale DESeq2 normalized count matrix (default: `true`).

8. **`plotsPerColumn`**

	Number of plots per column in the `summaryPlot` output file. Maximum number of plots allowed per column are `5` (default: `2`).

9. **`plotsPerRow`**
	
	Number of plots per row in the `summaryPlot` output file. Maximum number of plots allowed per row are `5` (default: `2`).
10. **`outputPrefix`**

	Prefix name for the output file (default: `deg-patterns`).

# Outputs
## Required outputs
This tool will **always** provide the following outputs:

1. **`degReport`**

	Tab delimited table describing the association between each element and the assigned cluster. 

| Column name         |	Definition                                                                                                                         |
|:------------------- |:---------------------------------------------------------------------------------------------------------------------------------- |
| [*element_ID*]        | Unique identifier for each element. Eg., geneID and	sampleID.                                                                      |
| merge               | Interaction specified in the designFormula while running DESeq2.                                                                   |
| value               | Z-score using the mean and the standard deviation from elements at the same merge level. This is the value displayed on the y-axis of the plots. |
| [*metadata_columns*]  | Columns representing the metadata associated with the particular element. Eg. for a gene it can be tissue, time, and sex.          |
| sizeFactor          | Geometric mean considering elements with the same `merge` value (ie., the `sizeFactor` will be the same for all elements at the same `explanatoryVar` and `groupVar` level). |
| cluster             | Cluster assigned to the particular element.                                                                                         |

2. **`elementClusterMap`**

	Tab delimited table describing what cluster has been assigned to each element. Two columns namely, `id`, which represents the element identification number and `cluster`, which representing the assigned cluster are present.

3. **`clusterCount`**

	Tab delimited table summarizing the number of elements per cluster. Two columns namely, `cluster`, which represents the particular cluster ID and `count`, which represents the number of times that cluster is repeated are present.

4. **`summaryPlot`**

	PDF file displaying plots of all the identified clusters. The x-axis represents the factors from the `explanatoryVar` input, while the y-axis represents the [Z-score](https://www.investopedia.com/terms/z/zscore.asp) ranges. Z-score represents the value's relationship with the mean and is measured in terms of standard deviation from the mean. For example, a zero value indicates that it is equal to the mean, while a value of positive or negative one indicates that it's one standard deviation above or below the mean respectively. An example of the plot is shown below:

   ![*degPatterns plot*.](example1.png)

5. **`clusterPlots`**

	Compressed `.tar.gz` file with individual [Plotly](https://plotly.com/r/) cluster plots.    

# Example
## Get input data
This tool expects a DESeq2 object in RDS format. This can be generated by running the [matrix-to-deseq2](https://platform.batchx.io/batchx/tools/bioinformatics%2Fdeseq2%2Fmatrix-to-deseq2) tool found in the BatchX marketplace. The RDS object generated from [Example 4](https://platform.batchx.io/batchx/tools/bioinformatics%2Fdeseq2%2Fmatrix-to-deseq2/documentation#example-4) will be used here as input (i.e., `time_series.deseq2.RDS`).  

## Upload input data into BatchX
In case the RDS object is not on BatchX use the following command to upload this file to the BatchX file system:

```console
bx cp time_series.deseq2.RDS bx://test/degreports/
```

## Submit job
Submit a job to cluster genes expressing similarly across `time` and belonging to two different `tissue`.

```console
bx submit lpantano-team@degreport/deg-patterns:0.0.4 '{
    "deseq2Object": "bx://test/degreports/time_series.deseq2.RDS",
    "explanatoryVar": "time",
    "qValue": 0.5,
    "log2FoldChange": 0.1,
    "plotsPerColumn": 4,
    "plotsPerRow": 4,
    "removeOutliers": false,
    "groupVar": "tissue",
    "outputPrefix": "time-tissue",
    "minElements": 1
}'
```

The above job will create two clusters which upon visualization looks like the following:

![*degPatterns clusters for example run*.](example2.png)

# Links
- [DEGreport vignette](https://bioconductor.org/packages/release/bioc/vignettes/DEGreport/inst/doc/DEGreport.html#report-from-deseq2-analysis)
- [degPattern documentation](https://lpantano.github.io/DEGreport/reference/degPlotCluster.html)
- [degPatterns tutorial](https://hbctraining.github.io/DGE_workshop/lessons/08_DGE_LRT.html)
- [DESeq2 vignette](http://bioconductor.org/packages/devel/bioc/vignettes/DESeq2/inst/doc/DESeq2.html)

# Tool versions

* **DESeq2:** 1.36.0

* **DEGreport:** 1.32.0

* **dplyr:** 1.0.9

.correct_fdr <- function(tb, fdr){
    if (fdr == "lfdr-stat"){
        values <- tb[["stat"]]
        names(values) <- rownames(tb)
        values <- values[!is.na(values)]
        lfdr <- fdrtool(values, plot = F)[["lfdr"]]
        names(lfdr) = names(values)
        tb[names(lfdr), "padj"] <- lfdr
    }else if (fdr == "lfdr-pvalue"){
        values <- tb[["pvalue"]]
        names(values) <- rownames(tb)
        values <- values[!is.na(values)]
        lfdr <- fdrtool(values, plot = F)[["lfdr"]]
        names(lfdr) = names(values)
        tb[names(lfdr), "padj"] <- lfdr
    }else if (is.numeric(fdr)){
        stopifnot(length(fdr) == nrow(tb))
        tb[["padj"]] <- lfdr
    }else if (fdr !="default"){
        stop(fdr, "not supported.")
    }
    tb
}

.guessResults <- function(object, what, alpha, fdr="default"){
    coef <- match(what[[1]], resultsNames(object))
    if (is.na(coef) & length(what) == 1L)
        stop("coef ", what, " not found in resultsNames(dds).")
    if (length(what) == 1)
        res <- results(object, name = what)
    else res <- results(object, contrast = what)
    if (fdr != "default")
        res <- .correct_fdr(res, fdr)
    res[order(res[["padj"]]),]
}

.guessShrunken <- function(object, what, unShrunken, method){
    coef <- match(what[[1]], resultsNames(object))
    object <- object[rownames(unShrunken),]
    if (is.na(coef) & length(what) == 1L)
        stop("coef ", what, " not found in resultsNames().")
    if (length(what) > 1 & method == "apeglm"){
        warning("apeglm only works with coefficients, changing to normal.")
        method = "normal"
    }
    if (length(what) == 1)
        res <- lfcShrink(
            dds = object,
            coef = coef,
            res = unShrunken,
            type = method)
    else res <- lfcShrink(object, contrast = what,
                          res = unShrunken, type = method)
    # res <- .correct_fdr(res, fdr)
    res[order(res[["padj"]]),]
}

.createComb <- function(dds, combs){
    lapply(combs, function(comb){
        colData(dds)[[comb]] %>%
            unique %>%
            combn(., 2, simplify = FALSE) %>%
            lapply(., function(x) c(comb, x))
    }) %>% unlist(., recursive = FALSE)
}

.normalizeNames <- function(combs, dds) {
    lapply(combs, function(x) {
        if (grepl("^[0-9]+$", x))
            return(colnames(colData(dds))[[as.numeric(x)]])
        else return(x)
    }) %>% unlist(., recursive = FALSE)
}

.guessComb <- function(dds, combs, contrast, pairs = FALSE){
    all_combs <- list()
    if (!is.null(combs)){
        if (length(combs) > 1)
            combs <- .normalizeNames(combs, dds)
        if (sum(combs %in% names(colData(dds))) > 0 & !pairs)
            all_combs <- lapply(combs, function(comb)
                resultsNames(dds) %>%
                .[grepl(comb, resultsNames(dds))]) %>%
                unlist(., recursive = FALSE)
        if (sum(combs %in% names(colData(dds))) > 0 & pairs)
            all_combs <- .createComb(dds, combs)
        if (sum(combs %in% resultsNames(dds)) > 1)
            all_combs <- c(all_combs, combs)
    }

    if (!is.null(contrast))
        all_combs <- c(all_combs, contrast)
    all_combs <- unique(all_combs)
    contrast_string <- lapply(all_combs,
                              function(x) {
                                  if (length(x) == 3)
                                      return(paste0(x[1L], "_",
                                                    paste(x[2L:3L],
                                                          collapse = "_vs_")))
                                  paste(x, collapse = ":")
                              })
    names(all_combs) <- contrast_string
    all_combs
}

#' Automatize the use of `results()` for multiple comparisons
#'
#' This function will extract the output of [DESeq2::results()]
#' and [DESeq2::lfcShrink()] for multiple comparison using:
#'
#' * coefficients
#' * contrast
#' * Multiple columns in `colData` that match coefficients
#' * Multiple columns in `colData` to create all possible
#' contrasts
#' @param dds [DESeq2::DESeqDataSet] obcject.
#' @param combs Optional vector indicating the coefficients or columns
#'   fom `colData(dds)` to create group comparisons.
#' @param contrast Optional vector to specify contrast. See [DESeq2::results()].
#' @param alpha Numeric value used in independent filtering in [DESeq2::results()].
#' @param skip Boolean to indicate whether skip shrinkage. For instance
#'   when it comes from LRT method.
#' @param type Type of shrinkage estimator. See [DESeq2::lfcShrink()].
#' @param pairs Boolean to indicate whether create all comparisons or only
#'   use the coefficient already created from `DESeq2::resultsNames()`.
#' @param fdr type of fdr correction. `default` is FDR value,
#'  `lfdr-stat` is
#'  for local FDR using the statistics of the test, 
#'  `lfdr-pvalue` is for local FDR using the p-value of the test. fdrtools
#'  needs to be installed and loaded by the user
#'   
#' @author Lorena Pantano
#'
#' @return [DEGSet] with unSrunken and Srunken results.
#' @examples
#' library(DESeq2)
#' dds <- makeExampleDESeqDataSet(betaSD=1)
#' colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
#'   design(dds) <-  ~ condition + treatment
#' dds <- DESeq(dds)
#' res <- degComps(dds, combs = c("condition", 2),
#'                 contrast = list("treatment_B_vs_A", c("condition", "A", "B")))
#' # library(fdrtools)
#' #res <- degComps(dds,contrast = list("treatment_B_vs_A"),
#' #                fdr="lfdr-stat")
#' @export
degComps <- function(dds, combs = NULL, contrast = NULL,
                     alpha = 0.05, skip = FALSE,
                     type = "normal",
                     pairs = FALSE,
                     fdr = "default") {
    stopifnot(class(dds)[1] == "DESeqDataSet")
    stopifnot(fdr %in% c("default", "lfdr-stat", "lfdr-pvalue"))
    all_combs <- .guessComb(dds,
                            combs = combs, contrast = contrast,
                            pairs = pairs)
    message("Doing ", length(all_combs), " element(s).")

    message("Doing results() for each element.")
    resUnshrunken <- lapply(all_combs, function(x) .guessResults(dds, x, alpha, fdr))
    resShrunken <- list()
    default <- "raw"
    if (!skip){
        default <- "shrunken"
        
        message("Doing lcfSrink() for each element.")
        resShrunken <- lapply(names(all_combs), function(x){
            r <- .guessShrunken(dds,
                           all_combs[[x]],
                           resUnshrunken[[x]],
                           type)
            r$padj <- resUnshrunken[[x]]$padj
            r
            })
            
        
        names(resShrunken) <- names(all_combs)
    }
    attributes(default) <- list(default = default)
    rdsList <- lapply(names(all_combs), function(x)
        new("DEGSet", list(raw = resUnshrunken[[x]],
                           shrunken = resShrunken[[x]]),
            default = default)
        )
    names(rdsList) <- names(all_combs)
    if (length(rdsList) == 1){
        rdsList <- rdsList[[1]]
    }
    rdsList
}

.plot_shrunken <- function(res_all, res_all_subset){
    p <- ggplot(res_all, aes_string("baseMean_shrunken",
                                    "log2FoldChange_shrunken")) +
        geom_point(size = 0.8, color = "black") +
        geom_point(data = res_all[res_all[["sign"]],], 
                   aes_string("baseMean_shrunken",
                              "log2FoldChange_shrunken"),
                   color = "red", size = 0.9) + 
        scale_x_log10(
            breaks = trans_breaks("log10", function(x) 10L ^ x),
            labels = trans_format("log10", math_format(10L ^ .x))) + # nolint
        annotation_logticks(sides = "b") +
        xlab("mean expression across all samples") +
        ylab(expression(log[2]*" fold change")) + # nolint
        scale_color_manual(values = c("black","red", "green")) +
        guides(color = "none") + 
        geom_point(data = res_all_subset,
                   aes_string("baseMean_unshrunken",
                              "log2FoldChange_unshrunken"),
                   color = "blue") +
        geom_segment(data = res_all_subset,
                     aes_string(x = "baseMean_unshrunken",
                                xend = "baseMean_shrunken",
                                y = "log2FoldChange_unshrunken", 
                                yend = "log2FoldChange_shrunken"),
                     color = "blue",
                     arrow = arrow(length = unit(0.05, "inches"),
                                   type = "closed"))
    p
}

.plot_raw <- function(res_all){
    res_all[["sign"]] <- res_all[["padj"]] < 0.05 * 1
    res_all <- as.data.frame(res_all)
    ggplot(res_all, aes_string("baseMean",
                               "log2FoldChange")) +
        geom_point(size = 0.8, color = "black") +
        geom_point(data = res_all[res_all[["sign"]],], 
                   aes_string("baseMean",
                              "log2FoldChange"),
                   color = "red", size = 0.9) + 
        scale_x_log10(
            breaks = trans_breaks("log10", function(x) 10L ^ x),
            labels = trans_format("log10", math_format(10L ^ .x))) + # nolint
        annotation_logticks(sides = "b") +
        xlab("mean expression across all samples") +
        ylab(expression(log[2]*" fold change")) + # nolint
        scale_color_manual(values = c("black","red", "green")) +
        guides(color = "none")
}

.plot_correlation <- function(res_all){
    ggplot(res_all, aes_string("log2FoldChange_unshrunken",
                               "log2FoldChange_shrunken")) +
        geom_point(size = 0.8, color = "black") +
        geom_point(data = res_all[res_all[["sign"]],], 
                   aes_string("log2FoldChange_unshrunken",
                              "log2FoldChange_shrunken"),
                   color = "red", size = 0.9) + 
        xlab("fold change - unshrunken") +
        ylab(expression(log[2]*" fold change - shrunken")) + # nolint
        scale_color_manual(values = c("black","red", "green")) +
        guides(color = "none")
}

.merge_results <- function(results, raw){
    res_unshrunken <- deg(results, "raw")
    res_shrunken <- deg(results, "shrunken")
    base_mean <- "baseMean_shrunken"
    if (raw)
        base_mean <- "baseMean_unshrunken"
    log2fc <- "log2FoldChange_shrunken"
    if (raw)
        log2fc <- "log2FoldChange_unshrunken"
    res_all <- rownames_to_column(as.data.frame(res_unshrunken),
                                  var = "ID") %>%
        inner_join(rownames_to_column(as.data.frame(res_shrunken),
                                      var = "ID"), 
                   by = "ID", suffix = c("_unshrunken","_shrunken"))
    
    res_all[["sign"]] <- res_all[["padj_shrunken"]] < 0.05 * 1
    res_all <- res_all %>% filter(!is.na(.[["padj_shrunken"]]))
    res_all
}

#' MA-plot from base means and log fold changes
#'
#' MA-plot addaptation to show the shrinking effect.
#'
#' @author Victor Barrera
#' @author Rory Kirchner
#' @author Lorena Pantano
#' 
#' @param results [DEGSet] class.
#' @param title *Optional*. Plot title.
#' @param label_points Optionally label these particular points.
#' @param label_column Match label_points to this column in the results.
#' @param limit Absolute maximum to plot on the log2FoldChange.
#' @param diff Minimum difference between logFoldChange before and
#'   after shrinking.
#' @param raw Whether to plot just the unshrunken log2FC.
#' @param correlation Whether to plot the correlation of the two logFCs.
#' 
#' @docType methods
#' @rdname degMA
#' @name degMA
#' 
#' @return MA-plot [ggplot].
#' @examples 
#' library(DESeq2)
#' dds <- makeExampleDESeqDataSet(betaSD=1)
#' dds <- DESeq(dds)
#' res <- degComps(dds, contrast = list("condition_B_vs_A"))
#' degMA(res)
#' @export
degMA <- function(results, 
                    title = NULL,
                    label_points = NULL,
                    label_column = "symbol",
                    limit = NULL,
                    diff = 5,
                    raw = FALSE,
                    correlation = FALSE) {
    stopifnot(class(results)[1] == "DEGSet")
    if (raw & correlation)
        stop("Use one or another. Incompatible parameters.")
    if (length(names(results)) == 1)
        raw = TRUE
    if (raw){
        p <- .plot_raw(deg(results))
    }else{
        res_all <- .merge_results(results, raw)    
        toplot <- (abs(res_all[["log2FoldChange_shrunken"]] - res_all[["log2FoldChange_unshrunken"]])) >= diff
        if (!is.null(limit)){
            res_all[["log2FoldChange_shrunken"]][res_all[["log2FoldChange_shrunken"]] < -1 * limit] <-  -1 * limit
            res_all[["log2FoldChange_shrunken"]][res_all[["log2FoldChange_shrunken"]] > 1 * limit] <-  1 * limit
            res_all[["log2FoldChange_unshrunken"]][res_all[["log2FoldChange_unshrunken"]] < -1 * limit] <- -1 * limit
            res_all[["log2FoldChange_unshrunken"]][res_all[["log2FoldChange_unshrunken"]] > 1 * limit] <- 1 * limit
        }
        
        res_all_subset <- res_all[toplot,]
        
        if (correlation)
            p <- .plot_correlation(res_all)
        else p <- .plot_shrunken(res_all, res_all_subset)
    }
    if (!is.null(title)) {
        p <- p + ggtitle(title)
    }
    
    if (!is.null(label_points)) {
        labels <- res_all[res_all[[label_column]] %in% label_points, ]
        p <- p + geom_text(data = labels,
                           aes_string(base_mean,
                                      log2fc,
                                      label = "label_column"), size = 3L)
    }
    p
}


# DESeq2 summary helper ====
.summary <- function(object, contrast, alpha){
    fn <- summary
    if (compareVersion("7.0", R.Version()$minor) < 0)
        fn <- DESeq2::summary
    if (compareVersion("1.25", as.character(packageVersion("DESeq2"))) < 0)
        fn <- DESeq2::summary
    if (class(object)[1] == "DESeqDataSet"){
        if (is.null(contrast)) {
            contrast <- resultsNames(object)[[2L]]
        }
        return(capture.output(fn(.guessResults(object, contrast),
                                      alpha)))
    }
    if (class(object)[1] == "DESeqResults")
        return(capture.output(fn(object, alpha)))
    if (class(object)[1] == "DEGSet")
        return(capture.output(fn(deg(object), alpha)))
    stop("No class supported.")
}

#' Print Summary Statistics of Alpha Level Cutoffs
#'
#' @rdname degSummary
#' @name degSummary
#' 
#' @param object Can be [DEGSet] or [DESeqDataSet] or [DESeqResults].
#' @param alpha Numeric vector of desired alpha cutoffs.
#' @param contrast Character vector to use with [results()] function.
#' @param caption Character vector to add as caption to the table.
#' @param kable Whether return a [knitr::kable()] output. Default is data.frame.
#' @return [data.frame] or [knitr::kable()].
#' 
#' @author Lorena Pantano
#' @references 
#' * original idea of multiple alpha values and code syntax
#'   from  Michael Steinbaugh.
#' @examples
#' library(DESeq2)
#' data(humanGender)
#' idx <- c(1:5, 75:80)
#' counts <- assays(humanGender)[[1]]
#' dse <- DESeqDataSetFromMatrix(counts[1:1000, idx],
#'                               colData(humanGender)[idx,],
#'                               design = ~group)
#' dse <- DESeq(dse)
#' res1 <- results(dse)
#' res2 <- degComps(dse, contrast = c("group_Male_vs_Female"))
#' degSummary(dse, contrast = "group_Male_vs_Female")
#' degSummary(res1)
#' degSummary(res1, kable = TRUE)
#' degSummary(res2[[1]])
#' @export
degSummary <- function(
    object,
    alpha = c(0.1, 0.05, 0.01),
    contrast = NULL,
    caption = "",
    kable = FALSE) {

    if (!is.null(contrast)) {
        caption <- contrast
    }
    df <- lapply(seq_along(alpha), function(a) {
        info <- .summary(object, contrast, alpha[a]) %>%
            # Get the lines of interest from summary
            .[4L:8L] %>% 
            .[.[] != ""]
        parse <- info[1L:5L] %>%
            # Extract the values after the colon in summary
            sapply(function(a) {
                gsub("^.+\\:\\s(.+)\\s$", "\\1", a)
            }) %>%
            # Coerce to character here to remove names
            as.character
        data.frame(alpha = parse)
    }) %>%
        bind_cols %>%
        set_colnames(alpha) %>%
        set_rownames(c("LFC > 0 (up)",
                       "LFC < 0 (down)",
                       "outliers",
                       "low counts",
                       "cutoff")) 
    
    if (kable) 
        df %>%
        kable(caption = paste(caption)) %>%
        show
    else return(df)
}

context("results")
library(DESeq2)

dds <- makeExampleDESeqDataSet(betaSD=1)
colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
design(dds) <-  ~ condition + treatment
dds <- DESeq(dds)
res <- results(dds)
resComps <- degComps(dds, contrast = c("condition_B_vs_A"))

test_that("Combinations", {
    # expect_named(.guessComb(dds,
    #                         combs = c("condition"),
    #                         contrast = list("treatment_B_vs_A", c("condition", "A", "B")),
    #                         pairs = TRUE),
    #              c("condition_1_vs_2",
    #                "treatment_B_vs_A",
    #                "condition_A_vs_B"))
    expect_type(.normalizeNames(c("condition", 2), dds), "character")
    expect_output(.createComb(dds, c("treatment", "condition")) %>% str,
                  "List of 2")
})

test_that("Results",{
    # expect_match(.guessResults(dds, c("condition", "A", "B"), 0.05) %>% class,
    #             "DESeqResults")
    # expect_match(.guessResults(dds, "condition_B_vs_A", 0.05) %>% class,
    #             "DESeqResults")
    expect_error(.guessResults(dds, "condition_C_vs_A", 0.05))
    # expect_match(.guessShrunken(dds, c("condition", "A", "B"), res, "normal") %>%
    #                  class,
    #              "DESeqResults")
    expect_error(.guessResults(dds, "condition_C_vs_A", res))
    expect_type(degSummary(dds, contrast = "condition_B_vs_A"), "list")
    expect_type(degSummary(res), "list")
    expect_type(degSummary(resComps), "list")
    expect_error(degSummary(colData(dds)))
    expect_output(degSummary(res, kable = TRUE), "|")
})

test_that("Correct", {
    res <- .guessResults(dds, c("condition", "A", "B"), 0.05, "default")
    #expect_s4_class(.correct_fdr(res, "lfdr-stat"), "DESeqResults")
    #expect_s4_class(.correct_fdr(res, "lfdr-pvalue"), "DESeqResults")
    expect_error(.correct_fdr(res, "other"))
    expect_equal(res, .correct_fdr(res, "default"))
})

context("Classes and Methods")
library(tibble)
library(DESeq2)

y <- matrix(rnbinom(80L, size = 1L, mu = 10L), nrow = 20L)
d <- DGEList(counts = y, group = rep(1L:2L, each = 2L),
             lib.size = rep(c(1000L:1001L), 2L))
rownames(d[["counts"]]) <- paste("gene", 1L:nrow(d[["counts"]]), sep=".")
d <- estimateCommonDisp(d)
de <- exactTest(d)
rawR <- topTags(de, n = Inf)

dds <- makeExampleDESeqDataSet(betaSD = 1)
colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
design(dds) <-  ~ condition + treatment
dds <- DESeq(dds)
raw <- results(dds)
res <- degComps(dds, combs = c("condition"))


test_that("DEGSet",{

    expect_type(as.DEGSet(raw, extras = list(shrunken = raw)), "list")
    expect_error(as.DEGSet(raw, extras = list(shrunken = rawR)),
                 "DESeqResults")
    
    expect_type(as.DEGSet(rawR,  extras = list(shrunken = rawR)), "list")
    expect_error(as.DEGSet(rawR,  extras = list(shrunken = raw)), "TopTags")
    
    expect_true(abs(deg(res, "raw")[["log2FoldChange"]][[1]]) >
                    abs(deg(res, "shrunken")[["log2FoldChange"]][[1]]))
    # expect_match(degDefault(res), "shrunken")
    expect_error(deg(res, "fake"))
    expect_identical(res[["shrunken"]], deg(res))
    expect_true(deg(res, tidy = "tibble") %>% is_tibble)
    expect_equal(deg(res, top = 5) %>% nrow, 5)
    
    expect_type(significants(res), "character")
    expect_type(significants(res[[1]]), "character")
    expect_type(significants(res), "character")
    expect_true(significants(res, full = TRUE) %>% is_tibble)
    
    # expect_s4_class(degCorrect(res, "lfdr-stat"), "DEGSet")
    
    expect_true(degMA(res, diff = 4) %>%
                    class %>% .[[2]] == "ggplot")
    expect_true(degMA(res, diff = 4, raw = TRUE) %>%
                    class %>% .[[2]] == "ggplot")
    expect_true(degMA(res, diff = 4, correlation = TRUE) %>%
                    class %>% .[[2]] == "ggplot")
    expect_error(degMA(res, diff = 4, raw = TRUE, correlation = TRUE))
})

#' Deprecated functions in package DEGreport
#' 
#' These functions are provided for compatibility with older versions
#' of DEGreport only and will be defunct at the next release.
#' 
#' @rdname DEGreport-deprecated
#' 
#' @details
#' The following functions are deprecated and will be made defunct; 
#' use the replacement indicated below:
#' * degRank, degPR, degBIcmd, degBI, degFC, degComb, degNcomb: DESeq2::lcfShrink. 
#' This function was trying to avoid big FoldChange
#' in variable genes. There are other methods nowadays like lcfShrink function.


#' DEGreport
#' 
#' @importFrom S4Vectors SimpleList DataFrame
#' @importMethodsFrom BiocGenerics plotMA
#' @importFrom "grDevices" "dev.off" "jpeg" "pdf" "rgb" "colorRampPalette"
#' @importFrom "methods" "slotNames"
#' @importFrom "stats" "as.dist" "as.hclust" "cor" "cor.test"
#'             "cutree" "density" "median.default" "quantile" "sd"
#'             "time" "var" "na.omit" "aov" "prcomp" "median"
#'             "cmdscale" "dist" "hclust" "p.adjust" "xtabs"
#'             "as.dendrogram"
#' @importFrom scales trans_breaks trans_format math_format
#' @importFrom ConsensusClusterPlus ConsensusClusterPlus calcICL
#' @import ggrepel
#' @import ggplot2
#' @importFrom ggplot2 layer ggproto
#' @importFrom ggdendro dendro_data
#' @import utils
#' @import edgeR
#' @import cluster
#' @import logging
#' @importFrom Biobase rowMax rowMin
#' @importFrom DESeq2 plotCounts rlog results resultsNames lfcShrink
#' @importFrom SummarizedExperiment SummarizedExperiment
#'             colData assay assays rowData
#' @importFrom psych corr.test
#' @importFrom grid textGrob
#' @importFrom broom tidy
#' @importFrom cowplot draw_plot
#' @importFrom cowplot ggdraw
#' @importFrom cowplot plot_grid
#' @importFrom dplyr arrange
#' @importFrom dplyr select select_if
#' @importFrom dplyr summarise
#' @importFrom dplyr mutate mutate_all mutate_if
#' @importFrom dplyr filter rename
#' @importFrom dplyr group_by ungroup
#' @importFrom dplyr "%>%"
#' @importFrom dplyr n
#' @importFrom dplyr distinct
#' @importFrom dplyr left_join inner_join right_join
#' @importFrom dplyr bind_rows
#' @importFrom dplyr bind_cols
#' @importFrom tidyr spread gather unite
#' @importFrom tibble column_to_rownames remove_rownames rownames_to_column
#' @importFrom tibble as_tibble
#' @importFrom ComplexHeatmap HeatmapAnnotation Heatmap
#' @importFrom circlize colorRamp2
#' @importFrom RColorBrewer brewer.pal
#' @importFrom reshape melt melt.data.frame
#' @importFrom knitr kable
#' @importFrom methods new slot
#' @importFrom rlang sym
#' @importFrom magrittr set_colnames set_rownames
#' @importFrom stringr str_split
"_PACKAGE"

library(getopt)
library(DESeq2)
library(DEGreport)
library(dplyr)
library(ggplot2)
library(ggpubr)
library(plotly)

# arguments
spec <- matrix(c(
    'deseq2Object', 'd', 1, "character", "DESeq2 object (required)",
    'qValue', 'q', 1, "numeric", "Q-value threshold to filter elements from the DESeq2 object (required)",
    'log2FoldChange', 'l', 1, "numeric", "Min absolute log2 fold-change threshold to filter elements from the DESeq2 object (required)",
    'minElements', 'm', 1, "numeric", "Min number of elements to form a cluster (required)",
    'explanatoryVar', 'e', 1, "character", "Explanatory variable in DESeq2 object (e.g. time). Will be used on the x-axis of the plot (required)",
    'groupVar', 'g', 1, "character", "Group variable in DESeq2 object (e.g. genotype). Determines how samples are grouped (optional)",
    'clusterMethod', 'c', 1, "character", "Cluster method. Either consensus or diana (required)",
    'removeOutliers', 'r', 1, "logical", "Remove outliers from the cluster distribution. (required)",
    'scale', 's', 1, "logical", "Scale DESeq2 normalized count matrix. (required)",
    'plotsPerColumn', 'n', 1, "numeric", "Number of plots per column in `summaryPlot´ file (required)",
    'plotsPerRow', 'w', 1, "numeric", "Number of plots per row in `summaryPlot´ file (required)",
    'degReport', 'o', 1, "character", "Output degReport table (required)",
    'elementClusterMap', 'u', 1, "character", "Output table with element - cluster relationship (required)",
    'clusterCount', 't', 1, "character", "Output table summarizing the number of elements per cluster (required)",
    'summaryPlot', 'p', 1, "character", "Output summary cluster plot (required)",
    'plotlyDirectory', 'y', 1, "character", "Output directory to store plotly cluster plots (required)",
    'help', 'h',0, "logical", "this help"),ncol=5,byrow=T)
opt = getopt(spec)

# Check parameters
if (!is.null(opt$help) || is.null(opt$deseq2Object) || is.null(opt$qValue) 
    || is.null(opt$log2FoldChange) || is.null(opt$minElements) || is.null(opt$explanatoryVar) 
    || is.null(opt$clusterMethod) || is.null(opt$removeOutliers) || is.null(opt$scale) || is.null(opt$plotsPerColumn) 
    || is.null(opt$plotsPerRow) || is.null(opt$degReport) || is.null(opt$elementClusterMap) || is.null(opt$clusterCount) 
    || is.null(opt$summaryPlot) || is.null(opt$plotlyDirectory)) {
    stop(cat(paste(getopt(spec, usage=T),"\n")));
}

# Libraries version
print(paste("DESeq2 version: ", package.version("DESeq2"), sep= ""))
print(paste("DEGreport version: ", package.version("DEGreport"), sep= ""))
print(paste("dplyr version: ", package.version("dplyr"), sep= ""))

# deseq2Object
print("Reading DESeq2 object...")
deseq2Object <- readRDS(file = opt$deseq2Object)

# cluster analysis
print("Extracting desing used to generate DESeq2 object...")
design <- as.data.frame(colData(deseq2Object))
print(design)

# check explanatory variable
print(paste("Checking explanatory variable '", opt$explanatoryVar, "' is present in the DESeq2 design...", sep=""))
if (!(opt$explanatoryVar %in% names(design))){
    stop(paste(opt$explanatoryVar, " is not present in DESeq2 object's design.", sep=""))
}

# relevel factor based on design
print("Releveling explanatory variable based on DESeq2 design...")
expVar <- dplyr::pull(design, opt$explanatoryVar)
expVar <- factor(expVar, unique(as.character(expVar)))
design[which(names(design) == opt$explanatoryVar)] <- expVar

# check group variable
if (!is.null(opt$groupVar)) {
    print(paste("Checking group variable '", opt$groupVar, "' is present in the DESeq2 design...", sep=""))
    if (!(opt$groupVar %in% names(design))){
        stop(paste(opt$design, " is not present in DESeq2 object's design.", sep=""))
    }
}

# results
print("Extracting results from DESeq2 object...")
res <- results(deseq2Object)
resDataFrame <- as.data.frame(res)
resDataFrame <- resDataFrame[complete.cases(resDataFrame),]
print(paste("Number of elements in DESeq2 object: ", nrow(resDataFrame), sep=""))

# log2 threshold
print(paste("Filtering elements in DESeq2 results table by q-value: ", opt$qValue, sep=""))
resDataFrame <- resDataFrame[(resDataFrame$padj < opt$qValue),]
print(paste("Filtering elements in DESeq2 results table by log2FoldChange: ", opt$log2FoldChange, sep=""))
resDataFrame <- resDataFrame[(abs(resDataFrame$log2FoldChange) > opt$log2FoldChange),]
print(paste("Number of elements in DESeq2 object after filtering: ", nrow(resDataFrame), sep=""))

if (nrow(resDataFrame) < opt$minElements) {
    stop(paste("Not enough elements to create clusters: ", nrow(resDataFrame), sep=""))
}

# rlog
print("Applying log2 transformation to count matrix ...")
ma <- assay(rlog(deseq2Object))[row.names(resDataFrame),]

# degPatterns
print("Running degPatterns...")
print(paste("Min number of elements to form a cluster: ", opt$minElements, sep=""))
print(paste("Remove outliers of the cluster distribution: ", opt$removeOutliers, sep=""))
print(paste("Scale normalized count matrix: ", opt$scale, sep=""))
if (opt$clusterMethod == "consensus") {
    print("Cluster Method: ConsensusClusterPlus")
    opt$clusterMethod <- TRUE
} else {
    print("Cluster Method: cluster::diana()")
    opt$clusterMethod <- FALSE
}
clusters <- degPatterns(ma, design, time = opt$explanatoryVar, consensusCluster = opt$clusterMethod, 
                            reduce = opt$removeOutliers, col = opt$groupVar, minc = opt$minElements, scale = opt$scale)

# degReport
print("Exporting degReport...")
degReport <- clusters$normalized
names(degReport) <- c("Id", names(degReport)[2:ncol(degReport)])
write.table(degReport, file = opt$degReport, sep = "\t", quote = FALSE, row.names = FALSE)

# elementClusterMap
print("Exporting element-cluster map table...")
clusteredElements <- paste(degReport$Id,degReport$cluster,sep = "_-_")
uniqueClusteredElements <- unique(clusteredElements)
uniqueClusteredElementsSplit <- strsplit(uniqueClusteredElements, "_-_")
element <- vector(mode = "character", length = length(uniqueClusteredElementsSplit))
clusterElement <- vector(mode = "character", length = length(uniqueClusteredElementsSplit))
for(elementIndex in 1:length(uniqueClusteredElementsSplit)) {
  element[elementIndex] <- uniqueClusteredElementsSplit[[elementIndex]][1]
  clusterElement[elementIndex] <- uniqueClusteredElementsSplit[[elementIndex]][2]
}
elementClusterMap <- data.frame(id = element, cluster = clusterElement)
write.table(elementClusterMap, file = opt$elementClusterMap, sep = "\t", quote = FALSE, row.names = FALSE)

# clusterCount
elementClusterMap$cluster <- as.character(elementClusterMap$cluster)
clusterCount <-  count(elementClusterMap, cluster)
names(clusterCount) <- c("cluster", "count")
write.table(clusterCount, file = opt$clusterCount, sep = "\t", quote = FALSE, row.names = FALSE)

# clusterPlot
print("Generating single cluster plots...")
numberOfCluster <- sort(unique(degReport$cluster))
clusterPlotList <- list()
plotCount <- 0
groupAndColorVariable <- opt$groupVar
if(is.null(opt$groupVar)) {
    groupAndColorVariable <- opt$explanatoryVar
}
print("Cluster identified:")
print(numberOfCluster)
for(clusterId in numberOfCluster) {
    plotCount <- plotCount + 1
    clusterSubset <- degReport %>% dplyr::filter(cluster == clusterId)
    clusterSubsetSummary <- clusterSubset %>% group_by_at(c(opt$explanatoryVar, opt$groupVar)) %>% summarize (mean = mean(value), sd = sd(value)) %>% as.data.frame()
    clusterPlot <- ggplot(data = clusterSubsetSummary, aes_string(x = opt$explanatoryVar, y = "mean", group = groupAndColorVariable, color= groupAndColorVariable)) +
                        geom_line(size=1) +
                        geom_errorbar(aes(ymin=mean-sd, ymax=mean+sd), width=.5, position=position_dodge(0.05)) +
                        geom_point() +
                        ggtitle(paste("Cluster ", clusterId, sep = "")) +
                        theme(legend.title = element_text(size = 7)) +
                        theme(axis.title.x = element_text(size = 7)) +
                        theme(axis.title.y = element_text(size = 7)) +
                        theme(legend.text = element_text(size = 5)) +
                        theme(axis.text = element_text(size = 5))
    clusterPlotList[[plotCount]] <- clusterPlot
    clusterPlotly <- ggplotly(clusterPlot) %>% config(displaylogo=FALSE,
                                            modeBarButtonsToRemove  = list("sendDataToCloud", "hoverClosest3d", 
                                                                            "autoScale2d", "hoverClosestCartesian", "hoverCompareCartesian", "zoom2d", 
                                                                            "pan2d", "select2d", "lasso2d", "zoomIn2d", "zoomOut2d", "toggleSpikelines"))
    samplePlotlyFile <- paste(opt$plotlyDirectory, "cluster-", clusterId, ".html", sep = "")                                                     
    htmlwidgets::saveWidget(clusterPlotly, samplePlotlyFile)
}
# summaryPlot
print("Generating cluster plots summary...")
pdf(onefile = TRUE,  file = opt$summaryPlot)
if (length(clusterPlotList) > 0) {
    names(clusterPlotList) <- seq_along(clusterPlotList)
    clusterPlotList[sapply(clusterPlotList, is.null)] <- NULL
    plotIndex <- 0
    pageCount <- 1
    plotCountLimit <- (opt$plotsPerColumn * opt$plotsPerRow)
    while((plotIndex + plotCountLimit) < length(clusterPlotList)) {
        print(paste("Summary plot page: ", pageCount, sep = ""))
        if (length(clusterPlotList) > plotIndex + plotCountLimit) {
            plotsInPageList <- clusterPlotList[(plotIndex + 1):(plotIndex + plotCountLimit)]
            summaryPlot <- ggarrange(plotlist = plotsInPageList, nrow = opt$plotsPerColumn, ncol = opt$plotsPerRow)
            print(summaryPlot)
        }
       plotIndex <- plotIndex + plotCountLimit
       pageCount <- pageCount + 1
    }
    if (plotIndex < length(clusterPlotList)) {
        print(paste("Summary plot page: ", pageCount, sep = ""))
        plotsInPageList <- clusterPlotList[(plotIndex + 1):length(clusterPlotList)]
        summaryPlot <- ggarrange(plotlist = plotsInPageList, nrow = opt$plotsPerColumn, ncol = opt$plotsPerRow)
        print(summaryPlot)
    }
}
dev.off()

#' @name humanGender
#' @title DGEList object for DE genes betwen Male and Females
#' @description DGEList object for DE genes betwen Male and Females
#' @docType data
#' @usage data(humanGender)
#' @format DGEList
#' @source gEUvadis
#' @author Lorena Pantano, 2017-08-37
NULL

library(ggplot2)
library(ggimage)

p <- ggplot(data.frame(x=1,y=1,image="inst/sticker/flag_hex.png"), aes(x,y)) +
    geom_image(aes(image=image), size=1.06) + theme_void()

hexSticker::sticker(p,
                    url = "www.bioconductor.org",
                    u_size = 5,
                    package = "DEGreport", 
                    s_x = 1.03,
                    s_y = 0.98, 
                    s_width = 2.05,
                    s_heigh = 2.05,
                    p_x = 1,
                    p_y = 1.17,
                    h_color = "white",
                    h_fill = "transparent",
                    h_size = 1,
                    p_color = "black",
                    p_size = 24,
                    filename="inst/sticker/degreport.png")

# Add function for PCA of discarded genes in the independing filtering by deseq2

#' Filter genes by group
#' 
#' This function will keep only rows that have a minimum counts of
#' 1 at least in a \code{min} number of samples (default 80%).
#' 
#' @param counts Matrix with expression data, columns are samples
#'   and rows are genes or other feature.
#' @param metadata Data.frame with information about
#'   each column in counts matrix. Rownames should match
#' \code{colnames(counts)}.
#' @param group Character column in metadata used to
#'   group samples and applied the cutoff.
#' @param min Percentage value indicating the minimum
#'   number of samples in each group that should have
#'   more than 0 in count matrix.
#' @param minreads Integer minimum number of reads to consider 
#'   a feature expressed.
#' @return count \code{matrix} after filtering genes (features)
#'   with not enough expression in any group.
#' @examples
#' data(humanGender)
#' library(SummarizedExperiment)
#' idx <- c(1:10, 75:85)
#' c <- degFilter(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], "group", min=1)
#' @export
degFilter <- function(counts, metadata, group, min=0.8, minreads=0){
    .unique_group <- as.character(unique(metadata[,group]))
    .keep = sapply(.unique_group, function(g){
        .samples = rownames(metadata)[metadata[, group] == g]
        .n = length(.samples)
        rowSums(counts[, .samples] > minreads) >= .n * min
    })
    counts[rowSums(.keep) > 0, ]
}
    

#' Plot main figures showing p-values distribution and mean-variance correlation
#' 
#' This function joins the output of \code{\link[DEGreport]{degMean}}, 
#' \code{\link[DEGreport]{degVar}} and \code{\link[DEGreport]{degMV}} in a
#' single plot. See these functions for further information.
#' 
#' @param counts Matrix with counts for each samples and each gene.
#' @param groups Character vector with group name for each sample in the
#'   same order than counts column names.
#' @param object [DEGSet] oobject.
#' @param pvalue pvalues of DEG analysis.
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dds <- DESeq(dds)
#' res <- results(dds)
#' degQC(counts(dds, normalized=TRUE), colData(dds)[["group"]],
#'   pvalue = res[["pvalue"]])
#' @export
degQC <- function(counts, groups, object=NULL, pvalue=NULL){
    if (is.null(pvalue) & is.null(object))
        stop("You need to provide DEGset object or pvalue.")
    if (!is.null(object)[1] & class(object)[1] != "DEGSet")
        stop("Object should be a DEGSet class.")
    stopifnot(class(counts)[1] %in% c("matrix", "data.frame"))
    
    if (class(object)[1] == "DEGSet"){
        df <- deg(object, tidy = "tibble")
        if (length(intersect(rownames(counts), df[["gene"]])) < nrow(df))
            stop("Not all features in DEGSet are in counts table.")
        counts <- counts[df[["gene"]],]
        pvalue <- df[["pvalue"]]
        padj <- df[["padj"]]
    }else{
        padj <- p.adjust(pvalue, method = "fdr")
    }

    counts <- counts[!is.na(padj),]
    pvalue <- pvalue[!is.na(padj)]
    padj <- padj[!is.na(padj)]
    pmean <- degMean(pvalue, counts) +
        xlab("pvalues along expression quantiles") +
        guides(fill = "none")
    pvar <- degVar(pvalue, counts) +
        xlab("pvalues along variance quantiles") +
        guides("") +
        theme(legend.position="top") +
        theme(legend.text=element_text(size=3),
              legend.title=element_blank(),
              legend.key.size = unit(0.5,"line"))
    pmv <- degMV(groups, padj, counts)
    suppressWarnings(ggdraw() + 
        draw_plot(pvar, 0, 0.4, 0.6, 0.6 ) +
        draw_plot(pmv, 0.6, 0, 0.4, 0.7) +
        draw_plot(pmean, 0, 0, 0.6, 0.4))
}

#' Distribution of gene ratios used to calculate Size Factors.
#' 
#' This function check the median ratio normalization used by
#' DESeq2 and similarly by edgeR to visualy check whether
#' the median is the best size factor to represent depth.
#' 
#' @aliases degCheckFactors
#' @param counts Matrix with counts for each samples and each gene.
#'   row number should be the same length than pvalues vector.
#' @param each Plot each sample separately.
#' @return ggplot2 object
#' @details This function will plot the gene ratios for each sample. To calculate
#' the ratios, it follows the simliar logic than DESeq2/edgeR uses, where the expression
#' of each gene is divided by the mean expression of that gene. The distribution
#' of the ratios should approximate to a normal shape and the factors should be similar
#' to the median of distributions. If some samples show different distribution,
#' the factor may be bias due to some biological or technical factor.
#' @references 
#' * Code to calculate size factors comes from
#'   [DESeq2::estimateSizeFactorsForMatrix()].
#' @examples
#' data(humanGender)
#' library(SummarizedExperiment)
#' degCheckFactors(assays(humanGender)[[1]][, 1:10])
#' @export
degCheckFactors <-
    function(counts, each = FALSE)
    {
        counts <- as.data.frame(counts)
        geoMeanNZ <- function(x) {
            if (all(x == 0)) { 0 } else { exp( sum(log(x[x > 0])) / length(x) ) }
        }
        geoMeans <- apply(counts, 1, geoMeanNZ)
        loggeomeans <- log(geoMeans)
        df <- lapply(colnames(counts), function(s) {
            cnts <- counts[[s]]
            r <- (log(cnts) - loggeomeans)[is.finite(loggeomeans) & cnts > 0]
            data.frame(ratios = r,
                       sample = s,
                       stringsAsFactors = FALSE)
        }) %>% bind_rows()
        p <- ggplot(df, aes(ratios, group = sample)) +
            geom_density() +
            theme_bw() +
            xlim(-4, 4)
        if (each)
            p <- p + facet_wrap(~sample)
        p
    }


#' Distribution of pvalues by expression range
#' 
#' This function plot the p-values distribution colored by
#' the quantiles of the average count data.
#' 
#' @param pvalues pvalues of DEG analysis.
#' @param counts  Matrix with counts for each samples and each gene.
#'   row number should be the same length than pvalues vector.
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dds <- DESeq(dds)
#' res <- results(dds)
#' degMean(res[, 4], counts(dds))
#' @export
degMean <-
    function(pvalues, counts)
{
    meanv  <-  apply(counts, 1, mean)
    q <- quantile(meanv, seq(.1, 1, .1))
    q <- q[!duplicated(q)]
    meanvfac <- cut(meanv,
                breaks=c(0, q),
                labels=names(q),
                right=TRUE)
    pvalfac  <-  cut(pvalues,
               breaks=c(-1, seq(.1, 1, .1)),
               labels=seq(0.1, 1, 0.1), right=TRUE)
    d  <-  data.frame(pvalues=factor(pvalfac),
                meanv=factor(meanvfac))
    suppressWarnings(ggplot(d, aes(pvalues, fill=meanv))+
        geom_bar()+
        theme_bw() +
        scale_fill_brewer("mean\nquantiles", palette="RdYlBu")) +
        labs(x="p-values", y="# genes")
}

#' Distribution of pvalues by standard desviation range
#' 
#' This function pot the p-valyes distribution colored by
#' the quantiles of the standard desviation of count data.
#' 
#' @param pvalues pvalues of DEG analysis
#' @param counts Matrix with counts for each samples and each gene.
#'   row number should be the same length than pvalues vector.
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dds <- DESeq(dds)
#' res <- results(dds)
#' degVar(res[, 4], counts(dds))
#' @export
degVar <-
    function(pvalues, counts)
{
    sdv <- apply(counts, 1, sd)
    q <- quantile(sdv, seq(.1, 1, .1))
    q <- q[!duplicated(q)]
    sdvfac <- cut(sdv,
                breaks=c(0, q),
                labels=names(q),
                right=TRUE)
    pvalfac <- cut(pvalues,
               breaks=c(-1, seq(.1, 1, .1)),
               labels=seq(0.1, 1, 0.1), right=TRUE)
    d <- data.frame(pvalues=factor(pvalfac), 
                sdv=factor(sdvfac))
    suppressWarnings(ggplot(d, aes(pvalues, fill=sdv))+
        geom_bar()+
        theme_bw() +
        scale_fill_brewer("variance\nquantiles", palette="RdYlBu") +
        labs(x="p-values", y="# genes"))
}

#' Correlation of the standard desviation and the mean of the abundance of a
#' set of genes.
#' 
#' @param group Character vector with group name for each sample in the
#'   same order than counts column names.
#' @param pvalues pvalues of DEG analysis.
#' @param counts Matrix with counts for each samples and each gene.
#' @param sign Defining the cutoff to label significant features.
#'   row number should be the same length than pvalues vector.
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dds <- DESeq(dds)
#' res <- results(dds)
#' degMV(colData(dds)[["group"]],
#'       res[, 4],
#'       counts(dds, normalized = TRUE))
#' @export
degMV <-
    function(group, pvalues, counts, sign=0.01)
{
    var_ma <- sapply(unique(as.character(group)), function(g){
        apply(counts[, group==g], 1, sd, na.rm=TRUE)
    })    
    sdv <- apply(var_ma, 1, max)
    mean_ma <- sapply(unique(as.character(group)), function(g){
        apply(counts[, group==g], 1, median, na.rm=TRUE)
    })    
    meanv <- apply(mean_ma, 1, min)
    pv <- cut(pvalues, breaks=c(-1, sign, 1.1),
          labels=c("Sign", "NoSig"))
    # pv[is.na(pvalues)] <- "NoSig"
    d <- data.frame(pvalues=pv, max_sd=log2(sdv), 
                min_median=log2(meanv))
    red <- d[d[["pvalues"]] == "Sign" & !is.na(d[["pvalues"]]),]
    black <- rgb(0.9, 0.9, 0.9, 0.6)
    suppressWarnings(ggplot(d, aes_string("min_median", "max_sd"))+
                         geom_point(data = d, color = black) +
                         geom_point(data = red, color = "red") +
                         scale_color_manual(values=c("red", black))+
                         theme_bw()+
                         stat_quantile(aes(min_median, max_sd), colour="blue",
                                       quantiles = c(0.025, 0.975), 
                                       linetype=2, formula=y ~ x)) +
        theme(legend.position="top")
    }

#' Distribution of expression of DE genes compared to the background
#'
#' @aliases degMB
#' @param tags  List of genes that are DE.
#' @param group Character vector with group name for each sample in the
#'   same order than counts column names.
#' @param counts  Matrix with counts for each samples and each gene
#'   Should be same length than pvalues vector.
#' @param pop number of random samples taken for background comparison
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dds <- DESeq(dds)
#' res <- results(dds)
#' degMB(row.names(res)[1:20], colData(dds)[["group"]],
#'   counts(dds, normalized = TRUE))
#' @export
degMB <-
    function(tags, group, counts, pop=400)
{
    delen <- length(tags)
    g <- ""

    # if counts is tiny, sample with replacement
    if(nrow(counts) < pop) {
        warning("The number of genes < samples requested, sampling with replacement.")
        replace = TRUE
    }
    else {
        replace = FALSE
    }
    rand <- sample(row.names(counts), pop, replace=replace)
    
    sign_group <- lapply(unique(as.character(group)), function(g){
        m <- apply(counts[tags, group==g, drop=FALSE], 1, mean, na.rm=TRUE)
        data.frame(mean = m, group = g,
                   type = paste("significants -", g),
                   stringsAsFactors = FALSE)
    })    
    
    rand_group <- lapply(unique(as.character(group)), function(g){
        m <- apply(counts[rand, group==g, drop=FALSE], 1, mean, na.rm=TRUE)
        data.frame(mean = m, group = g,
                   type = paste("background -", g),
                   stringsAsFactors = FALSE)
    }) 

    res <- bind_rows(c(sign_group, rand_group))

    suppressWarnings(ggplot(res, aes_string("type", "mean", 
        fill="group", colour="group"))+
        geom_violin(alpha=0.2)+
        scale_y_log10()+
        theme_bw())
}

#' Distribution of the standard desviation of
#'     DE genes compared to the background
#' @aliases degVB
#' @param tags List of genes that are DE.
#' @param group Character vector with group name for each sample in the
#'   same order than counts column names.
#' @param counts  matrix with counts for each samples and each gene.
#'     Should be same length than pvalues vector.
#' @param pop Number of random samples taken for background comparison.
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dds <- DESeq(dds)
#' res <- results(dds)
#' degVB(row.names(res)[1:20], colData(dds)[["group"]],
#'   counts(dds, normalized = TRUE))
#' @export
degVB <-
    function(tags, group, counts, pop=400)
{
    delen <- length(tags)
    g <- ""
    # if counts is tiny, sample with replacement
    if(nrow(counts) < pop) {
        warning("The number of genes < samples requested, sampling with replacement.")
        replace = TRUE
    }
    else {
        replace = FALSE
    }
    rand <- sample(row.names(counts), pop)
    sign_group <- lapply(unique(as.character(group)), function(g){
        m <- apply(counts[tags, group==g, drop=FALSE], 1, sd, na.rm=TRUE)
        data.frame(sd = m, group = g,
                   type = paste("significants -", g),
                   stringsAsFactors = FALSE)
    })    
    
    rand_group <- lapply(unique(as.character(group)), function(g){
        m <- apply(counts[rand, group==g, drop=FALSE], 1, sd, na.rm=TRUE)
        data.frame(sd = m, group = g,
                   type = paste("background -", g),
                   stringsAsFactors = FALSE)
    }) 
    
    res <- bind_rows(c(sign_group, rand_group))
    
    suppressWarnings(ggplot(res, aes_string("type", "sd", 
                                            fill="group", colour="group"))+
                         geom_violin(alpha=0.2)+
                         scale_y_log10()+
                         theme_bw())
    }

degNcomb <- function() {
    .Deprecated("DESeq2::lcfShrink")
}

degComb <- function()
{
    .Deprecated("DESeq2::lcfShrink")
}

degFC <- function()
{
    .Deprecated("DESeq2::lcfShrink")
}

degBI <- function()
{
    .Deprecated("DESeq2::lcfShrink")
}

degBIcmd <- function()
{
    .Deprecated("DESeq2::lcfShrink")
}

degRank <- function()
{
    .Deprecated("DESeq2::lcfShrink")
}


degPR <- function()
{
    .Deprecated("DESeq2::lcfShrink")
}

#' Create a deg object that can be used to plot expression values
#'     at shiny server:runGist(9930881)
#' @aliases degObj
#' @param counts Output from get_rank function.
#' @param design Colour used for each gene.
#' @param outfile File that will contain the object.
#' @return R object to be load into vizExp.
#' @examples 
#' data(humanGender)
#' library(SummarizedExperiment)
#' degObj(assays(humanGender)[[1]], colData(humanGender), NULL)
#' @export
degObj <-
    function(counts, design, outfile)
{
    deg <- NULL
    deg <- list(counts, design)
    if (!is.null(outfile))
        save(deg, file=outfile)
    if (is.null(outfile))
        message("please, give an output file name.")
}

# context("Plots")
# 
# data(humanGender)
# idx <- c(1:5, 75:80)
# counts <- assays(humanGender)[[1]]
# dse <- DESeqDataSetFromMatrix(counts[1:1000, idx],
#                               colData(humanGender)[idx,],
#                               design = ~group) %>% DESeq
# 
# res <- results(dse)
# 
# plot_cor <- function(dse){
#     ggplot(as.data.frame(counts(dse)),
#            aes(x = NA20502, y = NA20504)) +
#         geom_point() +
#         geom_cor(method = "kendall")
# }
# 
# # test_that("colors", {
# #     expect_equal(length(degColors(colData(dse))), 5)
# #     expect_equal(degColors(colData(dse), TRUE)[[2]] %>% class, "function")
# #     expect_equal(degColors(colData(dse), cat_values = c("red", "white"))[[1]],
# #                  c(Female = "red", Male = "white"))
# #     expect_error(degColors(data.frame(group = c("1", "2", "3")), palette = "Fake"))
# # })
# 
# test_that("degcovariates", {
#     resCov <- degCovariates(log2(counts(dse) + 0.5), colData(dse))
#     rows <- resCov[["corMatrix"]][resCov[["corMatrix"]][["fdr"]] < 0.05,"compare"]
#     rows <- rows[!is.na(rows)]
#     expect_true(length(rows) == 2)
#     expect_error(degCovariates(log2(counts(dse) + 0.5), colData(dse)[2:1,]))
# })
# 
# test_that("testGenes", {
#     expect_true(degPlotWide(dse, rownames(dse)[1:10], group = "group") %>%
#                     class %>% .[[2]] == "ggplot")
#     expect_true(degPlot(dse, res = res, n = 3, xs = "group", group = "group") %>%
#                     class %>% .[[2]] == "ggplot")
# })
# 
# test_that("singleFunctions", 
#     {
# 
#         expect_true(degMean(res[,4], counts(dse)) %>%
#                         class %>% .[[2]] == "ggplot")
#         expect_true(degVar(res[,4], counts(dse)) %>%
#                               class %>% .[[2]] == "ggplot")
#         expect_true(degMV(colData(dse)[["group"]], res[,4],
#                           counts(dse)) %>%
#                         class %>% .[[2]] == "ggplot")
#         detag <- row.names(res[1:10,])
#         expect_true(degMB(detag, colData(dse)[["group"]],
#                           counts(dse)) %>%
#                         class %>% .[[2]] == "ggplot")
#         expect_true(degVB(detag, colData(dse)[["group"]],
#                           counts(dse)) %>%
#                         class %>% .[[2]] == "ggplot")
#         expect_true(degVolcano(as.data.frame(res[,c("log2FoldChange", "pvalue")])) %>%
#                         class %>% .[[2]] == "ggplot")
#         expect_true(degPCA(counts(dse)) %>%
#                         class %>% .[[2]] == "ggplot")
#         expect_true(degMDS(counts(dse)) %>%
#                         class %>% .[[2]] == "ggplot")
#        
#         expect_true(plot_cor(dse) %>%
#                         class %>% .[[2]] == "ggplot")
#     })

# find inter class correlation between factor and continuous covariates
# inspired from http://stats.stackexchange.com/questions/108007/correlations-with-categorical-variables
.getfactorcontassociationstatistics <- function(factorcontnames,
                                                covariates,
                                                na.action='remove',
                                                alpha = 0.05){
    if (na.action == "remove")
        covariates = na.omit(covariates[, factorcontnames])

    covariates[,2] <- as.numeric(covariates[,2])
    data <- list(x = covariates[, 1L],
                y = covariates[, 2L])
    stats <- cor.test(scale(data[[1L]]),
                     scale(data[[2L]]),
                     method = "kendall", exact = FALSE)
    cor <- stats[["estimate"]]
    pval <- stats[["p.value"]]

    return(c(estimate = cor, pval = pval))
}

# function to run principal component analysis
.runpca <- function(genesbysamples, scale_data_for_pca = TRUE,
                    min_pve_pct_pc = 1.0) {

    # estimate variance in data by pc:
    pca.res <- prcomp(t(genesbysamples), center = TRUE,
                      scale. = scale_data_for_pca, retx = TRUE)

    # examine how much variance is explained by pcs,
    # and consider those with pve >= (min_pve_pct_pc %):
    pc.var <- pca.res$sdev^2L
    pve <- 100L * (pc.var / sum(pc.var))
    npca <- max(1L, length(which(pve >= min_pve_pct_pc)))

    samplepcvals <- pca.res$x[, 1L:npca, drop = FALSE]

    list(samplepcvals = samplepcvals, pve = pve)
}

# function to calculate correlation and plot
# cat vs continous
# ICCest(Chick, weight, data = ChickWeight, CI.type = "S")
# broom::tidy(aov(weight~Chick, data = ChickWeight))
# data("Arthritis") # cat vs cat
# tab <- xtabs(~Improved + Treatment, data = Arthritis)
# summary(assocstats(tab))
# 
# assocstats(UCBAdmissions)
.calccompletecorandplot <- function(compare_data, covar_data,
                                    correlationtype, title,
                                    weights = NULL,
                                    exclude_vars_from_fdr=NULL,
                                    max_fdr = 0.1) {
    # get factor and continuous covariates
    character_vars <- lapply(covar_data, class) == "character"
    if (sum(character_vars) > 0 )
        covar_data[, character_vars] <- apply(covar_data[, character_vars,
                                                         drop = FALSE],
                                              1L, as.factor)

    factorcovariates <- select_if(covar_data, is.factor) %>% colnames
    contcovariates <- select_if(covar_data, is.numeric) %>% colnames
    
    all_covariates <- cbind(covar_data[, contcovariates, drop = FALSE],
                            covar_data[, factorcovariates, drop = FALSE] %>%
                                mutate_all(as.numeric))
    
    cov_cor <- corr.test(compare_data,
                         all_covariates,
                         use = 'pairwise.complete.obs',
                         method = correlationtype,
                         adjust = "none")
    all_cor_vals <- cov_cor[["r"]]
    all_cor_p <- cov_cor[["p"]]
    
    rownames(all_cor_vals) <- colnames(compare_data)
    colnames(all_cor_vals) <- colnames(all_covariates)
    rownames(all_cor_p) <- colnames(compare_data)
    colnames(all_cor_p) <- colnames(all_covariates)
    
    effects.significantcovars <- all_cor_vals
    #effects.significantcovars[all_cor_p > max_fdr] <- 0
    effects.significantcovars <- colSums(abs(effects.significantcovars) * replicate(dim(effects.significantcovars)[2L], weights / sum(weights)))
    effects.significantcovars <- effects.significantcovars[order(abs(effects.significantcovars), decreasing = TRUE)]

    cor_mat <- melt(all_cor_p, varnames = c("compare", "covar"))
    colnames(cor_mat)[colnames(cor_mat) == "value"] <- "pvalue"

    cor_mat[["compare"]] <- factor(cor_mat[["compare"]],
                                   levels = rownames(all_cor_p))
    cor_mat[["covar"]] <- factor(cor_mat[["covar"]],
                                 levels = colnames(all_cor_p))

    # cor_mat[["r"]] <- melt(all_cor_vals)[["value"]]
    cor_mat[["r"]] <- as.vector(all_cor_vals)
    cor_mat[["fdr"]] <- p.adjust(cor_mat[["pvalue"]], method = "fdr")
    return(list(mat = cor_mat,
                effects.significantcovars = effects.significantcovars))
}

.generate_scatter_plot <- function(metadata, corMat){
    if (sum(corMat[["fdr"]] < 0.1, na.rm = T) == 0) return(NULL)
    plist <- apply(corMat[corMat[["fdr"]] < 0.1, ], 1, function(row){
        xs <- strsplit(row[1], " ")[[1]][1]
        ys <- row[2]
        ggplot(metadata, aes_string(x = xs, y = ys)) +
            geom_point() +
            ggtitle(paste(xs, ys))
    })
    names <- corMat[corMat[["fdr"]] < 0.1, ] %>%
        mutate(name = paste(compare, covar, sep = ":")) %>% 
        .[["name"]]
    names(plist) <- names
    plist
}

.numeric_effect_size <- function(covar_numeric, smart = TRUE){
    .smart <- function(v) {
        if (min(v) >= 0 & max(v)<=1)
            return(max(v)-min(v))
        if (min(v) >= 0 & max(v)<=100)
            return(max(v)-min(v))
        1 - min(v/max(v))
    }
    apply(covar_numeric,
          2, function(v){
              if (smart)
                  return(.smart(v))
              max(v)-min(v)
          }
    )
}

.effect_size <- function(ma, covar_numeric, covar_factors,
                         smart = TRUE){
    ma_sd <- data.frame(covar = colnames(covar_numeric),
                   effect_size = .numeric_effect_size(covar_numeric,
                                                      smart),
                   stringsAsFactors = FALSE)
    if (ncol(covar_factors) > 0){
        ma_sd <- bind_rows(ma_sd,
                           data.frame(covar = colnames(covar_factors),
                   effect_size = 1,
                   stringsAsFactors = FALSE)
        )
    }

    ma <- left_join(ma, ma_sd, by = "covar")
    ma[["effect_size"]][ma[["effect_size"]] < 0.01] <- 0.01
    ma[["effect_size"]][ma[["effect_size"]] > 1] <- 1
    ma[["type_variable"]] <- "categorical"
    ma[["type_variable"]][ma[["covar"]]  %in% colnames(covar_numeric)] <- "numeric"
    ma
}

.model <- function(data, method = "lm"){
    if (method == "lm"){
        pc_sig <- lm(PC~., data=data) %>% 
            broom::tidy()
    }
    # else if (method == "lasso"){
    #     pc_sig <- lm.lasso <- l1ce(PC ~ 0 + ., data=data, sweep.out = NULL) %>% 
    #         summary() %>% 
    #         .[["coefficients"]] %>% 
    #         broom::tidy()
    # }
    names(pc_sig) <- c("term", "estimate", "std.error", "statistic", "p.value")
    return(pc_sig)
}

# reduce covariates to significant ones that predict PCs
.reduce_covariates <- function(corMatrix, pcsMatrix, method = "lm"){
    pcs <- colnames(pcsMatrix)[grepl("PC[0-9]+", colnames(pcsMatrix))]
    significants <- lapply(pcs, function(pc){
        pc_var <- corMatrix %>% 
            filter(fdr < 0.01, grepl(pc, compare)) %>% 
            arrange(abs(r)) %>% 
            .[["covar"]] %>% 
            as.character()
        if (length(pc_var) == 0)
            return(NULL)
        data <- pcsMatrix[,c(pc, pc_var)]
        colnames(data)[1] = "PC"
        data[,2:ncol(data)] <- apply(data[,2:ncol(data), drop = FALSE], 2, scale)
        pc_sig <- .model(data, method)
        pc_sig[["PC"]] = pc
        pc_sig %>% filter(p.value < 0.05)
    }) %>% bind_rows()

    if (nrow(significants) == 0)
        return(data.frame(estimate=0, p.value=0, PC="1", term="noterm"))
    significants[,c("estimate", "p.value", "PC", "term")]
}

#' Find correlation between pcs and covariates
#'
#' This function will calculate the pcs using prcomp function,
#' and correlate categorical and numerical variables from
#' metadata. The size of the dots indicates the importance of the
#' metadata, for instance, when the range of the values is pretty
#' small (from 0.001 to 0.002 in ribosimal content),
#' the correlation results is not important. If black stroke lines
#' are shown, the correlation analysis has a FDR < 0.05 for that
#' variable and PC.
#' Only significant variables according the linear model are colored.
#' See details to know
#' how this is calculated. 
#' 
#' @author: Lorena Pantano, Victor Barrera, 
#'          Kenneth Daily and Thanneer Malai Perumal
#'
#' @param counts normalized counts matrix
#' @param metadata data.frame with samples metadata.
#' @param fdr numeric value to use as cutoff to determine
#'   the minimum fdr to consider significant correlations
#'   between pcs and covariates.
#' @param scale boolean to determine wether counts matrix should be
#'   scaled for pca. default FALSE.
#' @param minPC numeric value that will be used as cutoff
#'   to select only pcs that explain more variability than this.
#' @param correlation character determining the method for the
#'   correlation between pcs and covariates.
#' @param addCovDen boolean. Whether to add the covariates
#'   dendograme to the plot to see covariates relationship.
#'   It will show [degCorCov()] dendograme on top of the columns of
#'   the heatmap.
#' @param legacy boolean. Whether to plot the legacy version.
#' @param smart boolean. Whether to avoid normalization of the
#'   numeric covariates when calculating importance. This is not
#'   used if `legacy = TRUE`. See @details for more information.
#' @param method character. Whether to use `lm` to
#'   calculate the significance of the variable during reduction
#'   step. See @details for more information.
#' @param plot Whether to plot or not the correlation matrix.
#' @details This method is adapeted from Daily et al 2017 article.
#'   Principal components from PCA analysis are correlated with 
#'   covariates metadata. Factors are transformed to numeric variables.
#'   Correlation is measured by `cor.test` function with Kendall method
#'   by default.
#'   
#'   The size of the dot, or importance, indicates the importance of 
#'   the covariate based on the range of the values. Covariates
#'   where the range is very small (like a % of mapped reads that
#'   varies between 0.001 to 0.002) will have a very small size (0.1*max_size).
#'   The maximum value is set to 5 units.
#'   To get to importance, each covariate is normalized using this 
#'   equation: `1 - min(v/max(v))`,
#'   and the minimum and maximum values are set to
#'   0.01 and 1 respectively. For instance, 0.5 would mean there is at least
#'   50% of difference between the minimum value and the maximum value.
#'   Categorical variables are plot using the maximum size always, since
#'   it is not possible to estimate the variability. By default, it 
#'   won't do `v/max(v)` if the values are already between 0-1 or
#'   0-100 (already normalized values as rates and percentages).
#'   If you want to ignore the importance, use `legacy = TRUE`.
#'   
#'   Finally, a linear model is used to calculate the significance
#'   of the covariates effect on the PCs. For that, this function
#'   uses `lm` to regress the data and uses the p-value calculated by
#'   each variable in the model to define significance (pvalue < 0.05).
#'   Variables with a black stroke
#'   are significant after this step. Variables with grey stroke 
#'   are significant at the first pass considering p.value < 0.05
#'   for the correlation analysis.
#' @references 
#' Daily, K. et al.  Molecular, phenotypic, and sample-associated data to describe pluripotent stem cell lines and derivatives. Sci Data 4, 170030 (2017).
#'  
#' @return: list:
#' 
#' * plot, heatmap showing the signifcance of the variables. 
#' * corMatrix, correlation, p-value, FDR values
#'   for each covariate and PCA pais
#' * pcsMatrix: PCs loading for each sample
#' * scatterPlot: plot for each significant covariate and
#'   the PC values.
#' * significants: contains the significant covariates
#'   using a linear model to predict the coefficient
#'   of covariates that have some color in the plot.
#'   All the significant covariates from the liner model analysis
#'   are returned.
#' 
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' res <- degCovariates(log2(counts(dse)+0.5), colData(dse))
#' res <- degCovariates(log2(counts(dse)+0.5),
#'   colData(dse), legacy = TRUE)
#' res$plot
#' res$scatterPlot[[1]]
#' @export
degCovariates <- function(counts, metadata,
                          fdr = 0.1,
                          scale = FALSE,
                          minPC = 5.0,
                          correlation = "kendall",
                          addCovDen = TRUE,
                          legacy = FALSE,
                          smart = TRUE,
                          method = "lm",
                          plot = TRUE) {
    title <- paste(ifelse(scale, "s", "un-s"), "caled ",
                   " data in pca;\npve >= ",
                   minPC, "%;\n", correlation,
                   " cor ", sep = "")
    message(paste("\nrunning pca and calculating correlations for:\n",
                  title, sep = ""))
    metadata <- as.data.frame(metadata)
    
    metadata <- degClean(metadata) 
    
    covar_class <- sapply(metadata[1,], class)

    metadata <- metadata %>%
        mutate_all(as.numeric) %>% 
        as.data.frame() %>% 
        set_rownames(row.names(metadata))
    
    stopifnot(identical(colnames(counts), rownames(metadata)))
    
    pcares <- .runpca(genesbysamples = counts,
                      scale_data_for_pca = scale,
                      min_pve_pct_pc = minPC)
    
    samplepcvals <- pcares[["samplepcvals"]]
    pve <- pcares[["pve"]]
    original_names <- colnames(samplepcvals)
    pc_pct <- data.frame(
        pc = colnames(samplepcvals),
        pct = paste(" (",
                    sprintf("%.2f", pve[1L:ncol(samplepcvals)]), "%)",
                    sep = ""),
        stringsAsFactors = FALSE
        )

        # find covariates without any missing data
    samplesbyfullcovariates <- metadata[, which(apply(metadata, 2L,
                                                      function(dat) all(!is.na(dat)))), drop = FALSE]
    covar_class <- covar_class[colnames(samplesbyfullcovariates)]
    
    exclude_vars_from_fdr <- setdiff(colnames(metadata),
                                     colnames(samplesbyfullcovariates))
    
    covar_factors <- samplesbyfullcovariates[,names(covar_class)[covar_class != "numeric"], drop = FALSE]
    covar_numeric <- samplesbyfullcovariates[,names(covar_class)[covar_class == "numeric"], drop = FALSE]
    
    samplesbyfullcovariates = cbind(covar_factors, covar_numeric)
    
    corrRes <- .calccompletecorandplot(samplepcvals,
                                       samplesbyfullcovariates,
                                       correlation,
                                       title,
                                       weights =
                                           pve[1L:dim(samplepcvals)[2L]],
                                       exclude_vars_from_fdr)
    
    ma <- corrRes[["mat"]]
    ma[["r"]][ma[["fdr"]] > fdr] <- NA
    ma[["fdr"]][ma[["fdr"]] > fdr] <- NA
    corMa <- ma[, c("r", "compare", "covar")] %>%
        spread(!!sym("compare"), !!sym("r")) %>%
        remove_rownames() %>%
        column_to_rownames("covar")
    
    cor_meta <- .calccompletecorandplot(samplesbyfullcovariates,
                                        samplesbyfullcovariates,
                                        correlation,
                                        "",
                                        weights = 1L)
    
    corMeta <- cor_meta[["mat"]][, c("r", "compare", "covar")] %>%
        spread(!!sym("compare"), !!sym("r"), fill = 0) %>%
        remove_rownames() %>%
        column_to_rownames("covar") 
    
    hc <-  hclust(as.dist((1-corMeta)^2),
                method = "ward.D")
    
    ma[["covar"]] = as.character(ma[["covar"]])
    ma[["compare"]] = as.character(ma[["compare"]])

    ma <- .effect_size(ma, covar_numeric, covar_factors)
    
    samplepcvals <- as.data.frame(samplepcvals) %>% 
        set_colnames(original_names) %>% 
        rownames_to_column("samples")
    samplepcvals <- bind_cols(samplepcvals,
                              metadata)
    significants <- .reduce_covariates(ma, samplepcvals, method)
    scatterPlot <- .generate_scatter_plot(samplepcvals, corrRes[["mat"]])
    
    ma_plot <- left_join(ma,
                         significants,
              by = c("compare" = "PC", 
                     "covar" = "term")) %>% 
        left_join(pc_pct, by = c("compare" = "pc")) %>% 
        unite(col = "compare", !!sym("compare"), !!sym("pct"),
              sep = " ")
    ma_plot[["r"]][is.na(ma_plot[["p.value"]])] = NA
    if (legacy){
        if (addCovDen){
            p <- Heatmap(t(corMa),
                         name = "cor",
                         row_title = title,
                         cluster_rows = FALSE,
                         cluster_columns = hclust(as.dist((1-corMeta)^2),
                                                  method = "ward.D"),
                         col = colorRamp2(c(1, 0, -1),
                                          c("darkorange", "white", "darkblue")))
        }else{
            p <- ggplot(ma, aes_(fill = ~r,x = ~covar,y = ~compare)) +
                geom_tile() +
                theme_minimal() +
                ggtitle(title) +
                scale_fill_gradient2(low = "darkblue", high = "darkorange",
                                     guide = "colorbar", na.value = "grey90",
                                     limits = c(-1L, 1L)) +
                theme(axis.text.x = element_text(angle = 90L,
                                                 hjust = 1L,
                                                 vjust = 0.5))
            if (sum(ma[["r"]] > 0, na.rm = TRUE))
                p <- p + geom_text(data = ma[ma[["pvalue"]] < 0.05, ], aes(label="*"))
        }
        
    }else{
        
        dhc <- as.dendrogram(hc)
        # Rectangular lines
        ddata <- dendro_data(dhc, type = "rectangle")
        dn = ggplot(ddata[["segments"]]) + 
            geom_segment(aes(x = x, y = -y, xend = xend, yend = -yend)) + 
            scale_y_reverse(expand = c(0, 0)) +
            scale_x_continuous(expand=c(0.01, 0.01)) +
            xlab("") +
            ylab("") +
            theme_minimal() +
            theme(axis.text.x = element_blank(),
                  panel.grid.major = element_blank(),
                  panel.grid.minor = element_blank())
        
        
        ma_plot[["covar"]] <- factor(ma_plot[["covar"]],
                                levels = hc$labels[hc$order])

        tile <- ggplot(ma_plot, aes_(x = ~covar,y = ~compare,
                             size = ~effect_size,
                             color = ~r,
                             fill = ~r,
                             shape = ~type_variable)) +
            geom_point() +
            geom_point(data = filter(ma_plot, !is.na(fdr)),
                       stroke = 1, color="black") +
            geom_point(data = filter(ma_plot, pvalue<0.05),
                       stroke = 1, color="grey60") +
            theme_minimal() +
            scale_shape_manual(values = c(22, 21)) + 
            scale_size_continuous(name = "importance",
                                  limits = c(0.01, 1),
                                  range = c(0.01, 5)) + 
            scale_color_gradient2(low = "darkblue", high = "darkorange",
                                  guide = "colorbar", na.value = "grey90",
                                  limits = c(-1L, 1L)) +
            scale_fill_gradient2(low = "darkblue", high = "darkorange",
                                  guide = "colorbar", na.value = "grey90",
                                  limits = c(-1L, 1L)) +
            xlab("") +
            theme(axis.text.x = element_text(angle = 90L,
                                             hjust = 1L,
                                             vjust = 0.5),
                  legend.position = "bottom") +
            scale_x_discrete(expand=c(0.01,0.01))
        
        if (addCovDen){
            p <- plot_grid(
                dn, tile, align = "v",
                nrow = 2, rel_heights = c(1,4)
            ) + ggtitle(title)
        }else{
            p <- tile  + ggtitle(title)
        }
    }

    
    if (plot) print(p)

    invisible(list(
                plot = p,
                corMatrix = ma,
                pcsMatrix = samplepcvals,
                scatterPlot = scatterPlot,
                significants = significants
                ))
}

degClean <- function(ma){
    ma <- ma[,colSums(is.na(ma))<nrow(ma)]
    lapply(ma, function(x) {
        if (length(unique(x)) < length(x) * 0.20 & is.numeric(x))
            x <- as.factor(x)
        if ((class(x)[1] %in% c("factor", "character"))) {
            .f = as.factor(x)
            if (length(levels(.f)) < 2L)
                return(NULL)
            if (length(levels(.f)) > length(.f) * 0.80)
                return(NULL)
            return(.f)
        }
        if (sd(x, na.rm = TRUE) == 0)
            return(NULL)
        as.numeric(x)
    })  %>% Filter(Negate(is.null), .) %>% bind_cols %>%
        as.data.frame() %>%
        set_rownames(row.names(ma))
}

#' Calculate the correlation relationshipt among all covariates
#' in the metadata table
#'
#' This function will calculate the correlation among
#' all columns in the metadata
#'
#' @author: Lorena Pantano, Kenneth Daily and Thanneer Malai Perumal
#'
#' @param metadata data.frame with samples metadata.
#' @param fdr numeric value to use as cutoff to determine
#'   the minimum fdr to consider significant correlations
#'   between pcs and covariates.
#' @param use_pval boolean to indicate to use p-value instead of FDR to
#'   hide non-significant correlation.
#' @param ... Parameters to pass to [ComplexHeatmap::Heatmap()].
#'
#' @return: list:
#' a) cor, data.frame with pair-wise correlations, pvalues, FDR
#' b) corMat, data.frame with correlation matrix
#' c) fdrMat, data.frame with FDR matrix
#' b) plot, Heatmap plot of correlation matrix
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' cor <- degCorCov(colData(dse))
#' @export
degCorCov <- function(metadata, fdr=0.05, use_pval = FALSE, ...){
    clean <- degClean(metadata) %>%
        mutate_all(as.numeric)
    cor <- .calccompletecorandplot(clean,
                                   clean,
                                   "kendall",
                                   "",
                                   max_fdr = fdr,
                                   weights = 1L)
    #browser()
    if (use_pval){
        cor[["mat"]]['fdr'] <- cor[["mat"]]['pvalue']
    }
    corMat <- cor[["mat"]][, c("r", "compare", "covar")] %>%
        spread(!!sym("compare"), !!sym("r")) %>% remove_rownames() %>%
        column_to_rownames("covar")
    fdrMat <- cor[["mat"]][, c("fdr", "compare", "covar")] %>%
        spread(compare, fdr) %>% remove_rownames() %>%
        column_to_rownames("covar")

    corMat[fdrMat > fdr] <- 0
    # corMat[fdrMat > 0.05] <- NA
    if (sum(!is.na(corMat)) > 2) {
        p <- Heatmap(corMat, name = "cor-value",
                     col = colorRamp2(c(-1, 0, 1),
                                      c("steelblue", "white", "orange")),
                     row_dend_reorder = FALSE,
                     column_dend_reorder = FALSE, ...)
        print(p)
    }
    invisible(list(cor = cor[["mat"]], corMat = corMat,
                   fdrMat = fdrMat, plot = p))

}

options(repos = list(CRAN="http://cran.rstudio.com/"))
install.packages(c("getopt","ggpubr","plotly"))
if (!requireNamespace("BiocManager", dependecies=TRUE, quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("ComplexHeatmap", ask = FALSE)
if (!requireNamespace("devtools", dependecies=TRUE, quietly=TRUE))
  install.packages('devtools')
BiocManager::install("DEGreport", ask = FALSE)

# output plots as a way to avoid running the function again
# add plotGenes to documentation. Option to plot by names

.logger = function(toout,msg=""){
    logdebug(paste("\n\nchecking data:" , msg, "\n\n"))
    if (getLogger()[['level']]!=20)
        print(toout)
}

# It takes a table with gene and metadata information and convert
# it to be compatible with degPlotCluster
.process <- function(table, time, color){
    stopifnot(c("genes", "sample", time, "cluster", "expression")  %in% names(table))

        if (!is.null(color))
        stopifnot(color  %in% names(table))
    if (is.null(color)){
        grouped <- group_by(table, genes,
                            !!sym(time), cluster)
    }else{
        grouped <- group_by(table, genes,
                            !!sym(time), cluster, !!sym(color))
    }
    
    grouped %>% 
        summarise(expression=mean(expression)) %>% 
        group_by(!!sym("genes")) %>% 
        mutate(value = scale(expression)) %>% 
        ungroup()
}

#' Plot clusters from degPattern function output
#' 
#' This function helps to format the cluster plots from [degPatterns()].
#' It allows to control the layers and it returns a ggplot object that
#' can accept more ggplot functions to allow customization.
#' 
#' @param table `normalized` element from [degPatterns()] output.
#'   It can be a data.frame with the following columns in there:
#'   `genes, sample, expression, cluster, xaxis_column, color_column`.
#' @param time column name to use in the x-axis.
#' @param process whether to process the table if it is not
#'   ready for plotting.
#' @param color column name to use to color and divide the samples.
#' @param min_genes minimum number of genes to be added to the plot.
#' @param points Add points to the plot.
#' @param boxes Add boxplot to the plot.
#' @param smooth Add regression line to the plot.
#' @param lines Add gene lines to the plot.
#' @param facet Split figures based on cluster ID.
#' @param cluster_column column name if cluster is in a column
#'   with a different name. Usefull, to plot cluster with different
#'   cutoffs used when grouping genes from the clustering step.
#' @param prefix_title text to add before the cluster ID in the figure title.
#' @return [ggplot2] object.
#' @examples
#' data(humanGender)
#' library(SummarizedExperiment)
#' library(ggplot2)
#' ma <- assays(humanGender)[[1]][1:100,]
#' des <- colData(humanGender)
#' des[["other"]] <- sample(c("a", "b"), 85, replace = TRUE)
#' res <- degPatterns(ma, des, time="group", col = "other", plot = FALSE)
#' degPlotCluster(res$normalized, "group", "other")
#' degPlotCluster(res$normalized, "group", "other", lines = FALSE)
#' 
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#' table <- rownames_to_column(as.data.frame(ma), "genes") %>%
#'     gather("sample", "expression", -genes) %>%
#'     right_join(distinct(res$df[,c("genes", "cluster")]),
#'                by = "genes") %>%
#'     left_join(rownames_to_column(as.data.frame(des), "sample"),
#'               by = "sample") %>% 
#'               as.data.frame()
#' degPlotCluster(table, "group", "other", process = TRUE)
#' @export
degPlotCluster <- function(table, time, color = NULL,
                           min_genes = 10,
                           process = FALSE,
                           points = TRUE,
                           boxes = TRUE,
                           smooth = TRUE,
                           lines = TRUE,
                           facet = TRUE,
                           cluster_column = "cluster",
                           prefix_title = "Group: "){
    stopifnot(class(table)[1] == "data.frame")

    if (cluster_column  %in% colnames(table)){
        table[["cluster"]] = table[[cluster_column]]
    }
    if (process){
        table <- .process(table, time, color)
    }

    if ("cluster"  %in% colnames(table)){
        counts <- table(distinct(table, genes, cluster)[["cluster"]])
        counts <- counts[counts>=min_genes]
        if (length(counts)==0)
            stop("No clusters with min_genes > ", min_genes)
        table <- inner_join(table,
                            data.frame(cluster = as.integer(names(counts)),
                                       title = paste(prefix_title,
                                                      names(counts),
                                                      "- genes:" ,
                                                      counts),
                                       stringsAsFactors = FALSE),
                            by = "cluster")
    }

    if (is.null(color)){
        color = "dummy"
        table[[color]] = ""
        lines = FALSE
    }
    table[["line_group"]] = paste(table[["genes"]],
                                  table[[color]])
    
    
    splan <- length(unique(table[[time]])) - 1L
    table$title <- table$title %>% as.factor()
    old <- table$title %>% levels() %>% stringi::stri_extract_first(., regex="\\d+") %>% as.integer()
    index <- sort(old, index.return = TRUE)[[2]]
    table$title <- factor(table$title, levels = levels(table$title)[index])
    p <- ggplot(table, aes_string(x = time, y = "value",
                                  fill = color, color = color))
    
    if (boxes)
        p <- p + geom_boxplot(alpha = 0,
                              outlier.size = 0,
                              outlier.shape = NA, )
    if (points)
        p <- p + 
        geom_point(alpha = 0.4, size = 1,
                   position = position_jitterdodge(dodge.width = 0.9))
    if (smooth)
        p <- p + 
        stat_smooth(aes_string(x = time, y = "value",
                               group = color, color = color),
                    se = FALSE,
                    method = "lm", formula = y~poly(x, splan))
    if (lines)
        p <- p + geom_line(aes_string(group = "line_group"), alpha = 0.1)
    if (facet)
        p <- p + facet_wrap(~title)
    
    p <- p + 
        theme(axis.text.x = element_text(angle = 90, hjust = 1, vjust = 0.5)) +
        ylab("Z-score of gene abundance") +
        xlab("")
    p + theme_bw() 
    
}

.plot_benchmarking_curve <- function(benchmarking){
    if (is.null(benchmarking))
        return(NULL)
    df <- benchmarking[["genes"]]
    nc <- apply(df[2:ncol(df)], 2, function(c){
        length(unique(c))
    })
    ng <- apply(df[2:ncol(df)], 2, function(c){
        sum(!is.na(c))
    })
    data.frame(cutoff = names(nc),
               cluster = rev(nc),
               genes = rev(ng),
               pct_variance = unlist(benchmarking[["pcts"]])) %>% 
        ggplot(aes_string("cutoff", "pct_variance", group = 1)) +
        geom_line() +
        geom_text(aes_string(label="nc"), nudge_y = 1) +
        geom_text(aes_string(label="ng"), nudge_y = -1) +
        theme_minimal() +
        theme(axis.text.x = element_text(angle = 90, hjust=1))
}

.plot_benchmarking <- function(normalized, benchmarking,
                               time, color){

    if (is.null(benchmarking))
        return(NULL)
    benchmarking[["pcts"]] <- benchmarking[["pcts"]][lapply(benchmarking[["pcts"]],length)>0] 
    p <- lapply(names(benchmarking[["pcts"]]), function(serie){
        table <- normalized %>% group_by(!!sym(color),
                                         !!sym(time),
                                         !!sym(serie)) %>% 
            summarise(value = median(value))
        n = length(unique(table[[serie]]))
        p <- ggplot(table, aes_string(x = time, y = "value",
                                 color = color, group = serie)) +
            geom_line() +
            guides(color="none") +
            ggtitle(paste0(serie, ": ",
                          round(benchmarking[["pcts"]][[serie]]),
                          "% clusters:",
                          n)) +
            theme_minimal() +
            theme(title = element_text(size=6),
                  axis.title.x=element_blank(),
                  axis.text.x=element_blank(),
                  axis.ticks.x=element_blank(),
                  axis.title.y=element_blank(),
                  axis.text.y=element_blank(),
                  axis.ticks.y=element_blank())
        
        if (color == "colored"){
            p <- p + scale_color_manual(values = "black")
        }
        p
    }) %>% plot_grid(plotlist = .)
    p
}

# plot group of genes according time and group
.plot_cluster  <- function(norm_sign, g_in_c, xs , groups, title, fixy=NULL) {
    ma <- as.data.frame(norm_sign)[g_in_c,]
    ma_long <- suppressMessages(melt(cbind(genes = row.names(ma), ma),
                                     variable_name = "sample"))
    ma_long$x <- xs[ma_long$sample]
    ma_long$group <- groups[ma_long$sample]
    plotting_data <- ma_long
    p <- suppressWarnings(degPlotCluster(ma_long, "x", "group", facet = FALSE))
    p <- p +
        ggtitle(paste("Group:", title, "(", length(g_in_c), " genes )"))
    if (!is.null(fixy))
        p <- p + ylim(fixy[1], fixy[2])
    if (length(unique(groups)) == 1) {
        p <- p + scale_color_brewer(guide = "none", palette = "Set2") +
            scale_fill_brewer(guide = "none", palette = "Set2")
    }else{
        p <- p + scale_color_brewer(palette = "Set2") +
            scale_fill_brewer(palette = "Set2")
    }
    p
}

# Scale from 1 to 0 the expression genes
.scale <- function(e){
    scale(e)
}

.pct_var <- function(counts, clusters){
    clusters_sd <- lapply(unique(clusters), function(c){
        counts[names(clusters[clusters == c]),, drop = FALSE] %>% 
            apply(., 2, sd) %>% 
            median(.)
    }) %>% unlist() %>%  median
    (1 - clusters_sd / median(apply(counts, 2, sd))) * 100
}

.reduce <- function(groups, counts_group){
    lapply(unique(groups), function(g){
        ma <- counts_group[names(groups)[groups==g],]
        nokeep <- apply(ma, 2L, function(x){
            out <- boxplot(x, plot = FALSE)$out
            rownames(ma)[which(x %in% out)]
        }) %>% as.vector() %>% unlist() %>% unique()
        keep <- setdiff(rownames(ma), nokeep)
        group <- rep(g, length(keep))
        names(group) <- keep
        group
    }) %>%  unlist()
}

.select_pattern <- function(df){
    if(!("bycor" %in% colnames(df)))
        selected = df[df[["fdr"]] < 0.05, "gene"]
    selected = df[df[["cor"]] > 0.7, "gene"]
    selected = selected[!is.na(selected)]
    groups = rep(1, length(selected))
    names(groups) = selected
    groups
}

# find correlation to a pattern
.find_pattern <- function(counts_group, reference){
    cor_ma = apply(counts_group, 1, function(x){
        if(length(x) > 2){
            c = cor.test(x, reference, method = "kendall")
            return(data.frame(cor = c$estimate, pval=c$p.value))
        }else{
            c = cor(x, reference, method = "kendall")
            return(data.frame(cor = c[1], pval=0, bycor = 1))
        }
    }) %>%  bind_rows()
    cor_ma[["fdr"]] = p.adjust(cor_ma[["pval"]], "fdr") 
    cor_ma[["gene"]] = rownames(counts_group)
    cor_ma
}

# use diana package to detect clusters
.make_clusters <- function(counts_group){
    m <- (1 - cor(t(counts_group), method = "kendall"))
    d <- as.dist(m^2)
    c <- diana(d, diss = TRUE, stand = FALSE)
    c
}

.make_concensus_cluster <- function(counts_group, ...){
    ConsensusClusterPlus(t(counts_group),
                         reps = 500, maxK = 10,
                         distance = "pearson", finalLinkage = "ward.D2",
                         ...)
}

.select_concensus_genes <- function(c){
    icl <- calcICL(c)
    consensus <- icl[["itemConsensus"]] %>%
        group_by(!!!sym("k")) %>% 
        summarise(score = quantile(!!!sym("itemConsensus"),
                                   0.75,
                                   na.rm = TRUE)) %>% 
        arrange(desc(score), k) %>% .[["k"]] %>% .[1]
    
    df <- icl[["itemConsensus"]] %>%
        filter(k == consensus, itemConsensus > 0.9) %>% 
        .[,c("item", "cluster")]
    
    genes <- df[["cluster"]]
    names(genes) <- df[["item"]]
    
    genes
}

.select_genes <- function(c, counts_group, minc=15,
                          reduce=FALSE, cutoff=0.30, nClusters = NULL){
    if (is.null(nClusters)){
        h <- c$dc
        select <- cutree(as.hclust(c), h = h)
        select <- select[select %in% names(table(select))[table(select) > minc]]}
    else {
        select <- cutree(as.hclust(c), k = nClusters)
        select <- select[select %in% names(table(select))[table(select) > minc]]}
    
    if (reduce)
        select <- .reduce(select, counts_group)
        select <- select[select %in% names(table(select))[table(select) > minc]]
        message("Working with ", length(select), " genes after filtering: minc > ",minc)
    return(select)
}

.benckmark_cutoff <- function(tree, counts, minc = 15){
    series <- unique(round(tree$height, digits = 3))
    series <- sort(series[2:length(series)])
    list_clusters <- lapply(series, function(s) {
        select <- cutree(as.hclust(tree), h = s)
        select <- select[select %in% names(table(select))[table(select) > minc]]
    })
    list_pct <- lapply(list_clusters, function(c) {
        .pct_var(counts, c)
    })
    names(list_clusters) <- paste0("cutoff", series)
    names(list_pct) <- paste0("cutoff", series)
    df_clusters <- lapply(names(list_clusters), function(s){
        c = list_clusters[[s]]
        if (length(c) == 0) {return(data.frame())}
        data.frame(genes = make.names(names(c)),
                   cluster = c,
                   cutoff = s,
                   stringsAsFactors = F)
    }) %>% bind_rows() %>% 
        distinct() %>% 
        spread(cutoff, cluster)
    list(genes=df_clusters, pcts=list_pct)
}

# summarize matrix into groups and scale if needed
.summarize_scale <- function(ma, group, scale = TRUE){
    counts_group = t(sapply(rownames(ma), function(g){
        sapply(levels(group), function(i){
            idx = which(group == i)
            mean(ma[g, idx], na.rm = TRUE)
        })
    }))
    colnames(counts_group) = levels(group)
    .logger(head(counts_group), "summarize_scale::counts_group")
    .logger(head(group), "summarize_scale::group")
    if (scale) {
        norm_sign <- t(apply(counts_group, 1, .scale))
    }else{
        norm_sign <- counts_group
    }
    colnames(norm_sign) = colnames(counts_group)
    norm_sign
}

.median_per_cluster <- function(ma, clusters){
    # matrix, and df
    .logger(table(clusters[["cluster"]]),
            ".median_per_cluster::table=clusters")
    .logger(head(clusters), ".median_per_cluster::clusters")
    .t = do.call(rbind, lapply(unique(clusters$cluster), function(nc){
        .g = as.character(clusters$genes[clusters$cluster == nc])
        .e = apply(ma[.g, ], 2, median.default, na.rm = TRUE)
        .e
    }))
    rownames(.t) = unique(clusters$cluster)
    colnames(.t) = colnames(ma)
    .t
}

.filter <- function(df, co = 4){
    .sum <- table(df[["cluster"]])
    .pass <- names(.sum)[.sum > co]
    df[df[["cluster"]] %in% .pass,]
}

.get_features <- function(genes, norm, mapping){
    if (is.null(mapping))
        return(data.frame(touse = intersect(genes, rownames(norm)), pair = "."))
    df <- mapping[match(genes, mapping[, 1]),]
    names(df) = c("pair","touse")
    df
}

.plot_base <- function(genes, norm, raw, metadata, time, col, nc, name){
    .logger(length(intersect(genes, rownames(norm))), "num genes")
    .logger(head(norm), "Plot expression")
    .logger(head(metadata), "Plot expression")
    p <- .plot_cluster(norm, genes,
                       metadata[,time],
                       metadata[,col], nc) +
        ggtitle( paste("Pattern of the Genes in", name) )
    print(p)
    .logger(head(melt(as.data.frame(raw[genes,]))), "Plot expression")
    p <- ggplot(melt(as.data.frame(raw[genes,])),
               aes_string(x = "value", color = "variable")) + geom_density() +
        ggtitle( paste("Expression of the Genes in", name) )
    print(p)

}

.integrate <- function(nc1, .exp_base, .clus_base, .norm_base, .metadata_group,
                       .maraw_base, .ma, .norm, .metadata, time, col, summarize,
                       col_transformed, name, mapping=NULL){
    .genes_nc1 <- as.character(.clus_base$genes[.clus_base$cluster == nc1])
    keep_info <-  .get_features(.genes_nc1, .norm, mapping)
    keep <- as.character(unique(keep_info$touse))
    if (length(keep) < 5)
        return(NULL)
    clusters <- degPatterns(as.matrix(.ma[keep,]),
                           .metadata,
                           minc = 5, summarize = summarize,
                           time = time, col = col)
    if (length(clusters$pass) == 0)
        return(NULL)
    .exp <- .median_per_cluster(.norm,
                               clusters$df %>%
                               .[clusters$df$cluster %in% clusters$pass,])
    df <- lapply(rownames(.exp), function(nc2){
        .genes_nc2 = as.character(clusters$df$genes[clusters$df$cluster==nc2])
        .est = cor.test(.exp_base[nc1,], .exp[nc2,colnames(.exp_base)])
        data.frame(nc1=nc1,
                   nc2=paste0(nc1, ".", nc2),
                   cor=.est$estimate,
                   pval=.est$p.value,
                   gene=keep_info$touse[keep_info$touse %in% .genes_nc2],
                   pair=keep_info$pair[keep_info$touse %in% .genes_nc2])
    })
    .logger(head(df[[1]]), ".integrate::final_df")
    do.call(rbind, df)
}

.group_metadata <- function(metadata, time, col, summarize){
    if (is.null(col)) {
        col_transformed = "condition"
        metadata[,col_transformed] = rep("one_group", nrow(metadata))
    }
    if (!summarize %in% names(metadata))
        metadata[,summarize] = paste0(metadata[,col_transformed], metadata[,time])

    metadata_groups = metadata %>% dplyr::distinct(!!sym(summarize), .keep_all = TRUE)
    rownames(metadata_groups) = metadata_groups[,summarize]
    return(metadata_groups)
}


#' Integrate data coming from degPattern into one data object
#'
#' The simplest case is if you want to conbine the pattern profile
#' for gene expression data and proteomic data. It will use the first element
#' as the base for the integration. Then, it will loop through clusters
#' and run [degPatterns] in the second data set to detect patterns that match
#' this one.
#' @param matrix_list list expression data for each element
#' @param cluster_list list df item from degPattern output
#' @param metadata_list list data.frames from each element
#' with design experiment. Normally \code{colData} output
#' @param summarize character column to use to group samples
#' @param time character column to use as x-axes in figures
#' @param col character column to color samples in figures
#' @param scale boolean scale by row expression matrix
#' @param mapping data.frame mapping table in case elements use
#' different ID in the row.names of expression matrix. For instance,
#' when integrating miRNA/mRNA.
#' @return A data.frame with information on what genes are in each cluster in
#' all data set, and the correlation value for each pair cluster comparison.
degMerge <- function(matrix_list, cluster_list, metadata_list,
                     summarize="group", time="time", col="condition",
                     scale=TRUE, mapping=NULL){
    # basicConfig(level='FINEST')
    stopifnot(length(matrix_list) > 1 | length(metadata_list) > 1)
    stopifnot(length(matrix_list) == length(metadata_list))
    matrixnorm_list = list()
    cluster_expression = list()
    metadata_groups_list = list()
    matrixabs_list = list()
    for (name in names(matrix_list)) {
        .logger(name, msg = "Name list")
        metadata = metadata_list[[name]]
        matrixnorm_list[[name]] = .summarize_scale( as.matrix(matrix_list[[name]]),
                          group = metadata[, summarize],
                          scale = scale)
        .logger(head(matrix_list[[name]]), msg = "cluster Matrix")
        matrixabs_list[[name]] = .summarize_scale(as.matrix(matrix_list[[name]]),
                                                    group = metadata[,summarize],
                                                    scale = FALSE)
        .logger(head(matrixnorm_list[[name]]), msg = "Matrix norm DF")

        if (is.null(col))
            col_transformed = "condition"

        metadata_groups_list[[name]] = .group_metadata(metadata, time, col, summarize)
        .logger(metadata_groups_list[[name]], "Metadata")
        if (name %in% names(cluster_list)) {
            cluster = cluster_list[[name]]
            cluster_list[[name]] = .filter(cluster[as.character(cluster$genes) %in% rownames(matrix_list[[name]]), ])
            cluster_expression[[name]] = .median_per_cluster(
                matrixnorm_list[[name]],
                cluster_list[[name]])
        }
        .logger(head(cluster_expression[[name]]), "Cluster summarized")
    }

    base = names(matrix_list)[1]
    .norm_base = matrixnorm_list[[base]]
    .exp_base = cluster_expression[[base]]
    .clus_base = cluster_list[[base]]
    .metadata_group = metadata_groups_list[[base]]
    .maraw_base = matrixabs_list[[base]]
    cat("\n\n## Integration of :", paste(names(matrix_list)),"{.tabset}\n\n")
    df = lapply(rownames(.exp_base), function(nc1){
        .genes_nc1 = as.character(.clus_base$genes[.clus_base$cluster == nc1])
        cat("\n\n### Cluster number ", nc1, "\n\n")
        .plot_base(.genes_nc1, .norm_base, .maraw_base, .metadata_group,
                   time, col_transformed, nc1, paste(base, nc1))
        df = lapply(names(matrix_list), function(name) {
            if (name == base) return(NULL)
            .norm = matrixnorm_list[[name]]
            .ma =  matrix_list[[name]]
            .metadata = metadata_list[[name]]
            map = NULL
            if (name %in% names(mapping))
                map = mapping[[name]]
            .logger(head(map), "Mapping")
            .integrate(nc1, .exp_base, .clus_base, .norm_base, .metadata_group,
                       .maraw_base, .ma, .norm, .metadata,
                       time, col, summarize, col_transformed, name, map)
        })
        do.call(rbind, df)
    })
    df

}

.table_w_fc <- function(dds, contrast){
    if (!contrast[[1]][1] %in% names(colData(dds))) {
        stop("column not in dds object:", contrast[[1]][1])
    }
    fc_df <- do.call(rbind, lapply(contrast, function(cntr){
        tb <- results(dds, contrast = c(cntr[1], cntr[2], cntr[3]),
                      tidy = "TRUE")
        tb  %>% .[, c("row", "log2FoldChange")] %>%
            mutate(comp = paste0(cntr[2], "vs", cntr[3]))
    }))
    fc_df <- fc_df %>% tidyr::spread(comp, log2FoldChange)
    rownames(fc_df) <- fc_df$row
    fc_df[,2:ncol(fc_df)]
}

.run_cluster_profiler <- function(out_df, FDR, FC, org, minc=30){
    .res = as.data.frame(out_df)
    .idx = .res$padj < FDR & .res$absMaxLog2FC > FC
    .idx[is.na(.idx)] = FALSE
    cat("\n\n### GO ontology of DE genes (log2FC >" , FC, " and FDR <", FDR, "): ", sum(.idx),"\n\n")
    ego <- enrichGO(gene = row.names(out_df[.idx,]), keytype = "ENSEMBL",
                    OrgDb = org, ont = "BP", pAdjustMethod = "BH",
                    pvalueCutoff = 0.01, qvalueCutoff = 0.05, readable = TRUE)

    if ("result" %in%  slotNames(ego)) {
        print(knitr::kable(simplify(ego)@result[,1:7]))
        cat("\n\n")
        return(ego)
    }
    return(NULL)
}

.convertIDs <- function(ids, from, to, db, ifMultiple=c("putNA", "useFirst")) {
    stopifnot( inherits( db, "AnnotationDb" ) )
    ifMultiple <- match.arg( ifMultiple )
    if (sum(ids %in% keys(db, from))==0)
        return(ids)
    suppressMessages( selRes <- AnnotationDbi::select(
        db, keys = ids, keytype = from, columns = c(from,to) ) )
    if ( ifMultiple == "putNA" ) {
        duplicatedIds <- selRes[ duplicated( selRes[,from] ), from ]
        selRes <- selRes[ !(selRes[,from] %in% duplicatedIds), ]
    }
    return( selRes[ match( ids, selRes[,from] ), to ] )
}

.save_file <- function(dat, fn, basedir=".", csv=TRUE){
    if (is.null(basedir))
        return(invisible(NULL))
    tab <- cbind(id=data.frame(id=row.names(dat)), as.data.frame(dat))
    sep="\t"
    quote=FALSE
    if (csv){
        sep=","
        quote=TRUE
    }
    write.table(tab, file.path(basedir, fn), quote=quote, sep=sep, row.names=F)
}


.pca_loadings = function(counts, ntop=500) {
    pca <- prcomp((t(counts)))
    percentVar <- pca$sdev^2/sum(pca$sdev^2)
    names(percentVar) = colnames(pca$x)
    pca$percentVar = percentVar
    pca
}

#' Complete report from DESeq2 analysis
#'
#' @param res  output from [DESeq2::results()] function.
#' @param dds  [DESeq2::DESeqDataSet()] object.
#' @param rlogMat matrix from [DESeq2::rlog()] function.
#' @param name string to identify results
#' @param org an organism annotation object, like org.Mm.eg.db.
#' NULL if you want to skip this step.
#' @param FDR int cutoff for false discovery rate.
#' @param do_go boolean if GO enrichment is done.
#' @param FC int cutoff for log2 fold change.
#' @param group string column name in colData(dds) that
#' separates samples in meaninful groups.
#' @param xs string column name in colData(dss)
#' that will be used as X axes in plots (i.e time)
#' @param path_results character path where files are stored.
#' NULL if you don't want to save any file.
#' @param contrast list with character vector indicating the
#' fold change values from different comparisons to add to the output table.
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dse <- DESeq(dse)
#' res <- degResults(dds = dse, name = "test", org = NULL,
#'   do_go = FALSE, group = "group", xs = "group", path_results = NULL)
#' @export
degResults <- function(res=NULL, dds, rlogMat=NULL, name,
                                org=NULL, FDR=0.05, do_go=FALSE,
                                FC=0.1, group="condition", xs="time",
                                path_results =".",
                                contrast=NULL){
    if (is.null(rlogMat)){
        message("Doing rlog...")
        rlogMat <- assay(rlog(dds))
    }
    if (is.null(res)){
        message("Getting result...")
        res <- results(dds)
    }
    cat(paste("## Comparison: ", name, "{.tabset} \n\n"))
    metadata = as.data.frame(colData(dds))
    metadata = metadata[,!grepl("replaceable", names(metadata)),
                        drop=FALSE]
    out_df = as.data.frame(res)
    out_df = out_df[!is.na(out_df$padj),]
    out_df = out_df[order(out_df$padj),]
    if (! is.null(org)){
        out_df$symbol = .convertIDs(rownames(out_df),
                                    "ENSEMBL", "SYMBOL", org, "useFirst")
        out_df$description = .convertIDs(rownames(out_df),
                                         "ENSEMBL", "GENENAME", org, "useFirst")
    }

    cat("\n",paste(capture.output(summary(res)[1:8]), collapse = "<br>"),"\n")

    if (!is.null(contrast)){
        fc_df <- .table_w_fc(dds, contrast)
        cols_names <- names(fc_df)
        out_df <- cbind(out_df, fc_df[rownames(out_df),])
        out_df$absMaxLog2FC <- rowMax(abs(as.matrix(out_df[, cols_names])))
    }else{
        out_df$absMaxLog2FC <- abs(out_df$log2FoldChange)
    }

    fn = paste(name, "_de.csv", sep="")
    fn_log = paste(name, "_log2_counts.csv", sep="")
    .save_file(out_df, fn, path_results, csv=TRUE)
    .save_file(rlogMat, fn_log, path_results)
    cat("\n\nDifferential expression file at: ", fn)
    cat("\n\nNormalized counts matrix file at: ", fn_log)

    cat("\n\n### MA plot plot\n\n")
    DESeq2::plotMA(res)

    cat("\n\n### Volcano plot\n\n")
    stats = as.data.frame(res[,c(2,6)])
    degVolcano(stats, title=name, lfc.cutoff=1.5)

    cat("\n\n### QC for DE genes\n")
    show= !is.na(out_df$pvalue)
    p = degQC(rlogMat[row.names(out_df)[show],],
              metadata[,xs],
              pvalue = out_df[["pvalue"]][show])
    print(p)

    sign = row.names(out_df)[out_df$padj<FDR & !is.na(out_df$padj) & out_df$absMaxLog2FC > FC]
    cat("\n\n### Most significants, FDR<", FDR,
        " and log2FC > ", FC, ": ", length(sign),"\n")

    if (length(sign) < 2){
        cat("Too few genes to plot.")
    }else{
        Heatmap(rlogMat[sign, ], show_row_names = FALSE,
                top_annotation = HeatmapAnnotation(df = metadata),
                clustering_method_rows = "ward.D2",
                clustering_method_columns = "ward.D2",
                clustering_distance_columns = "correlation"
        )
        print(degMDS(rlogMat[sign,],condition = metadata[,xs])+
                  theme_minimal())
    }
    cat("\n")

    cat("\n\n### Plots top 9 most significants\n")
    degPlot(dds, out_df, xs = xs, group = group)
    cat("\n")

    cat("\n\n### Top DE table\n\n")
    print(kable(head(out_df, 20)))
    cat("\n\n")
    goterm = ""
    if (do_go){
        df <- .run_cluster_profiler(out_df, FDR, FC, org)
        if (!is.null(df)){
            .save_file(df@result, paste0(name, "_goenrich.csv"), path_results)
            goterm = df
        }


    }
    return(list(sign=sign, table=out_df, go_res=goterm))
}


#' smart PCA from count matrix data
#'
#' nice plot using ggplot2 from prcomp function
#'
#' @param counts matrix with count data
#' @param metadata dara.frame with sample information
#' @param pc1 character PC to plot on x-axis
#' @param pc2 character PC to plot on y-axis
#' @param condition character column in metadata to use to color samples
#' @param name character if given, column in metadata to print label
#' @param shape character if given, column in metadata to shape points
#' @param data Whether return PCA data or just plot the PCA.
#' @author Lorena Pantano, Rory Kirchner, Michael Steinbaugh
#' @return if `results <-` used, the function return the output
#'   of [prcomp()].
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#' colData(humanGender)[idx,], design=~group)
#' degPCA(log2(counts(dse)+0.5), colData(dse),
#'   condition="group", name="group", shape="group")
#' @export
degPCA <- function(counts, metadata = NULL, condition=NULL,
                   pc1 = "PC1", pc2 = "PC2",
                   name = NULL, shape = NULL,
                   data = FALSE){
    pc <- .pca_loadings(counts)
    # error if pc1/2 are not in columns of pc
    idx1 <- which(names(pc[["percentVar"]]) == pc1)
    idx2 <- which(names(pc[["percentVar"]]) == pc2)
    comps <- data.frame(pc[["x"]])
    comps[["Name"]] <- rownames(comps)
    
    if (!is.null(metadata))
        comps <- bind_cols(comps,
                           as.data.frame(metadata)[as.character(comps$Name), ,
                                                   drop=FALSE])
    # [Feature] check metadata has name, shape, condition
    
    if (!is.null(metadata))
        stopifnot(all.equal(colnames(counts), rownames(metadata)))
    
    p <- ggplot(comps, aes_string(pc1, pc2,
                                  color = condition,
                                  shape = shape)) +
        geom_point(size = 3)
    
    if (!is.null(condition)){
        if (is.factor(comps[[condition]]))
            p <- p + scale_color_brewer(palette = "Set2")
    }
    
    if (!is.null(name))
        p <- p + geom_text(aes_string(label = name),
                           nudge_x = 1, nudge_y = 1)

    p <- p +
        xlab(paste0(pc1, ": ",
                    round(pc[["percentVar"]][idx1] * 100),
                    "% variance")) +
        ylab(paste0(pc2, ": ",
                    round(pc[["percentVar"]][idx2] * 100),
                    "% variance"))
    
    if(data)
        return(list(pca = pc, plot = p))
    p
}

#' Plot MDS from normalized count data
#'
#' Uses cmdscale to get multidimensional scaling of data matrix,
#' and plot the samples with ggplot2.
#' @param counts matrix samples in columns, features in rows
#' @param condition vector define groups of samples in counts.
#' It has to be same order than the count matrix for columns.
#' @param k integer number of dimensions to get
#' @param d type of distance to use, c("euclidian", "cor").
#' @param xi number of component to plot in x-axis
#' @param yi number of component to plot in y-axis
#' @return ggplot2 object
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' degMDS(counts(dse), condition = colData(dse)[["group"]])
#' @export
degMDS = function(counts, condition=NULL, k=2, d="euclidian", xi=1, yi=2) {
    if (d == "euclidian"){
        distances = dist(t(counts))
    }else if (d == "cor"){
        distances = as.dist((1-cor(counts))^2)
    } else {
        stop(d, " is not implemented.")
    }
    fit = cmdscale(distances, eig = TRUE, k = k)
    eigs = data.frame(variance_explained = fit$eig / sum(fit$eig))
    xnames = paste0("PC", 1:k, " ", round(eigs[1:k, 1] * 100, digits = 2), "%")
    df = as.data.frame(fit$points[, c(xi,yi)])
    names(df) = c("one", "two")
    df[["label"]] = rownames(df)
    if(!is.null(condition)) {
        df[["condition"]] = condition
        p = ggplot(df, aes_string("one", "two",
                                  label = "label", color = "condition")) +
            geom_text(aes_string("one", "two",
                                 label = "label"), size = 3) +
            labs(x = xnames[xi], y = xnames[yi]) +
            scale_x_continuous(expand = c(0.3, 0.3))
    } else {
        p = ggplot(df, aes_string("one", "two")) +
            geom_text(aes_string("one", "two",
                                 label = "label"), size = 3) +
            labs(x = xnames[xi], y = xnames[yi]) +
            scale_x_continuous(expand = c(0.3, 0.3))

    }

    p
}

.remove_low_difference <- function(ma, groupDifference, each_step){
    if (!each_step){
        keep <- rowMax(ma) - rowMin(ma) > groupDifference
    } else {
        keep <- sapply(1:(ncol(ma) -1), function(i){
            abs(ma[,i] - ma[,i + 1]) > groupDifference
        }) %>% apply(., 1, function(x) all(x))
    }
    message("Filtering ", sum(!keep), " after groupDifference applied.")
    if (sum(keep) < 10)
        stop("After applying groupDifference: ", groupDifference,
             " number of features in matrix is less than 10.",
             " Reduce the value and try again.")
    ma[keep,]
}

#' Make groups of genes using expression profile.
#' 
#'  
#' Note that this function doesn't calculate significant
#' difference between groups, so the
#' matrix used as input should be already filtered to contain only
#' genes that are significantly different or the most interesting genes
#' to study.
#'
#' @aliases degPatterns
#' @param ma  log2 normalized count matrix
#' @param metadata  data frame with sample information. Rownames
#'   should match \code{ma} column names
#'   row number should be the same length than p-values vector.
#' @param minc integer minimum number of genes in a group that
#' will be return
#' @param summarize character column name in metadata that will be used to group
#'   replicates. If the column doesn't exist it'll merge the `time` and
#'   the `col` columns, if `col` doesn't exist it'll use `time` only.
#'   For instance, a merge between summarize and time parameters:
#'   control_point0 ... etc
#' @param time character column name in metadata that will be used as
#'   variable that changes, normally a time variable.
#' @param col character column name in metadata to separate
#'   samples. Normally control/mutant
#' @param consensusCluster Indicates whether using [ConsensusClusterPlus]
#'   or [cluster::diana()]
#' @param reduce boolean remove genes that are outliers of the cluster
#'   distribution. `boxplot` function is used to flag a gene in any
#'   group defined by `time` and `col` as outlier and it is removed
#'   from the cluster. Not used if `consensusCluster` is TRUE.
#' @param cutoff This is deprecated.
#' @param scale boolean scale the \code{ma} values by row
#' @param pattern numeric vector to be used to find patterns like this
#'   from the count matrix. As well, it can be a character indicating the
#'   genes inside the count matrix to be used as reference.
#' @param groupDifference Minimum abundance difference between the
#'   maximum value and minimum value for each feature. Please, 
#'   provide the value in the same range than the `ma` value
#'    ( if `ma` is in log2, `groupDifference` should be inside that range).
#' @param eachStep Whether apply `groupDifference` at each stem over
#'   `time` variable. **This only work properly for one group
#'    with multiple time points**.
#' @param plot boolean plot the clusters found
#' @param fixy vector integers used as ylim in plot
#' @param nClusters an integer scalar or vector with the desired number of groups
#' @param skipDendrogram a boolean to run or not dendextend. Temporary fix to memory
#'                  issue in linux.
#' @details 
#' It can work with one or more groups with 2 or
#' more several time points. 
#' Before calculating the genes similarity among samples,
#' all samples inside the same time point (`time` parameter) and
#' group (`col` parameter) are collapsed together, and the `mean`
#' value is the representation of the group for the gene abundance.
#' Then, all pair-wise gene expression is calculated using
#' `cor.test`  R function using kendall as the statistical
#' method. A distance matrix is created from those values.
#' After that, [cluster::diana()] is used for the 
#' clustering of gene-gene distance matrix and cut the tree using
#' the divisive coefficient of the clustering, giving as well by diana.
#' Alternatively, if `consensusCluster` is on, it would use
#' [ConsensusClusterPlus] to cut the tree in stable clusters.
#' Finally, for each group of genes, only the ones that have genes
#' higher than `minc` parameter will be added to the figure.
#' The y-axis in the figure is the results of applying `scale()` 
#' R function, what is similar to creating a 
#' `Z-score` where values are centered to the `mean`  and
#' scaled to the `standard desviation` by each gene.
#' 
#' The different patterns can be merged
#' to get similar ones into only one pattern. The expression
#' correlation of the patterns will be used to decide whether
#' some need to be merged or not.
#' @return list wiht two items:
#' *  `df` is a data.frame
#' with two columns. The first one with genes, the second
#' with the clusters they belong. 
#' * `pass` is a vector of the clusters that pass the `minc` cutoff.
#' * `plot` ggplot figure.
#' * `hr` clustering of the genes in hclust format.
#' * `profile` normalized count data used in the plot.
#' * `raw` data.frame with gene values summarized by biological replicates and
#'   with metadata information attached.
#' * `summarise` data.frame with clusters values summarized by group and
#'   with the metadata information attached.
#' * `normalized` data.frame with the clusters values 
#'   as used in the plot.
#' * `benchmarking` plot showing the different patterns at different
#'   values for clustering cuttree function.
#' * `benchmarking_curve` plot showing how the numbers of clusters and genes
#'   changed at different values for clustering cuttree function.
#' @examples
#' data(humanGender)
#' library(SummarizedExperiment)
#' library(ggplot2)
#' ma <- assays(humanGender)[[1]][1:100,]
#' des <- colData(humanGender)
#' des[["other"]] <- sample(c("a", "b"), 85, replace = TRUE)
#' res <- degPatterns(ma, des, time="group", col = "other")

#' # Use the data yourself for custom figures
#'  ggplot(res[["normalized"]],
#'         aes(group, value, color = other, fill = other)) +
#'   geom_boxplot() +
#'    geom_point(position = position_jitterdodge(dodge.width = 0.9)) +
#'    # change the method to make it smoother
#'    geom_smooth(aes(group=other), method = "lm")
#' @export
degPatterns = function(ma, metadata, minc=15, summarize="merge",
                       time="time", col=NULL,
                       consensusCluster = FALSE,
                       reduce=FALSE, cutoff=0.70,
                       scale=TRUE,
                       pattern = NULL,
                       groupDifference = NULL,
                       eachStep = FALSE,
                       plot=TRUE, fixy=NULL, nClusters = NULL,
                       skipDendrogram=TRUE){
    benchmarking <- NULL
    metadata <- as.data.frame(metadata)
    ma = ma[, row.names(metadata)]
    rownames(ma) = make.names(rownames(ma))
    if (is.null(col)){
        col = "colored"
        metadata[,col] = rep("one_group", nrow(metadata))
    }

    if (!summarize %in% names(metadata))
        metadata[,summarize] = as.factor(paste0(metadata[,col],
                                                metadata[,time]))

    # ensure there are no missing levels in summarize
    metadata[,summarize] = droplevels(metadata[,summarize])

    stopifnot(class(metadata)[1] == "data.frame")
    stopifnot(class(ma)[1] == "matrix" | class(ma)[1] == "data.frame")
    stopifnot(summarize %in% names(metadata))
    stopifnot(time %in% names(metadata))

    ma <- as.matrix(ma)
    if (!is.null(fixy))
        stopifnot(length(fixy) == 2)

    if (nrow(ma)>3000 & is.null(pattern))
        message("A large number of genes was given-- please, ",
                "make sure this is not an error. Normally, ",
                "only DE genes will be useful for this function.")
    message("Working with ", nrow(ma), " genes.")
    
    counts_group <- .summarize_scale(ma,
                                     metadata[[summarize]],
                                     FALSE)
    if (!is.null(groupDifference))
        counts_group <- .remove_low_difference(counts_group,
                                               groupDifference,
                                               eachStep)
    if (scale)
        norm_sign <- t(apply(counts_group, 1, .scale))
    else norm_sign <- counts_group
    
    colnames(norm_sign) <- colnames(counts_group)
    metadata_groups <- metadata %>%
        dplyr::distinct(!!sym(summarize), .keep_all = TRUE)
    rownames(metadata_groups) = metadata_groups[,summarize]
    norm_sign <- norm_sign[, row.names(metadata_groups), drop = FALSE]
    if (nrow(ma) == 1){
        p = .plot_cluster(norm_sign,
                      as.character(rownames(norm_sign)),
                      metadata_groups[,time],
                      metadata_groups[,col], rownames(norm_sign), fixy)
        cluster_genes <- c()
        groups <- as.factor("1")
        names(groups) <- c(row.names(ma))

    }else if (!consensusCluster & is.null(pattern)){
        cluster_genes = .make_clusters(counts_group)
        groups <- .select_genes(cluster_genes, norm_sign, minc,
                               reduce = reduce,
                               cutoff = cutoff, nClusters = nClusters)
        benchmarking <- .benckmark_cutoff(cluster_genes, norm_sign, minc)
    }else if (consensusCluster & is.null(pattern)){
        cluster_genes <- .make_concensus_cluster(counts_group)
        groups <- .select_concensus_genes(cluster_genes)
    }else if(is.character(pattern)){
        stopifnot(pattern %in% rownames(counts_group))
        if (length(pattern) > 1)
            reference <- rowMedians(counts_group[pattern, ])
        else reference <- counts_group[pattern, ]
        cluster_genes <- .find_pattern(counts_group, reference)
        groups <- .select_pattern(cluster_genes)
    }else if(is.numeric(pattern)){
        stopifnot(length(pattern) == ncol(counts_group))
        cluster_genes <- .find_pattern(counts_group, pattern)
        groups <- .select_pattern(cluster_genes)
    }
    temp <- names(groups)
    
    dend_plot <- NA
    if (length(unique(groups)) > 0 & is.null(nClusters) & !skipDendrogram){
        dend <- cluster_genes 
        h = dend$dc
        clust <- cutree(as.hclust(dend), h = h)
        clust.cutree <- dendextend:::cutree(dend, h = h, order_clusters_as_data = FALSE)
        dend <- as.dendrogram(dend, h = h)
        idx <- order(names(clust.cutree))
        clust.cutree <- clust.cutree[idx]
        df.merge <- merge(clust,clust.cutree,by='row.names')
        df.merge.sorted <- df.merge[order(df.merge$y),]
        lbls <- unique(df.merge.sorted$x)
        dend_plot <- dendextend::color_branches(dend, h = h, groupLabels = TRUE,
                                                warn = FALSE) %>% 
            dendextend::set("labels", "") %>% suppressWarnings()
        
        groups <- match(groups, lbls)
        
        if (plot)
            plot(dend_plot, xlab="", ylab="", main="", sub="", axes=FALSE, cex = 2)
    }
    if (length(unique(groups)) > 0 & is.numeric(nClusters) & !skipDendrogram){
        dend <- cluster_genes 
        clust <- cutree(as.hclust(dend), k = nClusters)
        clust.cutree <- dendextend:::cutree(dend, k = nClusters, order_clusters_as_data = FALSE)
        dend <- as.dendrogram(dend, k = nClusters)
        idx <- order(names(clust.cutree))
        clust.cutree <- clust.cutree[idx]
        df.merge <- merge(clust,clust.cutree,by='row.names')
        df.merge.sorted <- df.merge[order(df.merge$y),]
        lbls <- unique(df.merge.sorted$x)
        dend_plot <- dendextend::color_branches(dend, k = nClusters,
                                                groupLabels = TRUE, 
                                                warn = FALSE ) %>%
                                                dendextend::set("labels", "") %>% 
                                                suppressWarnings()
        groups <- match(groups, lbls)
        
        if (plot)
            plot(dend_plot, xlab="", ylab="", main="", sub="", axes=FALSE, cex = 2)
    }
    
    names(groups) <- temp
    df <- data.frame(genes = names(groups), 
                    cluster = groups, stringsAsFactors = FALSE)

    raw <- counts_group %>% as.data.frame %>% 
        rownames_to_column("genes") %>%
        gather(!!sym(summarize), "value", -genes) %>%
        inner_join(metadata_groups %>%
                       mutate_if(is.factor, as.character)) %>%
        inner_join(df, by = "genes")
    
    summarise <- raw %>%
        group_by(!!sym(summarize), !!sym("cluster"),
                 !!sym(time), !!sym(col)) %>%
        summarise(abundance = median(value),
                  n_genes = n()) %>% 
        ungroup()
    
    normalized <- norm_sign %>% as.data.frame() %>% 
        rownames_to_column("genes") %>%
        gather(!!sym(summarize), "value", -genes) %>%
        inner_join(metadata_groups %>%
                       mutate_if(is.factor, as.character)) %>%
        inner_join(df, by = "genes") 

    if (!is.null(benchmarking))
        normalized <- normalized %>% left_join(benchmarking[["genes"]], by = "genes")
    normalized[[time]] = factor(normalized[[time]],
                                levels = levels(metadata[[time]]))
    
    plot_benchmarking <- .plot_benchmarking(normalized, benchmarking, time, col)
    plot_benchmarking_curve <- .plot_benchmarking_curve(benchmarking)
    plotting_data <- list(norm = normalized, time = time, col = col, min_genes = minc) 
    if (length(unique(groups)) > 0){
        p <- degPlotCluster(normalized, time, col, min_genes = minc)
        if (!is.null(fixy))
            p <- p + ylim(fixy[1], fixy[2])
        if (plot)
            print(p)
    }
    
    invisible(list(df = df,
         pass = unique(groups),
         plot = p,
         dend = dend_plot,
         hr = cluster_genes,
         normalized = normalized,
         summarise = summarise,
         raw = raw,
         counts = ma[raw[["genes"]],],
         benchmarking = plot_benchmarking,
         benchmarking_curve = plot_benchmarking_curve))
}

context("Clustering")
library(DESeq2)

data(humanGender)
idx <- c(1:5, 75:80)
counts <- assays(humanGender)[[1]]
dse <- DESeqDataSetFromMatrix(counts[1:1000, idx],
                              colData(humanGender)[idx,],
                              design = ~group) %>% DESeq
ma <- assays(dse)[[1]]
des <- colData(dse)
des[["other"]] <- sample(c("a", "b"), length(idx), replace = TRUE)
res <- degPatterns(ma, des, time="group", col = "other", plot = FALSE)

test_that("cluster_plot", {
    expect_error(degPlotCluster(res, "group", "other"))
    # expect_is(degPlotCluster(res$normalized, "group", "other"), "gg")
})

# test_that("missingfactor", {
#    des_missing <- des
#    levels(des_missing$group) <- c(levels(des_missing$group), "missing")
#    expect_is(degPatterns(ma, des_missing, time="group", col = NULL, plot = FALSE), "list")
# 
# })

# test_that("transform", {
#     expect_gte(mean(.scale(counts(dse)[1,])), -0.5)
#     expect_lte(mean(.scale(counts(dse)[1,])), 0.5)
#     countsGroup <- .summarize_scale(counts(dse)[1:100,],
#                                     colData(dse)[["group"]])
#     expect_equal(mean(countsGroup), 0)
#     hc <- .make_clusters(countsGroup)
#     cluster0 <- .select_genes(hc, countsGroup, minc = 5, reduce = TRUE)
#     cluster <- .select_genes(hc, countsGroup, minc = 5)
#     expect_equal(unique(cluster), c(1, 2))
#     expect_equal(unique(cluster0), c(1, 2))
#     df <- data.frame(cluster = cluster, genes = names(cluster))
#     expect_equal(.median_per_cluster(countsGroup, df) %>% mean, 0)
#     expect_equal(.filter(df, 50) %>% nrow, 69)
#     expect_equal(.group_metadata(as.data.frame(colData(dse)),
#                                  "group", "group", "group") %>% nrow, 2)
# })

test_that("groupDifference", {
    ma <- matrix(rnorm(50), ncol = 2)
    ma[1:20, 2] <- 1000 + ma[1:20, 2]
    expect_equal(.remove_low_difference(ma, 500, FALSE) %>% nrow(), 20)
})

test_that("pct_variance", {
    clusters <- c(rep(1, 500), rep(2, 500))
    names(clusters) <- row.names(counts[1:1000, idx])
    pct <- .pct_var(counts[1:1000, idx], clusters)
    expect_true(pct < 100)
})

test_that("process", {
    library(dplyr)
    library(tidyr)
    library(tibble)
    table <- rownames_to_column(as.data.frame(ma), "genes") %>%
        gather("sample", "expression", -genes) %>%
        right_join(distinct(res$df[,c("genes", "cluster")]),
                   by = "genes") %>%
        left_join(rownames_to_column(as.data.frame(des), "sample"),
                  by = "sample")
    # expect_true("value"  %in%  names(.process(table, "group", NULL)))
})

#' DEGSet
#' 
#' S4 class to store data from differentially expression analysis.
#' It should be compatible with different package and stores the information
#' in a way the methods will work with all of them.
#' 
#' @details
#' For now supporting only [DESeq2::results()] output.
#' Use constructor [degComps()] to create the object.
#' 
#' The list will contain one element for each comparison done.
#' Each element has the following structure:
#' 
#' * DEG table
#' * Optional table with shrunk Fold Change when it has been done.
#' 
#' To access the raw table use `deg(dgs, "raw")`, to access the 
#' shrunken table use `deg(dgs, "shrunken")` or just `deg(dgs)`. 
#' 
#' @rdname DEGSet
#' @name DEGSet
#' @param resList List with results as elements containing log2FoldChange,
#'   pvalues and padj as column. Rownames should be feature names. Elements
#'   should have names.
#' @param default The name of the element to use by default.
#' @param extras List of extra tables related to the same comparison when using `as.DEGSet`.
#' @param object Different objects to be transformed to DEGSet when using `as.DEGSet`.
#' @param contrast To name the comparison when using `as.DEGSet`.
#' @param ... Optional parameters of the generic.
#' @aliases DEGSet-class DEGSet
#' @author Lorena Pantano
#' @examples
#' library(DESeq2)
#' library(edgeR)
#' library(limma)
#' dds <- makeExampleDESeqDataSet(betaSD = 1)
#' colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
#' design(dds) <-  ~ condition + treatment
#' dds <- DESeq(dds)
#' res <- degComps(dds, combs = c("condition"))
#' deg(res)
#' deg(res, tidy = "tibble")
#' # From edgeR
#' dge <- DGEList(counts=counts(dds), group=colData(dds)[["treatment"]])
#' dge <- estimateCommonDisp(dge)
#' res <- as.DEGSet(topTags(exactTest(dge)))
#' # From limma
#' v <- voom(counts(dds), model.matrix(~treatment, colData(dds)), plot=FALSE)
#' fit <- lmFit(v)
#' fit <- eBayes(fit, robust=TRUE)
#' res <- as.DEGSet(topTable(fit, n = "Inf"), "A_vs_B")
#' @export
DEGSet <- setClass("DEGSet",
                          contains = "list",
                          slots = c(default = "character"))

setValidity("DEGSet", function(object) {
    stopifnot(!is.null(names(object)))
    stopifnot(degDefault(object) %in% names(object))
    TRUE
})

.validate <- function(object){
    if (sum(c("raw", "shrunken") %in% names(object)) < 2)
        message("Some functions won't work without
                'raw' and 'shrunken' elements in the object.")
}

#' @rdname DEGSet
#' @export
DEGSet <- function(resList, default){
    stopifnot(!is.null(names(resList)))
    stopifnot(default %in% names(resList))
    
    new("DEGSet", resList,
        default = default)
}

# Wrap figure from \code{degMV} into a Nozzle object
figurepvaluebyvarexp <- 
    function(g, pvalues, counts, out)
    {
        message("Deprecated function since Nozzle.R1 is not longer available.")
        # p <- degQC(counts, g, pvalue = pvalues)
        # File = "fpvaluebyvarexp.jpg"
        # HFile = "fpvaluebyvaexpr.pdf"
        # jpeg(file.path(out, File) ,width=600, height=400, quality=100 );
        # print(p)
        # dev.off();
        # pdf(file.path( out, HFile));
        # print(p)
        # dev.off();
        
        # # create a figure and make it available for exporting
        # newFigure( File, fileHighRes=HFile, exportId="PVALBYVAREXP",
        #            "This figure shows the distribution of pvalues according
        #            the average expression and the variability of the feature. 
        #            It is taking the maximum value among group1 and group2" );
    }
# Wrap figure from \code{degMB} into a Nozzle object
figurebyexp <- 
    function(tags, g, counts, out, pop=400)
    {
        message("Deprecated function since Nozzle.R1 is not longer available.")
        # p <- degMB(tags, g, counts, pop)
        # File = "fexp.jpg"
        # HFile = "fexp.pdf"
        # jpeg(file.path(out, File), width = 600, height = 400, quality = 100 );
        # print(p)
        # dev.off();
        # pdf(file.path(out, HFile));
        # print(p)
        # dev.off();
        
        # # create a figure and make it available for exporting
        # newFigure(File, fileHighRes=HFile, exportId="EXP",
        #            "This figure shows the distribution of expression values
        #            of the two groups.");
    }
# Wrap figure from \code{degVB} into a Nozzle object
figurebyvar <- 
    function(tags, g, counts, out, pop=400)
    {
        message("Deprecated function since Nozzle.R1 is not longer available.")
        # p <- degVB(tags, g, counts, pop)
        # File = "fvar.jpg"
        # HFile = "fvar.pdf"
        # jpeg(file.path(out, File), width = 600, height = 400, quality = 100);
        # print(p)
        # dev.off();
        # pdf(file.path(out, HFile));
        # print(p)
        # dev.off();
        
        # # create a figure and make it available for exporting
        # newFigure(File, fileHighRes=HFile, exportId="EXP",
        #            "This figure shows the distribution of SD
        # of the two groups.");
    }

#' Create report of RNAseq DEG anlaysis
#' 
#' @description This function get the count matrix, pvalues, and FC of a 
#' DEG analysis and create a report to help to detect possible problems 
#'   with the data.
#' @aliases createReport
#' @param g Character vector with the group the samples belong to.
#' @param counts Matrix with counts for each samples and each gene. 
#'   Should be same length than pvalues vector.
#' @param tags Genes of DEG analysis
#' @param pvalues pvalues of DEG analysis
#' @param path path to save the figure
#' @param pop random genes for background
#' @param name name of the html file
#' @return A HTML file with all figures and tables
#' @export
createReport <- 
    function(g, counts, tags, pvalues, path,
             pop = 400, name="DEGreport")
    {
    message("Deprecated function since Nozzle.R1 is not longer available.")

    #     requireNamespace("Nozzle.R1")
    #     fg3 <- figurepvaluebyvarexp(g, pvalues, counts, path)
    #     fg4 <- figurebyexp(tags, g, counts, path, pop)
    #     fg5 <- figurebyvar(tags, g, counts, path, pop)
    #     report <- ""
    #     report  <- newCustomReport( "DEG Report " );
    #     report  <- addTo( 
    #         report, addTo( newSection( "Quality of DEG", class="results" ),
    #                        addTo(newSubSection("Pvalue vs abundance/variation"), fg3),    
    #                        addTo(newSubSection("Abundance distribution"), fg4),
    #                        addTo(newSubSection("Variation distribution"), fg5)
    #         ))
        
    #     writeReport(report, filename = file.path(path, name))
    }

#' Make nice colors for metadata
#' 
#' The function will take a metadata table and use Set2 palette when
#' number of levels is > 3 or a set or orange/blue colors other wise.
#' 
#' @param ann Data.frame with metadata information. Each column will be used to 
#'   generate a palette suitable for the values in there.
#' @param col_fun Whether to return a function for continuous variables 
#'   (compatible with [ComplexHeatmap::HeatmapAnnotation()]) or
#'   the colors themself (comparible with [pheatmap::pheatmap())]).
#' @param con_values Color to be used for continuous variables.
#' @param cat_values Color to be used for 2-levels categorical variables.
#' @param palette Palette to use from `brewer.pal()` for
#'   multi-levels categorical variables.
#' @examples 
#' data(humanGender)
#' library(DESeq2)
#' library(ComplexHeatmap)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:10, idx],
#'   colData(humanGender)[idx,], design=~group)
#' th <- HeatmapAnnotation(df = colData(dse),
#'                        col = degColors(colData(dse), TRUE))
#' Heatmap(log2(counts(dse)+0.5), top_annotation = th)
#' 
#' custom <- degColors(colData(dse), TRUE,
#'           con_values = c("white", "red"),
#'           cat_values = c("white", "black"),
#'           palette = "Set1")
#' th <- HeatmapAnnotation(df = colData(dse),
#'                         col = custom)
#' Heatmap(log2(counts(dse)+0.5), top_annotation = th)
#' @export
degColors <- function(ann, col_fun = FALSE,
                      con_values = c("grey80", "black"),
                      cat_values = c("orange", "steelblue"),
                      palette = "Set2"){
    col <- lapply(names(ann), function(a){
        if (class(ann[[a]][1])[1] == "numeric"){
            fn = colorRamp2(c(min(ann[[a]]),
                              max(ann[[a]])),
                            con_values)
            if (col_fun){
                return(fn)
            }else{
                return(fn(ann[[a]]))
            }
        }
        
        if (length(unique(ann[[a]])) < 3){
            v <- cat_values[1:length(unique(ann[[a]]))]
            names(v) <- unique(ann[[a]])
            return(v)
        }
        v <- brewer.pal(length(unique(ann[[a]])), palette)
        names(v) <- unique(ann[[a]])
        v
    })
    names(col) <- names(ann)
    col
}


#' Plot top genes allowing more variables to color and shape points
#'
#' @param dds [DESeq2::DESeqDataSet] object or SummarizedExperiment
#'   or Matrix or data.frame. In case of a DESeqDataSet object, always
#'   the normalized expression will be used
#'   from `counts(dds, normalized = TRUE)`.
#' @param res [DESeq2::DESeqResults] object.
#' @param n Integer number of genes to plot from the `res` object. It will
#'   take the top N using padj values to order the table.
#' @param genes Character of gene names matching rownames of count data.
#' @param xs Character, colname in colData that will be used as X-axes.
#' @param group Character, colname in colData to color points and add different
#'   lines for each level.
#' @param batch Character, colname in colData to shape points, normally used by
#'   batch effect visualization.
#' @param ann Columns in rowData (if available) used to print gene names. First
#'   element in the vector is the column name in rowData that matches the
#'   row.names of the `dds` or `count` object. Second element in the vector
#'   is the column name in rowData that it will be used as the title for each
#'   gene or feature figure.
#' @param metadata Metadata in case dds is a matrix.
#' @param slot Name of the slot to use to get count data.
#' @param log2 Whether to apply or not log2 transformation.
#' @param xsLab Character, alternative label for x-axis (default: same as xs)
#' @param ysLab Character, alternative label for y-axis..
#' @param color Color to use to plot groups. It can be one color, or a palette
#'   compatible with `ggplot2::scale_color_brewer()`.
#' @param groupLab Character, alternative label for group (default: same as group).
#' @param batchLab Character, alternative label for batch (default: same as batch).
#' @param sizePoint Integer, indicates the size of the plotted points (default 1).
#' @return ggplot showing the expresison of the genes
#' @examples 
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dse <- DESeq(dse)
#' degPlot(dse, genes = rownames(dse)[1:10], xs = "group")
#' degPlot(dse, genes = rownames(dse)[1:10], xs = "group", color = "orange")
#' degPlot(dse, genes = rownames(dse)[1:10], xs = "group", group = "group",
#'         color = "Accent")
#' @export
degPlot = function(dds, xs, res = NULL, n = 9, genes = NULL,
                   group = NULL, batch = NULL,
                   metadata = NULL,
                   ann = c("geneID", "symbol"),
                   slot = 1L,
                   log2 = TRUE,
                   xsLab = xs,
                   ysLab = "abundance",
                   color = "black",
                   groupLab = group, batchLab = batch,
                   sizePoint = 1){
    if (class(dds)[1] %in% c("data.frame", "matrix"))
        dds = SummarizedExperiment(assays = SimpleList(counts = as.matrix(dds)),
                                   colData = metadata)
    
    if ( !("assays" %in% slotNames(dds)) )
        stop("dds object doesn't have assays slot")
    
    if (!is.null(res))
        res <- res[order(res$padj),] %>% .[!is.na(res$padj),]
    
    if (is.null(genes))
        genes =  row.names(res)[1L:n]
    
    anno <- as.data.frame(rowData(dds))
    
    metadata = data.frame(colData(dds))
    if (class(dds)[1] == "DESeqDataSet")
        counts <- counts(dds, normalized = TRUE)
    else counts <- assays(dds)[[slot]]
    
    stopifnot(class(counts)[1] == "data.frame" | class(counts)[1] == "matrix")
    
    if (log2 & max(counts) < 500L)
        warning("Data seems to be already in log2. Please use log2 = FALSE.")
    if (log2){
        counts <- log2(counts + 0.2)
        ysLab <- paste("log2", ysLab)
    }

    newgenes <- genes
    if (ncol(anno) > 0) {
        name <- intersect(names(anno), ann)
        if (length(name) != 2L)
            message("No genes were mapped to rowData. check ann parameter values.")
        if (length(name) == 2L)
            newgenes <- anno[match(genes, anno[, ann[1L]]), ann[2L]]
        if (sum(is.na(newgenes)) > 0)
            warning(sum(is.na(newgenes)), " cannot be mapped to ", name[2L], 
                    ". Those will be skipped.")
    }
    
    dd <- melt(as.data.frame(counts[genes, , drop = FALSE]) %>%
                  mutate(gene = factor(newgenes, levels=newgenes)))
    colnames(dd) = c("gene", "sample", "count")
    
    dd <-dd[!is.na(dd[["gene"]]),]
    
    dd$xs = as.factor(metadata[as.character(dd$sample), xs])
    
    if (!is.null(group)) {
        dd[, groupLab] = as.factor(metadata[as.character(dd$sample), group])
    }else {
        groupLab = "fake"
        dd[, groupLab] = "fake"
    }
    
    if (!is.null(batch)) {
        dd[, batchLab] = as.factor(metadata[as.character(dd$sample), batch])
        
        p = ggplot(dd, aes_string(x = "xs", y = "count", color = groupLab,
                                  shape = batchLab))
    }else{
        p = ggplot(dd, aes_string(x = "xs", y = "count", color = groupLab))
    }
    
    p = p +
        # geom_violin(alpha=0.3) +
        stat_smooth(fill = "grey80", method = 'loess') +
        geom_point(size = sizePoint, alpha = 0.7,
                   position = position_jitterdodge(dodge.width = 0.9)) +
        facet_wrap(~gene, scales = "free_y") +
        xlab(xsLab) +
        ylab(ysLab)
    if (length(unique(dd[, groupLab])) == 1L) {
        stopifnot(length(color) == 1)
        p = p +
            scale_color_manual(guide = "none", values = color) +
            scale_fill_manual(guide = "none", values = color)
    }else{
        if (color == "black")
            color = "Set1"
        p = p +
            scale_color_brewer(palette = color) +
            scale_fill_brewer(palette = color)
    }
    p = p + theme_bw() +
        theme(strip.background = element_rect(fill = "white"),
              strip.text = element_text(colour = "black"))
    
    suppressWarnings(p)
}

#' Plot selected genes on a wide format
#'
#' @param counts [DESeq2::DESeqDataSet] object or expression matrix
#' @param genes character genes to plot.
#' @param group character, colname in colData to color points and add different
#' lines for each level
#' @param metadata data.frame, information for each sample. Not needed if
#' [DESeq2::DESeqDataSet] given as counts.
#' @param batch character, colname in colData to shape points, normally used by
#' batch effect visualization
#' @return ggplot showing the expresison of the genes on the x
#' axis
#' @examples
#' data(humanGender)
#' library(DESeq2)
#' idx <- c(1:10, 75:85)
#' dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
#'   colData(humanGender)[idx,], design=~group)
#' dse <- DESeq(dse)
#' degPlotWide(dse, rownames(dse)[1:10], group = "group")
#' @export
degPlotWide <- function(counts, genes, group, metadata=NULL, batch=NULL){
    if (is.null(metadata))
        metadata = data.frame(colData(counts))
    metadata = data.frame(metadata)
    if (class(counts)[1] == "DESeqDataSet") {
        dd = bind_rows(lapply(genes,function(gene){
            plotCounts(counts, gene,
                       intgroup = group, returnData = TRUE) %>%
                mutate(count = log2(count + 1)) %>%
                mutate(gene = gene, sample = row.names(metadata))}))
        dd$group = dd[[group]]
    }else if (class(counts)[1] == "matrix") {
        dd = melt(counts[genes, ])
        colnames(dd) = c("gene", "sample", "count")
        dd$group = as.factor(metadata[as.character(dd$sample), group])
    }else{
        stop("No supported for class", class(counts)[1])
    }
    if (is.null(group)) {
        dd$treatment = "one_group"
    }else{
        dd$treatment = dd[,"group"]
    }
    p = ggplot(dd, aes_string(x = "gene", y = "count", color = "treatment"))
    if (!is.null(batch)) {
        dd$batch = as.factor(metadata[dd$sample, batch])
        p = ggplot(dd, aes_string(x = "gene", y = "count",
                                  color = "treatment", shape = "batch"))
    }
    
    p = p +
        geom_point(position = position_jitterdodge(dodge.width = 0.9)) +
        xlab("Genes") +
        ylab("Normalized Counts") +
        theme_bw() +
        theme(axis.text.x = element_text(angle = 45L, hjust = 1L))
    p
}

fmt <- function(){
    function(x) format(x,nsmall = 1,scientific = FALSE)
}
#' Create volcano plot from log2FC and adjusted pvalues data frame
#'
#' @param stats data.frame with two columns: logFC and Adjusted.Pvalue
#' @param side plot UP, DOWN or BOTH de-regulated points
#' @param title title for the figure
#' @param pval.cutoff cutoff for the adjusted pvalue. Default 0.05
#' @param lfc.cutoff cutoff for the log2FC. Default 1
#' @param shade.alpha transparency value. Default 0.25
#' @param shade.colour background color. Default orange.
#' @param point.colour colours for points. Default gray
#' @param point.alpha transparency for points. Default 0.75
#' @param point.outline.colour Default darkgray
#' @param line.colour Defaul gray
#' @param plot_text data.frame with three columns: logFC, Pvalue, Gene name
#' @return The function will plot volcano plot together
#' with density of the fold change and p-values on the top and the
#' right side of the volcano plot.
#' @details 
#' This function was mainly developed by @jnhutchinson.
#' @author Lorena Pantano, John Hutchinson
#' @examples 
#' library(DESeq2)
#' dds <- makeExampleDESeqDataSet(betaSD = 1)
#' dds <- DESeq(dds)
#' stats <- results(dds)[,c("log2FoldChange", "padj")]
#' stats[["name"]] <- row.names(stats)
#' degVolcano(stats, plot_text = stats[1:10,])
#' @export
degVolcano <- function(stats, side="both", title="Volcano Plot with Marginal Distributions",
                                 pval.cutoff=0.05, lfc.cutoff=1, shade.colour="orange",
                                 shade.alpha=0.25, point.colour="gray", point.alpha=0.75,
                                 point.outline.colour="darkgray", line.colour="gray",
                                 plot_text=NULL) {
    if (class(stats)[1] == "DESeqResults")
        stats <- stats[, c("log2FoldChange", "padj")]
    if (class(stats)[1] == "DEGSet"){
        stats <- deg(stats, tidy = "data.frame")
        stats <- stats[, c("log2FoldChange", "padj")]
    }

    stats <- as.data.frame(stats)
    if (!any(side %in% c("both","down","up")) | length(side)>1)
        stop("side parameter should be: both, up or down.")
    if (ncol(stats)<2)
        stop("Need a data.frame with two columns: logFC and Adjusted.Pvalue")
    if (sum( rowSums(is.na(stats)) ) > 0)
        stats = stats[rowSums(is.na(stats)) == 0,]
    if (any(stats[,2]>1) | any(stats[,2]<0))
        stop("pvalues needs to be >0 and <1")
    names(stats) = c("logFC","adj.P.Val")
    stats[,2] = -log10(stats[,2] + 1e-10)
    
    # get range of log fold change and p-value values to setup plot borders
    range.lfc <- c(floor(min(stats$logFC)), ceiling(max(stats$logFC)))
    range.pval <- c(floor(min(stats$adj.P.Val)), ceiling(max(stats$adj.P.Val)))
    pval.cutoff <- -log10(pval.cutoff)

    #make scatter volcano plot
    scat.poly.up <- with(stats, data.frame(x=as.numeric(c(lfc.cutoff,  lfc.cutoff, max(range.lfc),max(range.lfc))), 
                                           y=as.numeric(c(pval.cutoff, max(range.pval), max(range.pval),pval.cutoff))))
    scat.poly.down <- with(stats, data.frame(x=as.numeric(c(-lfc.cutoff,  -lfc.cutoff, min(range.lfc),min(range.lfc))), 
                                             y=as.numeric(c(pval.cutoff, max(range.pval), max(range.pval),pval.cutoff))))
    
    scatter <- ggplot(stats, aes_string(x="logFC", y="adj.P.Val")) +
        geom_point(alpha=point.alpha, pch=21, fill=point.colour, color=point.outline.colour) +
        xlab("log2 fold change") + ylab("-log10(adjusted p-value)") +
        theme_bw()+
        theme(legend.position="none") +
        theme(plot.margin=unit(c(3,-5.5,4,3), "mm") )+
#         scale_x_continuous(limits = range.lfc, breaks = range.lfc[1]:range.lfc[2])+
        scale_y_continuous(labels=fmt(), limits = range.pval)+ labs(title="Volcano plot")
    if (side=="both" | side=="up")
        scatter = scatter + geom_polygon(data=scat.poly.up, aes_string(x="x",y="y"), fill=shade.colour, alpha=shade.alpha)
    if (side=="both" | side=="down")
        scatter = scatter + geom_polygon(data=scat.poly.down, aes_string(x="x",y="y"), fill=shade.colour, alpha=shade.alpha)

    if (!is.null(plot_text)){
        plot_text <- as.data.frame(plot_text)
        names(plot_text) = c("logFC", "adj.P.Val", "name")
        plot_text[,2] <- -log10(plot_text[,2] + 1e-10)
        scatter <- scatter + 
            geom_text_repel(data=plot_text, aes_string(x="logFC", y="adj.P.Val", label="name"), size=3)
    }
    scatter
}

.get_counts <- function(c, slot){
    if (class(c)[1] == "bcbioRNASeq")
        return(counts(c, "rlog"))
    if (class(c)[1] == "DESeqDataSet")
        return(counts(c, normalized = TRUE))
    if (class(c)[1] %in% c("SummarizedExperiment", "RangedSummarizedExperiment"))
        return(assays(c)[[slot]])
    if (class(c)[1] == "data.frame")
        return(c)
    if (class(c)[1] == "matrix")
        return(as.data.frame(c))
    stop("class ", class(c)[1], " no supported.")
}

.get_meta <- function(c){
    if (class(c)[1] %in% c("bcbioRNASeq", "DESeqDataSet",
                        "SummarizedExperiment", "RangedSummarizedExperiment"))
        return(data.frame(colData(c), stringsAsFactors = FALSE))
    return(NULL)
}

#' Plot gene signature for each group and signature
#' 
#' Given a list of genes beloging to a different classes, like
#' markers, plot for each group, the expression values for all the samples.
#' 
#' @param counts expression data. It accepts bcbioRNASeq, DESeqDataSet and
#'   SummarizedExperiment. As well, data.frame or matrix is supported, but
#'   it requires metadata in that case.
#' @param signature data.frame with two columns: a) genes that match
#'   row.names of counts, b) label to classify the gene inside a group.
#'   Normally, cell tissue name.
#' @param group character in metadata used to split data into different
#'   groups.
#' @param metadata data frame with sample information. Rownames
#'   should match \code{ma} column names
#'   row number should be the same length than p-values vector.
#' @param slot slotName in the case of SummarizedExperiment objects.
#' @param scale Whether to scale or not the expression.
#' @return ggplot plot.
#' @examples
#' data(humanGender)
#' data(geneInfo)
#' degSignature(humanGender, geneInfo, group = "group")
#' @export
degSignature <- function(counts, signature,
                         group = NULL, metadata = NULL,
                         slot = 1, scale = FALSE){
    c <- .get_counts(counts, slot)
    if (scale)
        c <- t(scale(t(c)))
    meta <- .get_meta(counts)
    if (is.null(meta))
        meta <- metadata
    stopifnot(group %in% colnames(meta))

    names(signature) <- c("gene", "signature")
    common <- intersect(row.names(c), signature[["gene"]])
    c[common, ] %>%  as.data.frame() %>% rownames_to_column("id") %>% 
        melt() %>%  data.frame(., stringsAsFactors = FALSE) %>% 
        set_colnames(c("gene", "sample", "expression")) %>% 
        mutate_if(is.factor, as.character) %>% 
        left_join(meta %>% rownames_to_column("degsample") %>% 
                      mutate_if(is.factor, as.character),
                  by = c("sample" = "degsample")) %>%
        left_join(signature %>% 
                      mutate_if(is.factor, as.character), by = "gene") %>% 
        group_by(!!sym(group), signature, sample) %>% 
        summarise(median = median(expression)) %>% 
        ungroup %>% 
        ggplot(aes_string(x = group, y = "median", color = "signature")) +
        geom_boxplot() +
        geom_jitter() +
        facet_wrap(~signature)
}

StatCor <- ggproto("StatCor", Stat,
                   compute_group = function(data, scales,
                                            xpos = NULL, ypos = NULL,
                                            method = "spearman") {
                       
                       data = data[!is.na(data$x) & !is.na(data$y),]
                       cr = cor.test(data$x, data$y, method = method)
                       x = xpos
                       if (is.null(x))
                           x = min(data$x) + abs((max(data$x) - min(data$x)) / 2)
                       y = ypos
                       if (is.null(y))
                           y = scales$y$range$range[2] + 0.1 * scales$y$range$range[2]
                       data.frame(x = x,
                                  y = y,
                                  label = paste0("Cor (", method, "): ",
                                                 round(cr[["estimate"]], digits = 2),
                                                 ", pval: ",
                                                 round(cr[["p.value"]], digits = 2)))
                       
                   },
                   required_aes = c("x", "y")
)

GeomCor <- ggproto("GeomText", Geom,
                   required_aes = c("x", "y"),
                   
                   default_aes = aes(
                       colour = "black", size = 3.88, angle = 0, hjust = 0.5,
                       vjust = 0.5, alpha = 1, family = "", fontface = 1, lineheight = 1.2
                   ),
                   
                   
                   draw_panel = function(data, panel_params, coord, parse = FALSE,
                                         na.rm = FALSE, check_overlap = FALSE) {
                       
                       lab <- data$label
                       if (parse) {
                           lab <- parse(text = as.character(lab))
                       }
                       data <- coord$transform(data, panel_params)
                       if (is.character(data$vjust)) {
                           data$vjust <- data$y
                       }
                       if (is.character(data$hjust)) {
                           data$hjust <- data$x
                       }

                       textGrob(
                           lab,
                           data$x, data$y, default.units = "native",
                           hjust = data$hjust, vjust = data$vjust,
                           gp = grid::gpar(
                               col = data$colour,
                               fontsize = data$size * 3,
                               fontfamily = data$family,
                               fontface = data$fontface,
                               lineheight = data$lineheight
                           )
                       )
                   },
                   
                   draw_key = draw_key_text
)

#' Add correlation and p-value to a [ggplot2] plot
#' 
#' `geom_cor` will add the correlatin, method and p-value to the plot
#'   automatically guessing the position if nothing else specidfied.
#'   family font, size and colour can be used to change the format.
#' 
#' @seealso [ggplot2::layer()]
#' @param mapping Set of aesthetic mappings created by [aes()] or
#'   [aes_()]. If specified and `inherit.aes = TRUE` (the
#'   default), it is combined with the default mapping at the top level of the
#'   plot. You must supply `mapping` if there is no plot mapping.
#' @param data The data to be displayed in this layer. There are three
#'    options:
#'
#'    If `NULL`, the default, the data is inherited from the plot
#'    data as specified in the call to [ggplot()].
#'
#'    A `data.frame`, or other object, will override the plot
#'    data. All objects will be fortified to produce a data frame. See
#'    [fortify()] for which variables will be created.
#'
#'    A `function` will be called with a single argument,
#'    the plot data. The return value must be a `data.frame.`, and
#'    will be used as the layer data.
#' @param inherit.aes If `FALSE`, overrides the default aesthetics,
#'   rather than combining with them. This is most useful for helper functions
#'   that define both data and aesthetics and shouldn't inherit behaviour from
#'   the default plot specification, e.g. [borders()].
#' @param method Method to calculate the correlation. Values are
#'   passed to [cor.test()]. (Spearman, Pearson, Kendall).
#' @param ypos Locate text at that position on the y axis.
#' @param xpos Locate text at that position on the x axis.
#' @param ... other arguments passed on to [layer()]. These are
#'   often aesthetics, used to set an aesthetic to a fixed value, like
#'   `color = "red"` or `size = 3`. They may also be parameters
#'   to the paired geom/stat.
#' @details It was integrated after reading this tutorial to extend
#' ggplot2 [layers](https://cran.r-project.org/web/packages/ggplot2/vignettes/extending-ggplot2.html)
#' @examples
#' data(humanGender)
#' library(SummarizedExperiment)
#' library(ggplot2)
#' ggplot(as.data.frame(assay(humanGender)[1:1000,]),
#'        aes(x = NA20502, y = NA20504)) +
#'   geom_point() +
#'   ylim(0,1.1e5) +
#'   geom_cor(method = "kendall", ypos = 1e5) 
#' @export
geom_cor <- function(mapping = NULL, data = NULL,
                     method = "spearman",
                     xpos = NULL, ypos = NULL,
                     inherit.aes = TRUE, ...) {
    layer(stat = StatCor,
                   data = data, mapping = mapping, geom = GeomCor,
                   position = "identity",
                   inherit.aes = inherit.aes,
                   params = list(na.rm = FALSE, method = method, ...)
    )
}

#' @name geneInfo
#' @title data.frame with chromose information for each gene
#' @description data.frame with chromose information for each gene
#' @docType data
#' @usage data(geneInfo)
#' @format data.frame
#' @source biomart
#' @author Lorena Pantano, 2014-08-14
NULL

#' @rdname deg
#' @export
setGeneric("deg", function(object, value=NULL, tidy = NULL, top = NULL, ...)
    standardGeneric("deg"))

#' @rdname degDefault
#' @export
setGeneric("degDefault", function(object)
    standardGeneric("degDefault"))

#' @rdname degDefault
#' @export
setGeneric("degCorrect", function(object, fdr)
    standardGeneric("degCorrect"))


#' @rdname significants
#' @export
setGeneric("significants", function(object, padj = 0.05, fc = 0,
                                    direction = NULL, full = FALSE, ...)
    standardGeneric("significants"))

#' @rdname DEGSet
#' @export
setGeneric("as.DEGSet", function(object, ...)
    standardGeneric("as.DEGSet"))

setMethod("show", "DEGSet",
          function(object){
              cat("Comparisons:" , .get_contrast_name(object))
              cat("\nResults in comparison:",
                  paste(names(object),
                        collapse =","))
              cat("\nDefault is:", degDefault(object))
          }
)

#' Method to get the default table to use.
#' 
#' @param object [DEGSet]
#' @author Lorena Pantano
#' @rdname degDefault
#' @export
setMethod("degDefault", signature("DEGSet"), 
          function(object) {
    slot(object, name = "default")
})


#' Method to re-calculate the padj column.
#' 
#' It can accept a list of new padj values matching the 
#' same dimmensions than the current vector.
#' 
#' @param object [DEGSet]
#' @param fdr It can be `fdr-stat`, `fdr-pvalue`, vector of new padj
#' @author Lorena Pantano
#' @rdname degDefault
#' @examples
#' library(DESeq2)
#' library(dplyr)
#' dds <- makeExampleDESeqDataSet(betaSD=1)
#' colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
#' design(dds) <-  ~ condition + treatment
#' dds <- DESeq(dds)
#' res <- degComps(dds, contrast = list("treatment_B_vs_A"))
#' @export
setMethod("degCorrect", signature("DEGSet"),
          function(object, fdr) {
              res <- lapply(names(object), function(n){
                  .correct_fdr(object[[n]], fdr)
              })
              names(res) <- names(object)
              new("DEGSet", res,
                  default = degDefault(object))
})

 
#' Method to get all table stored for an specific comparison
#'
#' @inheritParams degDefault
#' @param value Character to specify which table to use.
#' @param tidy Return data.frame, tibble or original class.
#' @param top Limit number of rows to return. Default: All.
#' @param ... Other parameters to pass for other methods.
#' @author Lorena Pantano
#' @rdname deg
#' @references  
#' * Testing if `top` is whole number or not comes from:
#'   https://stackoverflow.com/a/3477158
#' @export
setMethod("deg", signature("DEGSet"),
          function(object, value=NULL, tidy = NULL, top = NULL, ...) {
              df <- NULL
              stopifnot(value %in% c(NULL, names(object)))
              
              if (is.null(value))
                  df <- object[[degDefault(object)]]
              else df <- object[[value]]
              
              if (is.null(top))
                  top <- nrow(df)
              stopifnot(top%%1 == 0)

              if (is.null(tidy))
                  return(df %>% .[order(.[["padj"]]),] %>%
                             .[1:top,])
              if (tidy == "data.frame")
                  return(as.data.frame(df) %>%
                             .[order(.[["padj"]]),] %>%
                             .[1:top,])
              if (tidy == "tibble")
                  return(as.data.frame(df) %>%
                             rownames_to_column("gene") %>%
                             .[order(.[["padj"]]),] %>%
                             .[1:top,] %>%
                             as_tibble)
              stop("Not supported format, ", tidy)
          })

.filterTable <- function(df, direction, fcn, fc , fdr, padj){
    filterOut <- NULL
    if (is.null(direction))
        filterOut <- abs(df[[fcn]]) > fc & df[[fdr]] < padj
    else if (direction == "up")
        filterOut <- df[[fcn]] > fc & df[[fdr]] < padj
    else if (direction == "down")
        filterOut <- df[[fcn]] < (-1L * fc) & df[[fdr]] < padj
    else
        stop("Value ", direction, " is not valid: NULL, down, up.")
    return(filterOut)
}

#' Method to get the significant genes
#' 
#' Function to get the features that are significant
#' according to some thresholds from a [DEGSet],
#' [DESeq2::DESeqResults] and [edgeR::topTags].
#' 
#' @author Lorena Pantano
#' @inheritParams degDefault
#' @param padj Cutoff for the FDR column.
#' @param fc Cutoff for the log2FC column.
#' @param direction Whether to take down/up/ignore. Valid arguments are
#'   down, up and NULL.
#' @param full Whether to return full table or not.
#' @param newFDR Whether to recalculate the FDR or not.
#'   See https://support.bioconductor.org/p/104059/#104072.
#'   Only used when a list is giving to the method.
#' @param ... Passed to [deg]. Default: value = NULL.
#'   Value can be 'raw', 'shrunken'.
#' @rdname significants
#' @return a [dplyr::tbl_df] data frame. `gene` column has the feature name.
#'  In the case of using this method with the results from [degComps], 
#'  `log2FoldChange` has the higher foldChange from the comparisons, and
#'  `padj` has the padj associated to the previous column. Then, there is
#'  two columns for each comparison, one for the log2FoldChange and another
#'  for the padj.
#' @examples
#' library(DESeq2)
#' dds <- makeExampleDESeqDataSet(betaSD=1)
#' colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
#'   design(dds) <-  ~ condition + treatment
#' dds <- DESeq(dds)
#' res <- degComps(dds, contrast = list("treatment_B_vs_A",
#'                                      c("condition", "A", "B")))
#' significants(res, full = TRUE)
#' # significants(res, full = TRUE, padj = 1) # all genes
#' @export
setMethod("significants", signature("DEGSet"),
          function(object, padj = 0.05, fc = 0,
                   direction = NULL, full = FALSE, ...){
              fc <- abs(fc)
              df <-  as.data.frame(deg(object, ...))
              filterOut <- .filterTable(df, direction,
                                        "log2FoldChange", fc,
                                        "padj", padj)
              df <- df %>%
                  rownames_to_column("gene") %>%
                  subset(., filterOut) %>%
                  .[order(abs(.[["log2FoldChange"]]), decreasing = TRUE),]
              if (full)
                  return(as_tibble(df))
              return(df[["gene"]])
          })

#' @rdname significants
#' @export
setMethod("significants", signature("DESeqResults"),
          function(object, padj = 0.05, fc = 0,
                   direction = NULL, full = FALSE, ...){
              fc <- abs(fc)
              df <-  as.data.frame(object)
              filterOut <- .filterTable(df, direction,
                                        "log2FoldChange", fc,
                                        "padj", padj)
              
              df <- df %>%
                  rownames_to_column("gene") %>%
                  subset(., filterOut) %>%
                  .[order(abs(.[["log2FoldChange"]]), decreasing = TRUE),]
              
              if (full)
                  return(as_tibble(df))
              return(df[["gene"]])
              
          })

#' @rdname significants
#' @export
setMethod("significants", signature("TopTags"),
          function(object, padj = 0.05, fc = 0,
                   direction = NULL, full = FALSE, ...){
              fc <- abs(fc)
              df <-  as.data.frame(object)
              filterOut <- .filterTable(df, direction,
                                        "logFC", fc,
                                        "FDR", padj)

              df <- df %>%
                  rownames_to_column("gene") %>%
                  subset(., filterOut) %>%
                  dplyr::rename(padj = FDR, log2FoldChange = logFC) %>% 
                  .[order(abs(.[["logFC"]]), decreasing = TRUE),]
              if (full)
                  return(as_tibble(df))
              return(df[["gene"]])
              
          })

.supported <- function(x){
    return(class(x)[1] %in% c("DEGSet", "DESeqResults"))
}

.get_contrast_name <- function(x){
    if (class(x)[1] == "DEGSet"){
        table = deg(x)
    }else if (class(x)[1] == "DESeqResults"){
        table = x
    }else{
        stop("Format not supported: ", class(x)[1])
    }
    contrast <- slot(table, "elementMetadata") %>% 
        .[["description"]] %>% 
        .[2]
    str_split(contrast, ": ")[[1]][2] %>%
        make.names
}

.summarise_res <- function(df, cutoff){
    first = df[,c("gene", names(df)[grepl("log2", names(df))])] %>% 
        tidyr::pivot_longer(cols = matches("_vs_"),
                            names_to = "comparison", values_to = "value") %>% 
        dplyr::mutate(comparison = gsub("log2FoldChange_", "", comparison))
    second = df[,c("gene", names(df)[grepl("padj", names(df))])] %>% 
        tidyr::pivot_longer(cols = matches("_vs_"),
                            names_to = "comparison", values_to = "value") %>% 
        dplyr::mutate(comparison = gsub("padj_", "", comparison))
    dplyr::inner_join(
        first,
        second,
        by = c("gene", "comparison"), suffix = c("_fc", "_fdr")
    ) %>% dplyr::group_by(gene) %>% 
        dplyr::filter(value_fdr < cutoff) %>%
        dplyr::summarise(log2FoldChange = value_fc[which.max(value_fc)[1L]],
                         padj = value_fdr[which.max(value_fc)[1L]]) %>%
        dplyr::right_join(df, by = "gene")
}

#' @rdname significants
#' @export
setMethod("significants", signature("list"),
          function(object, padj = 0.05, fc = 0,
                   direction = NULL, full = FALSE,
                   newFDR = FALSE, ...){
              if (!full){
                  if (newFDR){
                      selected <- lapply(object, significants, 
                                         padj = 1, fc = 0,
                                         full = TRUE) %>%
                          bind_rows() %>% 
                          dplyr::mutate(fdr = p.adjust(padj, "fdr")) %>% 
                          subset(., .filterTable(., direction,
                                                 "log2FoldChange", fc,
                                                 "fdr", padj)) %>%
                          .[["gene"]] %>% 
                          unique()
                  }else{
                      selected <- lapply(object, significants, 
                                         padj = padj, fc = fc,
                                         direction = direction) %>%
                          unlist() %>% 
                          unique()
                  }
                  return(selected)
              }else{
                  object <- object[sapply(object, .supported)]
                  if (length(object) == 0){
                      message("Only DEGSet and DESeqResults objects are used.")
                      stop("No compatible objects remained.")
                  }
                  different_names <- unique(names(object))
                  if (length(different_names) != length(object))
                      different_names <- sapply(object, .get_contrast_name) %>% 
                      unique()
                  if(length(different_names) != length(object))
                      warning("Contrast names are repeated inside the list.")
                  names(object) <- different_names
                  df <- lapply(different_names, function(x){
                      top <- significants(object[[x]], padj = padj, fc = fc,
                                          direction = direction,
                                          full = full) 
                      top_renamed <- top %>% 
                          .[, c("log2FoldChange", "padj")] %>% 
                          set_colnames(paste(colnames(.), 
                                             x,
                                             sep = "_"))
                      top_renamed[["gene"]] <- top[["gene"]]
                      tidyr::pivot_longer(top_renamed, cols = matches("_vs_"),
                                          names_to = "variable", values_to =  "value")
                      }) %>% bind_rows() %>% 
                      dplyr::distinct() %>% 
                      tidyr::spread(., "variable", "value") %>% 
                      as_tibble()
                  df <- .summarise_res(df, padj)
                  return(df)
              }
          })

#' @rdname DEGSet
#' @export
setMethod("as.DEGSet", signature("TopTags"),
          function(object, default = "raw", extras = NULL){
    name <- paste("contrast_", object[["comparison"]], collapse = "-")
    df <- as.data.frame(object) %>% set_colnames(c("log2FoldChange",
                                                "baseMean",
                                                "pvalue",
                                                "padj")) %>%
        DataFrame
    news <- lapply(extras, function(e){
        if (class(e)[1] == "TopTags")
            return(as.data.frame(e) %>% set_colnames(c("log2FoldChange",
                                                       "baseMean",
                                                       "pvalue",
                                                       "padj")) %>%
                       DataFrame)
        stop(class(e)[1], " is not a TopTags object.")
    })
    names(news) <- names(extras)
    l <- c(list(raw = df), news)
    if (!(default  %in% names(l))){
        message(default, " not found in names of list:", paste(names(l)))
        message("Set it up to raw")
        default == "raw"
    }
    dge <- new("DEGSet", l,
               default = default)
    attr(dge, "comparison") <- name
    dge
})

#' @rdname DEGSet
#' @export
setMethod("as.DEGSet", signature("data.frame"),
          function(object, contrast, default = "raw", extras = NULL){
              stopifnot(!is.null(contrast))
              cols <- c("logFC", "AveExpr", "t", "P.Value", "adj.P.Val")
              stopifnot(cols  %in% names(object))
              name <- contrast
              df <- as.data.frame(object)[,cols] %>%
                  set_colnames(c("log2FoldChange",
                                 "baseMean",
                                 "stat",
                                 "pvalue",
                                 "padj")) %>%
                  DataFrame
              news <- lapply(extras, function(e){
                  if (class(e)[1] == "data.frame"){
                      stopifnot(cols  %in% names(object))
                      return(as.data.frame(e)[,cols] %>% 
                                 set_colnames(c("log2FoldChange",
                                                "baseMean",
                                                "stat",
                                                "pvalue",
                                                "padj")) %>%
                                 DataFrame)
                  }
              })
              names(news) <- names(extras)
              l <- c(list(raw = df), news)
              if (!(default  %in% names(l))){
                  message(default, " not found in names of list:", paste(names(l)))
                  message("Set it up to raw")
                  default == "raw"
              }
              dge <- new("DEGSet", l,
                         default = "raw")
              attr(dge, "comparison") <- name
              dge
          })

#' @rdname DEGSet
#' @export
setMethod("as.DEGSet", signature("DESeqResults"),
          function(object, default = "shrunken", extras = NULL){
    name <- slot(slot(object, "elementMetadata"), "listData")[[2L]][2L]
    name <- strsplit(name, ":")[[1L]][2L] %>%
        gsub(" ", "", .)
    df <- DataFrame(object)
    news <- lapply(extras, function(e){
        if (class(e)[1] == "DESeqResults")
            return(DataFrame(e))
        stop(class(e)[1], " is not a DESeqResults object.")
    })
    names(news) <- names(extras)
    l <- c(list(raw = df), news)
    if (!(default  %in% names(l))){
        message(default, " not found in names of list:", paste(names(l)))
        message("Set it up to raw")
        default = "raw"
    }
    dge <- new("DEGSet", l,
               default = default)
    attr(dge, "comparison") <- name
    .validate(dge)
    dge
})

# setMethod("DEGSetFromList", signature("list"),
#           function(object, default = "shrunken"){
#               dgeList <- lapply(object, function(o){
#                   if (class(o) == "DESeqResults")
#                       return(DEGSetFromDESeq2())
#                   else if (class(o) == "TopTags")
#                       return(DEGSetFromDESeq2())
#                   else
#                       message(class(o), " not supported, skipping.")
#               })
#               if (!is.null(names(list)))
#                 names(dgeList) <- names(list)
#               dgeList
# })

---
title: "QC and downstream analysis for differential expression RNA-seq"
shorttitle: "Toolkit for differential expression analysis"
author: "Lorena Pantano"
date: "`r BiocStyle::doc_date()`"
package: "`r BiocStyle::pkg_ver('DEGreport')`"
abstract: >
  DEGreport package version: `r packageVersion("DEGreport")`
output:
  rmarkdown::html_document:
    highlight: pygments
    toc: true
    fig_width: 5
vignette: >
  %\VignetteIndexEntry{QC and downstream analysis for differential expression RNA-seq}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding[utf8]{inputenc}
  %\VignetteKeywords{DifferentialExpression, Visualization, RNASeq, ReportWriting}
---

```{r setup, echo=FALSE, results="hide"}
library(BiocStyle)
knitr::opts_chunk$set(tidy=FALSE,
                      dev="png",
                      message=FALSE, error=FALSE,
                      warning=TRUE)
```	


[Lorena Pantano](lorena.pantano@gmail.com)
Harvard TH Chan School of Public Health, Boston, US

```{r load-data}
library(DEGreport)
data(humanGender)
```

## General QC figures from DE analysis

We are going to do a differential expression analysis with edgeR/DESeq2.
We have an object that is coming from the edgeR package. 
It contains a gene count matrix
for 85 TSI HapMap individuals, and the gender information. With that, we are 
going to apply the `glmFit` function or `r Biocpkg("DESeq2")` to get genes differentially expressed 
between males and females.

```{r experiment}
library(DESeq2)
idx <- c(1:10, 75:85)
dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
                              colData(humanGender)[idx,], design=~group)
dds <- DESeq(dds)
res <- results(dds)
```

We need to extract the experiment design data.frame where the condition is 
Male or Female.

```{r count-design}
counts <- counts(dds, normalized = TRUE)
design <- as.data.frame(colData(dds))
```

### Size factor QC

A main assumption in library size factor calculation of edgeR and DESeq2 (and others)
is that the majority of genes remain unchanged. Plotting the distribution
of gene ratios between each gene and the average gene can show how true this is.
Not super useful for many samples because the plot becomes crowed.

```{r check-factor}
degCheckFactors(counts[, 1:6])
```


### Mean-Variance QC plots

p-value distribution gives an idea on how well you model is capturing the input data
and as well whether it could be some problem for some set of genes. In general,
you expect to have a flat distribution with peaks at 0 and 1. In this case, we add
the mean count information to check if any set of genes are enriched in any
specific p-value range.

Variation (dispersion) and average expression relationship shouldn't be a factor among
the differentially expressed genes. When plotting average mean and standard deviation,
significant genes should be randomly distributed.

In this case, it would be good to look at the ones that are totally outside the expected 
correlation.

You can put this tree plots together using `degQC`.

```{r qc}
degQC(counts, design[["group"]], pvalue = res[["pvalue"]])
```


### Covariates effect on count data

Another important analysis to do if you have covariates is to calculate
the correlation between PCs from PCA analysis to different variables you may
think are affecting the gene expression. This is a toy example of how the
function works with raw data, where clearly library size correlates with 
some of the PCs.

```{r cov}
resCov <- degCovariates(log2(counts(dds)+0.5),
                        colData(dds))
```


### Covariates correlation with metrics

Also, the correlation among covariates and metrics from the analysis can
be tested. This is useful when the study has multiple variables, like in
clinical trials. The following code will return a correlation table, and
plot the correlation heatmap for all the covariates and metrics in a table.

```{r corcov}
cor <- degCorCov(colData(dds))
names(cor)
```


### QC report

A quick HTML report can be created with `createReport` to show whether
a DE analysis is biased to a particular set of genes. It contains the output
of `degQC, `degVB and `degMB`.

Note: Nozzle.R1 is not longer available since 3.16. You need to install 
the package manually before using this function.

```{r report, eval=FALSE}
createReport(colData(dds)[["group"]], counts(dds, normalized = TRUE),
             row.names(res)[1:20], res[["pvalue"]], path = "~/Downloads")
```


## Report from DESeq2 analysis

Here, we show some useful plots for differentially expressed genes.

### Contrasts

`DEGSet` is a class to store the DE results like the one from
`results` function. `r Biocpkg("DESeq2")` offers multiple way to ask for
contrasts/coefficients. With `degComps` is easy to get multiple
results in a single object:

```{r degComps}
degs <- degComps(dds, combs = "group",
                 contrast = list("group_Male_vs_Female",
                                 c("group", "Female", "Male")))
names(degs)
```

`degs` contains 3 elements, one for each contrast/coefficient asked for.
It contains the results output in the element `raw` and the output of
`lfcShrink` in the element `shrunken`.
To obtain the results from one of them, use the method `dge`:

```{r deg}
deg(degs[[1]])
```

By default it would output the `shrunken` table always, as defined by
`degDefault`, that contains the default table to get.

To get the original results table, use the parameter as this:

```{r raw}
deg(degs[[1]], "raw", "tibble")
```

Note that the format of the output can be changed to tibble, or data.frame with
a third parameter `tidy`.

The table will be always sorted by padj.

And easy way to get significant genes is:

```{r significants}
significants(degs[[1]], fc = 0, fdr = 0.05)
```

This function can be used as well for a list of comparisons:

```{r significants-list}
significants(degs, fc = 0, fdr = 0.05)
```


And it can returns the full table for a list:

```{r significants-list-full}
significants(degs, fc = 0, fdr = 0.05, full = TRUE)
```


Since log2FoldChange are shrunken, the method for DEGSet class now can
plot these changes as follow:

```{r plotMA}
degMA(degs[[1]], diff = 2, limit = 3)
```


The blue arrows indicate how foldchange is affected by this new feature.

As well, it can plot the original MA plot:

```{r plotMA-raw}
degMA(degs[[1]], diff = 2, limit = 3, raw = TRUE)
```


or the correlation between the original log2FoldChange and the new ones:

```{r plotMA-cor}
degMA(degs[[1]], limit = 3, correlation = TRUE)
```

### Volcano plots

Volcano plot using the output of `r Biocpkg("DESeq2")`. It mainly needs data.frame with
two columns (logFC and pVal). Specific genes can be plot using the option
`plot\_text` (subset of the previous data.frame with a 3rd column to be used
to plot the gene name).

```{r deseq2-volcano}
res[["id"]] <- row.names(res)
show <- as.data.frame(res[1:10, c("log2FoldChange", "padj", "id")])
degVolcano(res[,c("log2FoldChange", "padj")], plot_text = show)
```

Note that the function is compatible with DEGset. Using
`degVolcano(degs[[1]])` is valid.

### Gene plots

Plot top genes coloring by group. Very useful for experiments with nested 
groups. `xs` can be `time` or `WT`/`KO`, and `group` can be `treated`/`untreated`.
Another classification can be added, like `batch` that will plot points 
with different shapes.

```{r deseq2-gene-plots}
degPlot(dds = dds, res = res, n = 6, xs = "group")
```

Another option for plotting genes in a wide format:

```{r deseq2-gene-plot-wide}
degPlotWide(dds, rownames(dds)[1:5], group="group")
```


### Markers plots

Markers can be used to show whether different conditions are enriched in 
different markers. For instance, in this example, Females and Males show
different total expression for chromosome X/Y markers

```{r markers}
data(geneInfo)
degSignature(humanGender, geneInfo, group = "group")
```


### Full report

If you have a DESeq2 object, you can use degResults to create a full report
with markdown code inserted,
including figures and table with top de-regulated genes, GO enrichment
analysis and heatmaps and PCA plots. If you set \Rcode{path\_results},
different files will be saved there.

```{r deseq2}
resreport <- degResults(dds = dds, name = "test", org = NULL,
                        do_go = FALSE, group = "group", xs = "group",
                        path_results = NULL)
```

### Interactive shiny-app

Browsing gene expression can help to validate results or select some gene
for downstream analysis. Run the following lines if you want to visualize
your expression values by condition:

```{r shiny, eval=FALSE}
degObj(counts, design, "degObj.rda")
library(shiny)
shiny::runGitHub("lpantano/shiny", subdir="expression")
```

## Detect patterns of expression

In this section, we show how to detect pattern of expression. Mainly useful when
data is a time course experiment. `degPatterns` needs a expression
matrix, the design experiment and the column used to group samples.

```{r pattern}
ma = assay(rlog(dds))[row.names(res)[1:100],]
res <- degPatterns(ma, design, time = "group")
```

## Useful functions

This section shows some useful functions during DEG analysis.

### Filter genes by group

`degFilter` helps to filter genes with a minimum read count by group.

```{r filter, results="asis"}
cat("gene in original count matrix: 1000")
filter_count <- degFilter(counts(dds),
                          design, "group",
                          min=1, minreads = 50)
cat("gene in final count matrix", nrow(filter_count))
```

### Generate colors for metadata variables

This functions allows you to create colors for metadata columns
to be used as annotation for columns in a heatmap figure.

```{r degColors}
library(ComplexHeatmap)
th <- HeatmapAnnotation(df = colData(dds),
                        col = degColors(colData(dds), TRUE))
Heatmap(log2(counts(dds) + 0.5)[1:10,],
        top_annotation = th)

library(pheatmap)
pheatmap(log2(counts(dds) + 0.5)[1:10,], 
         annotation_col = as.data.frame(colData(dds))[,1:4],
         annotation_colors = degColors(colData(dds)[1:4],
                                       con_values = c("white",
                                                      "red")
                                       )
         )

```


# Session info

```{r sessionInfo}
sessionInfo()
```

%\VignetteIndexEntry{DEGreport}
%\VignetteKeywords{DifferentialExpression, Visualization, RNASeq, ReportWriting}
%\VignetteEngine{knitr::knitr}

\documentclass{article}
\usepackage[utf8]{inputenc}

<<knitr, echo=FALSE, results="hide">>=
library("knitr")
opts_chunk$set(tidy=FALSE,
               fig.width=6,fig.height=5,
               message=FALSE)
@ 


<<style, eval=TRUE, echo=FALSE, results="asis">>=
BiocStyle::latex()
@

\title{DEGreport }

\author{Lorena Pantano\thanks{\email{lorena.pantano@gmail.com}}}
\affil{Harvard TH Chan School of Public Health, Boston, US}

\begin{document}

\maketitle

\packageVersion{\Sexpr{BiocStyle::pkg_ver("DEGreport")}}
\date{Modified: 2 July, 2016. Compiled: \today}

\tableofcontents

\newpage

<<package-load,message=FALSE>>=
library(DEGreport)
data(humanGender)
@

\section{General QC figures from DE analysis}

We are going to do a differential expression analysis with edgeR/DESeq2.
We have an object that is coming from the edgeR package. 
It contains a gene count matrix
for 85 TSI HapMap individuals, and the gender information. With that, we are 
going to apply the `glmFit` function or `DESeq2` to get genes differentially expressed 
between males and females.

<<chunk-1>>=
library(DESeq2)
idx <- c(1:10, 75:85)
dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
                              colData(humanGender)[idx,], design=~group)
dds <- DESeq(dds)
res <- results(dds)
@

We need to extract the experiment design data.frame where the condition is 
Male or Female.

<<chunk-2>>=
counts <- counts(dds, normalized = TRUE)
design <- as.data.frame(colData(dds))
@

\subsection{Size factor QC}

A main assumption in library size factor calculation of edgeR and DESeq2 (and others)
is that the majority of genes remain unchanged. Plotting the distribution
of gene ratios between each gene and the average gene can show how true this is.
Not super useful for many samples because the plot becomes crowed.

<<chunk-size-factor>>=
degCheckFactors(counts[, 1:6])
@


\subsection{Mean-Variance QC plots}

p-value distribution gives an idea on how well you model is capturing the input data
and as well whether it could be some problem for some set of genes. In general,
you expect to have a flat distribution with peaks at 0 and 1. In this case, we add
the mean count information to check if any set of genes are enriched in any
specific p-value range.

Variation (dispersion) and average expression relationship shouldn't be a factor among
the differentially expressed genes. When plotting average mean and standard deviation,
significant genes should be randomly distributed.

In this case, it would be good to look at the ones that are totally outside the expected 
correlation.

You can put this tree plots together using \Rfunction{degQC}.

<<chunk-qc>>=
degQC(counts, design[["group"]], pvalue = res[["pvalue"]])
@


\subsection{Covariates effect on count data}

Another important analysis to do if you have covariates is to calculate
the correlation between PCs from PCA analysis to different variables you may
think are affecting the gene expression. This is a toy example of how the
function works with raw data, where clearly library size correlates with 
some of the PCs.

<<chunk-covariates>>=
resCov <- degCovariates(log2(counts(dds)+0.5),
                        colData(dds))
@


\subsection{Covariates correlation with metrics}

Also, the correlation among covariates and metrics from the analysis can
be tested. This is useful when the study has multiple variables, like in
clinical trials. The following code will return a correlation table, and
plot the correlation heatmap for all the covariates and metrics in a table.

<<chunk-cor-covariates>>=
cor <- degCorCov(colData(dds))
names(cor)
@


\subsection{QC report}

A quick HTML report can be created with \Rfunction{createReport} to show whether
a DE analysis is biased to a particular set of genes. It contains the output
of \Rfunction{degQC}, \Rfunction{degVB} and \Rfunction{degMB}.

<<chunk-qc-report, eval=FALSE>>=
createReport(colData(dds)[["group"]], counts(dds, normalized = TRUE),
             row.names(res)[1:20], res[["pvalue"]], path = "~/Downloads")
@


\section{Report from DESeq2 analysis}

Here, we show some useful plots for differentially expressed genes.

\subsection{Contrasts}


\Rclass{DEGSet} is a class to store the DE results like the one from
\Rfunction{results} function. \Biocpkg{DESeq2} offers multiple way to ask for
contrasts/coefficients. With \Rfunction{degComps} is easy to get multiple
results in a single object:

<<chunk-degComps>>=
degs <- degComps(dds, combs = "group",
                 contrast = list("group_Male_vs_Female",
                                 c("group", "Female", "Male")))
names(degs)
@

\Rfunction{degs} contains 3 elements, one for each contrast/coefficient asked for.
It contains the results output in the element \Rcode{raw} and the output of
\Rfunction{lfcShrink} in the element \Rfunction{shrunken}.
To obtain the results from one of them, use the method \Rfunction{dge}:

<<chunk-deg>>=
deg(degs[[1]])
@

By default it would output the \Rcode{shrunken} table always, as defined by
\Rfunction{degDefault}, that contains the default table to get.

To get the original results table, use the parameter as this:

<<chunk-deg-raw>>=
deg(degs[[1]], "raw", "tibble")
@

Note that the format of the output can be changed to tibble, or data.frame with
a third parameter \Rcode{tidy}.

The table will be always sorted by padj.

And easy way to get significant genes is:

<<chunk-significants>>=
significants(degs[[1]], fc = 0, fdr = 0.05)
@

This function can be used as well for a list of comparisons:

<<chunk-significants-list>>=
significants(degs, fc = 0, fdr = 0.05)
@

And it can returns the full table for a list:

<<chunk-significants-list-full>>=
significants(degs, fc = 0, fdr = 0.05, full = TRUE)
@

Since log2FoldChange are shrunken, the method for DEGSet class now can
plot these changes as follow:

<<chunk-plotMA>>=
plotMA(degs[[1]], diff = 2, limit = 3)
@

The blue arrows indicate how foldchange is affected by this new feature.

As well, it can plot the original MA plot:

<<chunk-plotMA-raw>>=
plotMA(degs[[1]], diff = 2, limit = 3, raw = TRUE)
@

or the correlation between the original log2FoldChange and the new ones:

<<chunk-plotMA-cor>>=
plotMA(degs[[1]], limit = 3, correlation = TRUE)
@

\subsection{Volcano plots}

Volcano plot using the output of \Biocpkg{DESeq2}. It mainly needs data.frame with
two columns (logFC and pVal). Specific genes can be plot using the option
\Rcode{plot\_text} (subset of the previous data.frame with a 3rd column to be used
to plot the gene name).

<<deseq2-volcano>>=
res[["id"]] <- row.names(res)
# show <- as.data.frame(res[1:10, c("log2FoldChange", "padj", "id")])
degVolcano(res[,c("log2FoldChange", "padj")])
@

Note that the function is compatible with DEGset. Using
\Rcode{degVolcano(degs[[1]])} is valid.

\subsection{Gene plots}

Plot top genes coloring by group. Very useful for experiments with nested 
groups. `xs` can be `time` or `WT`/`KO`, and `group` can be `treated`/`untreated`.
Another classification can be added, like `batch` that will plot points 
with different shapes.

<<deseq2-gene-plots>>=
degPlot(dds = dds, res = res, n = 6, xs = "group")
@

Another option for plotting genes in a wide format:

<<deseq2-gene-plot-wide>>=
degPlotWide(dds, rownames(dds)[1:5], group="group")
@


\subsection{Markers plots}

Markers can be used to show whether different conditions are enriched in 
different markers. For instance, in this example, Females and Males show
different total expression for chromosome X/Y markers

<<markers>>=
data(geneInfo)
degSignature(humanGender, geneInfo, group = "group")
@


\subsection{Full report}

If you have a DESeq2 object, you can use degResults to create a full report
with markdown code inserted,
including figures and table with top de-regulated genes, GO enrichment
analysis and heatmaps and PCA plots. If you set \Rcode{path\_results},
different files will be saved there.

<<deseq2>>=
resreport <- degResults(dds = dds, name = "test", org = NULL,
                        do_go = FALSE, group = "group", xs = "group",
                        path_results = NULL)
@

\subsection{Interactive shiny-app}

Browsing gene expression can help to validate results or select some gene
for downstream analysis. Run the following lines if you want to visualize
your expression values by condition:

<<chunk-shiny, eval=FALSE>>=
degObj(counts, design, "degObj.rda")
library(shiny)
shiny::runGitHub("lpantano/shiny", subdir="expression")
@

\section{Detect patterns of expression}

In this section, we show how to detect pattern of expression. Mainly useful when
data is a time course experiment. \Rfunction{degPatterns} needs a expression
matrix, the design experiment and the column used to group samples.

<<pattern>>=
ma = assay(rlog(dds))[row.names(res)[1:100],]
res <- degPatterns(ma, design, time = "group")
@

\section{Useful functions}

This section shows some useful functions during DEG analysis.

\Rfunction{degFilter} helps to filter genes with a minimum read count by group.

<<chunk-filter>>=
cat("gene in original count matrix: 1000")
filter_count <- degFilter(counts(dds),
                          design, "group",
                          min=1, minreads = 50)
cat("gene in final count matrix", nrow(filter_count))
@



\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genePlots.R
\name{degPlotWide}
\alias{degPlotWide}
\title{Plot selected genes on a wide format}
\usage{
degPlotWide(counts, genes, group, metadata = NULL, batch = NULL)
}
\arguments{
\item{counts}{\link[DESeq2:DESeqDataSet]{DESeq2::DESeqDataSet} object or expression matrix}

\item{genes}{character genes to plot.}

\item{group}{character, colname in colData to color points and add different
lines for each level}

\item{metadata}{data.frame, information for each sample. Not needed if
\link[DESeq2:DESeqDataSet]{DESeq2::DESeqDataSet} given as counts.}

\item{batch}{character, colname in colData to shape points, normally used by
batch effect visualization}
}
\value{
ggplot showing the expresison of the genes on the x
axis
}
\description{
Plot selected genes on a wide format
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dse <- DESeq(dse)
degPlotWide(dse, rownames(dse)[1:10], group = "group")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{degSummary}
\alias{degSummary}
\title{Print Summary Statistics of Alpha Level Cutoffs}
\usage{
degSummary(
  object,
  alpha = c(0.1, 0.05, 0.01),
  contrast = NULL,
  caption = "",
  kable = FALSE
)
}
\arguments{
\item{object}{Can be \link{DEGSet} or \link{DESeqDataSet} or \link{DESeqResults}.}

\item{alpha}{Numeric vector of desired alpha cutoffs.}

\item{contrast}{Character vector to use with \code{\link[=results]{results()}} function.}

\item{caption}{Character vector to add as caption to the table.}

\item{kable}{Whether return a \code{\link[knitr:kable]{knitr::kable()}} output. Default is data.frame.}
}
\value{
\link{data.frame} or \code{\link[knitr:kable]{knitr::kable()}}.
}
\description{
Print Summary Statistics of Alpha Level Cutoffs
}
\examples{
library(DESeq2)
data(humanGender)
idx <- c(1:5, 75:80)
counts <- assays(humanGender)[[1]]
dse <- DESeqDataSetFromMatrix(counts[1:1000, idx],
                              colData(humanGender)[idx,],
                              design = ~group)
dse <- DESeq(dse)
res1 <- results(dse)
res2 <- degComps(dse, contrast = c("group_Male_vs_Female"))
degSummary(dse, contrast = "group_Male_vs_Female")
degSummary(res1)
degSummary(res1, kable = TRUE)
degSummary(res2[[1]])
}
\references{
\itemize{
\item original idea of multiple alpha values and code syntax
from  Michael Steinbaugh.
}
}
\author{
Lorena Pantano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covariate.R
\name{degCorCov}
\alias{degCorCov}
\title{Calculate the correlation relationshipt among all covariates
in the metadata table}
\usage{
degCorCov(metadata, fdr = 0.05, use_pval = FALSE, ...)
}
\arguments{
\item{metadata}{data.frame with samples metadata.}

\item{fdr}{numeric value to use as cutoff to determine
the minimum fdr to consider significant correlations
between pcs and covariates.}

\item{use_pval}{boolean to indicate to use p-value instead of FDR to
hide non-significant correlation.}

\item{...}{Parameters to pass to \code{\link[ComplexHeatmap:Heatmap]{ComplexHeatmap::Heatmap()}}.}
}
\value{
: list:
a) cor, data.frame with pair-wise correlations, pvalues, FDR
b) corMat, data.frame with correlation matrix
c) fdrMat, data.frame with FDR matrix
b) plot, Heatmap plot of correlation matrix
}
\description{
This function will calculate the correlation among
all columns in the metadata
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
cor <- degCorCov(colData(dse))
}
\author{
: Lorena Pantano, Kenneth Daily and Thanneer Malai Perumal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/AllMethods.R
\name{degDefault}
\alias{degDefault}
\alias{degCorrect}
\alias{degDefault,DEGSet-method}
\alias{degCorrect,DEGSet-method}
\title{Method to get the default table to use.}
\usage{
degDefault(object)

degCorrect(object, fdr)

\S4method{degDefault}{DEGSet}(object)

\S4method{degCorrect}{DEGSet}(object, fdr)
}
\arguments{
\item{object}{\link{DEGSet}}

\item{fdr}{It can be \code{fdr-stat}, \code{fdr-pvalue}, vector of new padj}
}
\description{
It can accept a list of new padj values matching the
same dimmensions than the current vector.
}
\examples{
library(DESeq2)
library(dplyr)
dds <- makeExampleDESeqDataSet(betaSD=1)
colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
design(dds) <-  ~ condition + treatment
dds <- DESeq(dds)
res <- degComps(dds, contrast = list("treatment_B_vs_A"))
}
\author{
Lorena Pantano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/volcanoPlot.R
\name{degVolcano}
\alias{degVolcano}
\title{Create volcano plot from log2FC and adjusted pvalues data frame}
\usage{
degVolcano(
  stats,
  side = "both",
  title = "Volcano Plot with Marginal Distributions",
  pval.cutoff = 0.05,
  lfc.cutoff = 1,
  shade.colour = "orange",
  shade.alpha = 0.25,
  point.colour = "gray",
  point.alpha = 0.75,
  point.outline.colour = "darkgray",
  line.colour = "gray",
  plot_text = NULL
)
}
\arguments{
\item{stats}{data.frame with two columns: logFC and Adjusted.Pvalue}

\item{side}{plot UP, DOWN or BOTH de-regulated points}

\item{title}{title for the figure}

\item{pval.cutoff}{cutoff for the adjusted pvalue. Default 0.05}

\item{lfc.cutoff}{cutoff for the log2FC. Default 1}

\item{shade.colour}{background color. Default orange.}

\item{shade.alpha}{transparency value. Default 0.25}

\item{point.colour}{colours for points. Default gray}

\item{point.alpha}{transparency for points. Default 0.75}

\item{point.outline.colour}{Default darkgray}

\item{line.colour}{Defaul gray}

\item{plot_text}{data.frame with three columns: logFC, Pvalue, Gene name}
}
\value{
The function will plot volcano plot together
with density of the fold change and p-values on the top and the
right side of the volcano plot.
}
\description{
Create volcano plot from log2FC and adjusted pvalues data frame
}
\details{
This function was mainly developed by @jnhutchinson.
}
\examples{
library(DESeq2)
dds <- makeExampleDESeqDataSet(betaSD = 1)
dds <- DESeq(dds)
stats <- results(dds)[,c("log2FoldChange", "padj")]
stats[["name"]] <- row.names(stats)
degVolcano(stats, plot_text = stats[1:10,])
}
\author{
Lorena Pantano, John Hutchinson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_cor.R
\name{geom_cor}
\alias{geom_cor}
\title{Add correlation and p-value to a \link{ggplot2} plot}
\usage{
geom_cor(
  mapping = NULL,
  data = NULL,
  method = "spearman",
  xpos = NULL,
  ypos = NULL,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[=aes]{aes()}} or
\code{\link[=aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[=ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[=fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame.}, and
will be used as the layer data.}

\item{method}{Method to calculate the correlation. Values are
passed to \code{\link[=cor.test]{cor.test()}}. (Spearman, Pearson, Kendall).}

\item{xpos}{Locate text at that position on the x axis.}

\item{ypos}{Locate text at that position on the y axis.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[=borders]{borders()}}.}

\item{...}{other arguments passed on to \code{\link[=layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{color = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
\code{geom_cor} will add the correlatin, method and p-value to the plot
automatically guessing the position if nothing else specidfied.
family font, size and colour can be used to change the format.
}
\details{
It was integrated after reading this tutorial to extend
ggplot2 \href{https://cran.r-project.org/web/packages/ggplot2/vignettes/extending-ggplot2.html}{layers}
}
\examples{
data(humanGender)
library(SummarizedExperiment)
library(ggplot2)
ggplot(as.data.frame(assay(humanGender)[1:1000,]),
       aes(x = NA20502, y = NA20504)) +
  geom_point() +
  ylim(0,1.1e5) +
  geom_cor(method = "kendall", ypos = 1e5) 
}
\seealso{
\code{\link[ggplot2:layer]{ggplot2::layer()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degFilter}
\alias{degFilter}
\title{Filter genes by group}
\usage{
degFilter(counts, metadata, group, min = 0.8, minreads = 0)
}
\arguments{
\item{counts}{Matrix with expression data, columns are samples
and rows are genes or other feature.}

\item{metadata}{Data.frame with information about
each column in counts matrix. Rownames should match
\code{colnames(counts)}.}

\item{group}{Character column in metadata used to
group samples and applied the cutoff.}

\item{min}{Percentage value indicating the minimum
number of samples in each group that should have
more than 0 in count matrix.}

\item{minreads}{Integer minimum number of reads to consider
a feature expressed.}
}
\value{
count \code{matrix} after filtering genes (features)
with not enough expression in any group.
}
\description{
This function will keep only rows that have a minimum counts of
1 at least in a \code{min} number of samples (default 80\%).
}
\examples{
data(humanGender)
library(SummarizedExperiment)
idx <- c(1:10, 75:85)
c <- degFilter(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], "group", min=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/AllMethods.R
\name{significants}
\alias{significants}
\alias{significants,DEGSet-method}
\alias{significants,DESeqResults-method}
\alias{significants,TopTags-method}
\alias{significants,list-method}
\title{Method to get the significant genes}
\usage{
significants(object, padj = 0.05, fc = 0, direction = NULL, full = FALSE, ...)

\S4method{significants}{DEGSet}(object, padj = 0.05, fc = 0, direction = NULL, full = FALSE, ...)

\S4method{significants}{DESeqResults}(object, padj = 0.05, fc = 0, direction = NULL, full = FALSE, ...)

\S4method{significants}{TopTags}(object, padj = 0.05, fc = 0, direction = NULL, full = FALSE, ...)

\S4method{significants}{list}(
  object,
  padj = 0.05,
  fc = 0,
  direction = NULL,
  full = FALSE,
  newFDR = FALSE,
  ...
)
}
\arguments{
\item{object}{\link{DEGSet}}

\item{padj}{Cutoff for the FDR column.}

\item{fc}{Cutoff for the log2FC column.}

\item{direction}{Whether to take down/up/ignore. Valid arguments are
down, up and NULL.}

\item{full}{Whether to return full table or not.}

\item{...}{Passed to \link{deg}. Default: value = NULL.
Value can be 'raw', 'shrunken'.}

\item{newFDR}{Whether to recalculate the FDR or not.
See https://support.bioconductor.org/p/104059/#104072.
Only used when a list is giving to the method.}
}
\value{
a \link[dplyr:tbl_df]{dplyr::tbl_df} data frame. \code{gene} column has the feature name.
In the case of using this method with the results from \link{degComps},
\code{log2FoldChange} has the higher foldChange from the comparisons, and
\code{padj} has the padj associated to the previous column. Then, there is
two columns for each comparison, one for the log2FoldChange and another
for the padj.
}
\description{
Function to get the features that are significant
according to some thresholds from a \link{DEGSet},
\link[DESeq2:DESeqResults]{DESeq2::DESeqResults} and \link[edgeR:topTags]{edgeR::topTags}.
}
\examples{
library(DESeq2)
dds <- makeExampleDESeqDataSet(betaSD=1)
colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
  design(dds) <-  ~ condition + treatment
dds <- DESeq(dds)
res <- degComps(dds, contrast = list("treatment_B_vs_A",
                                     c("condition", "A", "B")))
significants(res, full = TRUE)
# significants(res, full = TRUE, padj = 1) # all genes
}
\author{
Lorena Pantano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\docType{methods}
\name{degMA}
\alias{degMA}
\title{MA-plot from base means and log fold changes}
\usage{
degMA(
  results,
  title = NULL,
  label_points = NULL,
  label_column = "symbol",
  limit = NULL,
  diff = 5,
  raw = FALSE,
  correlation = FALSE
)
}
\arguments{
\item{results}{\link{DEGSet} class.}

\item{title}{\emph{Optional}. Plot title.}

\item{label_points}{Optionally label these particular points.}

\item{label_column}{Match label_points to this column in the results.}

\item{limit}{Absolute maximum to plot on the log2FoldChange.}

\item{diff}{Minimum difference between logFoldChange before and
after shrinking.}

\item{raw}{Whether to plot just the unshrunken log2FC.}

\item{correlation}{Whether to plot the correlation of the two logFCs.}
}
\value{
MA-plot \link{ggplot}.
}
\description{
MA-plot addaptation to show the shrinking effect.
}
\examples{
library(DESeq2)
dds <- makeExampleDESeqDataSet(betaSD=1)
dds <- DESeq(dds)
res <- degComps(dds, contrast = list("condition_B_vs_A"))
degMA(res)
}
\author{
Victor Barrera

Rory Kirchner

Lorena Pantano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degVB}
\alias{degVB}
\title{Distribution of the standard desviation of
DE genes compared to the background}
\usage{
degVB(tags, group, counts, pop = 400)
}
\arguments{
\item{tags}{List of genes that are DE.}

\item{group}{Character vector with group name for each sample in the
same order than counts column names.}

\item{counts}{matrix with counts for each samples and each gene.
Should be same length than pvalues vector.}

\item{pop}{Number of random samples taken for background comparison.}
}
\value{
ggplot2 object
}
\description{
Distribution of the standard desviation of
DE genes compared to the background
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dds <- DESeq(dds)
res <- results(dds)
degVB(row.names(res)[1:20], colData(dds)[["group"]],
  counts(dds, normalized = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geneInfo.R
\docType{data}
\name{geneInfo}
\alias{geneInfo}
\title{data.frame with chromose information for each gene}
\format{
data.frame
}
\source{
biomart
}
\usage{
data(geneInfo)
}
\description{
data.frame with chromose information for each gene
}
\author{
Lorena Pantano, 2014-08-14
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covariate.R
\name{degCovariates}
\alias{degCovariates}
\title{Find correlation between pcs and covariates}
\usage{
degCovariates(
  counts,
  metadata,
  fdr = 0.1,
  scale = FALSE,
  minPC = 5,
  correlation = "kendall",
  addCovDen = TRUE,
  legacy = FALSE,
  smart = TRUE,
  method = "lm",
  plot = TRUE
)
}
\arguments{
\item{counts}{normalized counts matrix}

\item{metadata}{data.frame with samples metadata.}

\item{fdr}{numeric value to use as cutoff to determine
the minimum fdr to consider significant correlations
between pcs and covariates.}

\item{scale}{boolean to determine wether counts matrix should be
scaled for pca. default FALSE.}

\item{minPC}{numeric value that will be used as cutoff
to select only pcs that explain more variability than this.}

\item{correlation}{character determining the method for the
correlation between pcs and covariates.}

\item{addCovDen}{boolean. Whether to add the covariates
dendograme to the plot to see covariates relationship.
It will show \code{\link[=degCorCov]{degCorCov()}} dendograme on top of the columns of
the heatmap.}

\item{legacy}{boolean. Whether to plot the legacy version.}

\item{smart}{boolean. Whether to avoid normalization of the
numeric covariates when calculating importance. This is not
used if \code{legacy = TRUE}. See @details for more information.}

\item{method}{character. Whether to use \code{lm} to
calculate the significance of the variable during reduction
step. See @details for more information.}

\item{plot}{Whether to plot or not the correlation matrix.}
}
\value{
: list:
\itemize{
\item plot, heatmap showing the signifcance of the variables.
\item corMatrix, correlation, p-value, FDR values
for each covariate and PCA pais
\item pcsMatrix: PCs loading for each sample
\item scatterPlot: plot for each significant covariate and
the PC values.
\item significants: contains the significant covariates
using a linear model to predict the coefficient
of covariates that have some color in the plot.
All the significant covariates from the liner model analysis
are returned.
}
}
\description{
This function will calculate the pcs using prcomp function,
and correlate categorical and numerical variables from
metadata. The size of the dots indicates the importance of the
metadata, for instance, when the range of the values is pretty
small (from 0.001 to 0.002 in ribosimal content),
the correlation results is not important. If black stroke lines
are shown, the correlation analysis has a FDR < 0.05 for that
variable and PC.
Only significant variables according the linear model are colored.
See details to know
how this is calculated.
}
\details{
This method is adapeted from Daily et al 2017 article.
Principal components from PCA analysis are correlated with
covariates metadata. Factors are transformed to numeric variables.
Correlation is measured by \code{cor.test} function with Kendall method
by default.

The size of the dot, or importance, indicates the importance of
the covariate based on the range of the values. Covariates
where the range is very small (like a \% of mapped reads that
varies between 0.001 to 0.002) will have a very small size (0.1*max_size).
The maximum value is set to 5 units.
To get to importance, each covariate is normalized using this
equation: \code{1 - min(v/max(v))},
and the minimum and maximum values are set to
0.01 and 1 respectively. For instance, 0.5 would mean there is at least
50\% of difference between the minimum value and the maximum value.
Categorical variables are plot using the maximum size always, since
it is not possible to estimate the variability. By default, it
won't do \code{v/max(v)} if the values are already between 0-1 or
0-100 (already normalized values as rates and percentages).
If you want to ignore the importance, use \code{legacy = TRUE}.

Finally, a linear model is used to calculate the significance
of the covariates effect on the PCs. For that, this function
uses \code{lm} to regress the data and uses the p-value calculated by
each variable in the model to define significance (pvalue < 0.05).
Variables with a black stroke
are significant after this step. Variables with grey stroke
are significant at the first pass considering p.value < 0.05
for the correlation analysis.
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
res <- degCovariates(log2(counts(dse)+0.5), colData(dse))
res <- degCovariates(log2(counts(dse)+0.5),
  colData(dse), legacy = TRUE)
res$plot
res$scatterPlot[[1]]
}
\references{
Daily, K. et al.  Molecular, phenotypic, and sample-associated data to describe pluripotent stem cell lines and derivatives. Sci Data 4, 170030 (2017).
}
\author{
: Lorena Pantano, Victor Barrera,
Kenneth Daily and Thanneer Malai Perumal
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/results.R
\name{degComps}
\alias{degComps}
\title{Automatize the use of \code{results()} for multiple comparisons}
\usage{
degComps(
  dds,
  combs = NULL,
  contrast = NULL,
  alpha = 0.05,
  skip = FALSE,
  type = "normal",
  pairs = FALSE,
  fdr = "default"
)
}
\arguments{
\item{dds}{\link[DESeq2:DESeqDataSet]{DESeq2::DESeqDataSet} obcject.}

\item{combs}{Optional vector indicating the coefficients or columns
fom \code{colData(dds)} to create group comparisons.}

\item{contrast}{Optional vector to specify contrast. See \code{\link[DESeq2:results]{DESeq2::results()}}.}

\item{alpha}{Numeric value used in independent filtering in \code{\link[DESeq2:results]{DESeq2::results()}}.}

\item{skip}{Boolean to indicate whether skip shrinkage. For instance
when it comes from LRT method.}

\item{type}{Type of shrinkage estimator. See \code{\link[DESeq2:lfcShrink]{DESeq2::lfcShrink()}}.}

\item{pairs}{Boolean to indicate whether create all comparisons or only
use the coefficient already created from \code{DESeq2::resultsNames()}.}

\item{fdr}{type of fdr correction. \code{default} is FDR value,
\code{lfdr-stat} is
for local FDR using the statistics of the test,
\code{lfdr-pvalue} is for local FDR using the p-value of the test. fdrtools
needs to be installed and loaded by the user}
}
\value{
\link{DEGSet} with unSrunken and Srunken results.
}
\description{
This function will extract the output of \code{\link[DESeq2:results]{DESeq2::results()}}
and \code{\link[DESeq2:lfcShrink]{DESeq2::lfcShrink()}} for multiple comparison using:
}
\details{
\itemize{
\item coefficients
\item contrast
\item Multiple columns in \code{colData} that match coefficients
\item Multiple columns in \code{colData} to create all possible
contrasts
}
}
\examples{
library(DESeq2)
dds <- makeExampleDESeqDataSet(betaSD=1)
colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
  design(dds) <-  ~ condition + treatment
dds <- DESeq(dds)
res <- degComps(dds, combs = c("condition", 2),
                contrast = list("treatment_B_vs_A", c("condition", "A", "B")))
# library(fdrtools)
#res <- degComps(dds,contrast = list("treatment_B_vs_A"),
#                fdr="lfdr-stat")
}
\author{
Lorena Pantano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R, R/AllGenerics.R, R/AllMethods.R
\name{DEGSet}
\alias{DEGSet}
\alias{DEGSet-class}
\alias{as.DEGSet}
\alias{as.DEGSet,TopTags-method}
\alias{as.DEGSet,data.frame-method}
\alias{as.DEGSet,DESeqResults-method}
\title{DEGSet}
\usage{
DEGSet(resList, default)

DEGSet(resList, default)

as.DEGSet(object, ...)

\S4method{as.DEGSet}{TopTags}(object, default = "raw", extras = NULL)

\S4method{as.DEGSet}{data.frame}(object, contrast, default = "raw", extras = NULL)

\S4method{as.DEGSet}{DESeqResults}(object, default = "shrunken", extras = NULL)
}
\arguments{
\item{resList}{List with results as elements containing log2FoldChange,
pvalues and padj as column. Rownames should be feature names. Elements
should have names.}

\item{default}{The name of the element to use by default.}

\item{object}{Different objects to be transformed to DEGSet when using \code{as.DEGSet}.}

\item{...}{Optional parameters of the generic.}

\item{extras}{List of extra tables related to the same comparison when using \code{as.DEGSet}.}

\item{contrast}{To name the comparison when using \code{as.DEGSet}.}
}
\description{
S4 class to store data from differentially expression analysis.
It should be compatible with different package and stores the information
in a way the methods will work with all of them.
}
\details{
For now supporting only \code{\link[DESeq2:results]{DESeq2::results()}} output.
Use constructor \code{\link[=degComps]{degComps()}} to create the object.

The list will contain one element for each comparison done.
Each element has the following structure:
\itemize{
\item DEG table
\item Optional table with shrunk Fold Change when it has been done.
}

To access the raw table use \code{deg(dgs, "raw")}, to access the
shrunken table use \code{deg(dgs, "shrunken")} or just \code{deg(dgs)}.
}
\examples{
library(DESeq2)
library(edgeR)
library(limma)
dds <- makeExampleDESeqDataSet(betaSD = 1)
colData(dds)[["treatment"]] <- sample(colData(dds)[["condition"]], 12)
design(dds) <-  ~ condition + treatment
dds <- DESeq(dds)
res <- degComps(dds, combs = c("condition"))
deg(res)
deg(res, tidy = "tibble")
# From edgeR
dge <- DGEList(counts=counts(dds), group=colData(dds)[["treatment"]])
dge <- estimateCommonDisp(dge)
res <- as.DEGSet(topTags(exactTest(dge)))
# From limma
v <- voom(counts(dds), model.matrix(~treatment, colData(dds)), plot=FALSE)
fit <- lmFit(v)
fit <- eBayes(fit, robust=TRUE)
res <- as.DEGSet(topTable(fit, n = "Inf"), "A_vs_B")
}
\author{
Lorena Pantano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genePlots.R
\name{degPlot}
\alias{degPlot}
\title{Plot top genes allowing more variables to color and shape points}
\usage{
degPlot(
  dds,
  xs,
  res = NULL,
  n = 9,
  genes = NULL,
  group = NULL,
  batch = NULL,
  metadata = NULL,
  ann = c("geneID", "symbol"),
  slot = 1L,
  log2 = TRUE,
  xsLab = xs,
  ysLab = "abundance",
  color = "black",
  groupLab = group,
  batchLab = batch,
  sizePoint = 1
)
}
\arguments{
\item{dds}{\link[DESeq2:DESeqDataSet]{DESeq2::DESeqDataSet} object or SummarizedExperiment
or Matrix or data.frame. In case of a DESeqDataSet object, always
the normalized expression will be used
from \code{counts(dds, normalized = TRUE)}.}

\item{xs}{Character, colname in colData that will be used as X-axes.}

\item{res}{\link[DESeq2:DESeqResults]{DESeq2::DESeqResults} object.}

\item{n}{Integer number of genes to plot from the \code{res} object. It will
take the top N using padj values to order the table.}

\item{genes}{Character of gene names matching rownames of count data.}

\item{group}{Character, colname in colData to color points and add different
lines for each level.}

\item{batch}{Character, colname in colData to shape points, normally used by
batch effect visualization.}

\item{metadata}{Metadata in case dds is a matrix.}

\item{ann}{Columns in rowData (if available) used to print gene names. First
element in the vector is the column name in rowData that matches the
row.names of the \code{dds} or \code{count} object. Second element in the vector
is the column name in rowData that it will be used as the title for each
gene or feature figure.}

\item{slot}{Name of the slot to use to get count data.}

\item{log2}{Whether to apply or not log2 transformation.}

\item{xsLab}{Character, alternative label for x-axis (default: same as xs)}

\item{ysLab}{Character, alternative label for y-axis..}

\item{color}{Color to use to plot groups. It can be one color, or a palette
compatible with \code{ggplot2::scale_color_brewer()}.}

\item{groupLab}{Character, alternative label for group (default: same as group).}

\item{batchLab}{Character, alternative label for batch (default: same as batch).}

\item{sizePoint}{Integer, indicates the size of the plotted points (default 1).}
}
\value{
ggplot showing the expresison of the genes
}
\description{
Plot top genes allowing more variables to color and shape points
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dse <- DESeq(dse)
degPlot(dse, genes = rownames(dse)[1:10], xs = "group")
degPlot(dse, genes = rownames(dse)[1:10], xs = "group", color = "orange")
degPlot(dse, genes = rownames(dse)[1:10], xs = "group", group = "group",
        color = "Accent")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{degMDS}
\alias{degMDS}
\title{Plot MDS from normalized count data}
\usage{
degMDS(counts, condition = NULL, k = 2, d = "euclidian", xi = 1, yi = 2)
}
\arguments{
\item{counts}{matrix samples in columns, features in rows}

\item{condition}{vector define groups of samples in counts.
It has to be same order than the count matrix for columns.}

\item{k}{integer number of dimensions to get}

\item{d}{type of distance to use, c("euclidian", "cor").}

\item{xi}{number of component to plot in x-axis}

\item{yi}{number of component to plot in y-axis}
}
\value{
ggplot2 object
}
\description{
Uses cmdscale to get multidimensional scaling of data matrix,
and plot the samples with ggplot2.
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
degMDS(counts(dse), condition = colData(dse)[["group"]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{degPatterns}
\alias{degPatterns}
\title{Make groups of genes using expression profile.}
\usage{
degPatterns(
  ma,
  metadata,
  minc = 15,
  summarize = "merge",
  time = "time",
  col = NULL,
  consensusCluster = FALSE,
  reduce = FALSE,
  cutoff = 0.7,
  scale = TRUE,
  pattern = NULL,
  groupDifference = NULL,
  eachStep = FALSE,
  plot = TRUE,
  fixy = NULL,
  nClusters = NULL,
  skipDendrogram = TRUE
)
}
\arguments{
\item{ma}{log2 normalized count matrix}

\item{metadata}{data frame with sample information. Rownames
should match \code{ma} column names
row number should be the same length than p-values vector.}

\item{minc}{integer minimum number of genes in a group that
will be return}

\item{summarize}{character column name in metadata that will be used to group
replicates. If the column doesn't exist it'll merge the \code{time} and
the \code{col} columns, if \code{col} doesn't exist it'll use \code{time} only.
For instance, a merge between summarize and time parameters:
control_point0 ... etc}

\item{time}{character column name in metadata that will be used as
variable that changes, normally a time variable.}

\item{col}{character column name in metadata to separate
samples. Normally control/mutant}

\item{consensusCluster}{Indicates whether using \link{ConsensusClusterPlus}
or \code{\link[cluster:diana]{cluster::diana()}}}

\item{reduce}{boolean remove genes that are outliers of the cluster
distribution. \code{boxplot} function is used to flag a gene in any
group defined by \code{time} and \code{col} as outlier and it is removed
from the cluster. Not used if \code{consensusCluster} is TRUE.}

\item{cutoff}{This is deprecated.}

\item{scale}{boolean scale the \code{ma} values by row}

\item{pattern}{numeric vector to be used to find patterns like this
from the count matrix. As well, it can be a character indicating the
genes inside the count matrix to be used as reference.}

\item{groupDifference}{Minimum abundance difference between the
maximum value and minimum value for each feature. Please,
provide the value in the same range than the \code{ma} value
( if \code{ma} is in log2, \code{groupDifference} should be inside that range).}

\item{eachStep}{Whether apply \code{groupDifference} at each stem over
\code{time} variable. \strong{This only work properly for one group
with multiple time points}.}

\item{plot}{boolean plot the clusters found}

\item{fixy}{vector integers used as ylim in plot}

\item{nClusters}{an integer scalar or vector with the desired number of groups}

\item{skipDendrogram}{a boolean to run or not dendextend. Temporary fix to memory
issue in linux.}
}
\value{
list wiht two items:
\itemize{
\item \code{df} is a data.frame
with two columns. The first one with genes, the second
with the clusters they belong.
\item \code{pass} is a vector of the clusters that pass the \code{minc} cutoff.
\item \code{plot} ggplot figure.
\item \code{hr} clustering of the genes in hclust format.
\item \code{profile} normalized count data used in the plot.
\item \code{raw} data.frame with gene values summarized by biological replicates and
with metadata information attached.
\item \code{summarise} data.frame with clusters values summarized by group and
with the metadata information attached.
\item \code{normalized} data.frame with the clusters values
as used in the plot.
\item \code{benchmarking} plot showing the different patterns at different
values for clustering cuttree function.
\item \code{benchmarking_curve} plot showing how the numbers of clusters and genes
changed at different values for clustering cuttree function.
}
}
\description{
Note that this function doesn't calculate significant
difference between groups, so the
matrix used as input should be already filtered to contain only
genes that are significantly different or the most interesting genes
to study.
}
\details{
It can work with one or more groups with 2 or
more several time points.
Before calculating the genes similarity among samples,
all samples inside the same time point (\code{time} parameter) and
group (\code{col} parameter) are collapsed together, and the \code{mean}
value is the representation of the group for the gene abundance.
Then, all pair-wise gene expression is calculated using
\code{cor.test}  R function using kendall as the statistical
method. A distance matrix is created from those values.
After that, \code{\link[cluster:diana]{cluster::diana()}} is used for the
clustering of gene-gene distance matrix and cut the tree using
the divisive coefficient of the clustering, giving as well by diana.
Alternatively, if \code{consensusCluster} is on, it would use
\link{ConsensusClusterPlus} to cut the tree in stable clusters.
Finally, for each group of genes, only the ones that have genes
higher than \code{minc} parameter will be added to the figure.
The y-axis in the figure is the results of applying \code{scale()}
R function, what is similar to creating a
\code{Z-score} where values are centered to the \code{mean}  and
scaled to the \verb{standard desviation} by each gene.

The different patterns can be merged
to get similar ones into only one pattern. The expression
correlation of the patterns will be used to decide whether
some need to be merged or not.
}
\examples{
data(humanGender)
library(SummarizedExperiment)
library(ggplot2)
ma <- assays(humanGender)[[1]][1:100,]
des <- colData(humanGender)
des[["other"]] <- sample(c("a", "b"), 85, replace = TRUE)
res <- degPatterns(ma, des, time="group", col = "other")
# Use the data yourself for custom figures
 ggplot(res[["normalized"]],
        aes(group, value, color = other, fill = other)) +
  geom_boxplot() +
   geom_point(position = position_jitterdodge(dodge.width = 0.9)) +
   # change the method to make it smoother
   geom_smooth(aes(group=other), method = "lm")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degCheckFactors}
\alias{degCheckFactors}
\title{Distribution of gene ratios used to calculate Size Factors.}
\usage{
degCheckFactors(counts, each = FALSE)
}
\arguments{
\item{counts}{Matrix with counts for each samples and each gene.
row number should be the same length than pvalues vector.}

\item{each}{Plot each sample separately.}
}
\value{
ggplot2 object
}
\description{
This function check the median ratio normalization used by
DESeq2 and similarly by edgeR to visualy check whether
the median is the best size factor to represent depth.
}
\details{
This function will plot the gene ratios for each sample. To calculate
the ratios, it follows the simliar logic than DESeq2/edgeR uses, where the expression
of each gene is divided by the mean expression of that gene. The distribution
of the ratios should approximate to a normal shape and the factors should be similar
to the median of distributions. If some samples show different distribution,
the factor may be bias due to some biological or technical factor.
}
\examples{
data(humanGender)
library(SummarizedExperiment)
degCheckFactors(assays(humanGender)[[1]][, 1:10])
}
\references{
\itemize{
\item Code to calculate size factors comes from
\code{\link[DESeq2:estimateSizeFactorsForMatrix]{DESeq2::estimateSizeFactorsForMatrix()}}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report.R
\name{createReport}
\alias{createReport}
\title{Create report of RNAseq DEG anlaysis}
\usage{
createReport(g, counts, tags, pvalues, path, pop = 400, name = "DEGreport")
}
\arguments{
\item{g}{Character vector with the group the samples belong to.}

\item{counts}{Matrix with counts for each samples and each gene.
Should be same length than pvalues vector.}

\item{tags}{Genes of DEG analysis}

\item{pvalues}{pvalues of DEG analysis}

\item{path}{path to save the figure}

\item{pop}{random genes for background}

\item{name}{name of the html file}
}
\value{
A HTML file with all figures and tables
}
\description{
This function get the count matrix, pvalues, and FC of a
DEG analysis and create a report to help to detect possible problems
with the data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genePlots.R
\name{degColors}
\alias{degColors}
\title{Make nice colors for metadata}
\usage{
degColors(
  ann,
  col_fun = FALSE,
  con_values = c("grey80", "black"),
  cat_values = c("orange", "steelblue"),
  palette = "Set2"
)
}
\arguments{
\item{ann}{Data.frame with metadata information. Each column will be used to
generate a palette suitable for the values in there.}

\item{col_fun}{Whether to return a function for continuous variables
(compatible with \code{\link[ComplexHeatmap:HeatmapAnnotation]{ComplexHeatmap::HeatmapAnnotation()}}) or
the colors themself (comparible with [pheatmap::pheatmap())]).

[pheatmap::pheatmap())]: R:pheatmap::pheatmap())}

\item{con_values}{Color to be used for continuous variables.}

\item{cat_values}{Color to be used for 2-levels categorical variables.}

\item{palette}{Palette to use from \code{brewer.pal()} for
multi-levels categorical variables.}
}
\description{
The function will take a metadata table and use Set2 palette when
number of levels is > 3 or a set or orange/blue colors other wise.
}
\examples{
data(humanGender)
library(DESeq2)
library(ComplexHeatmap)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:10, idx],
  colData(humanGender)[idx,], design=~group)
th <- HeatmapAnnotation(df = colData(dse),
                       col = degColors(colData(dse), TRUE))
Heatmap(log2(counts(dse)+0.5), top_annotation = th)

custom <- degColors(colData(dse), TRUE,
          con_values = c("white", "red"),
          cat_values = c("white", "black"),
          palette = "Set1")
th <- HeatmapAnnotation(df = colData(dse),
                        col = custom)
Heatmap(log2(counts(dse)+0.5), top_annotation = th)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degObj}
\alias{degObj}
\title{Create a deg object that can be used to plot expression values
at shiny server:runGist(9930881)}
\usage{
degObj(counts, design, outfile)
}
\arguments{
\item{counts}{Output from get_rank function.}

\item{design}{Colour used for each gene.}

\item{outfile}{File that will contain the object.}
}
\value{
R object to be load into vizExp.
}
\description{
Create a deg object that can be used to plot expression values
at shiny server:runGist(9930881)
}
\examples{
data(humanGender)
library(SummarizedExperiment)
degObj(assays(humanGender)[[1]], colData(humanGender), NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degMB}
\alias{degMB}
\title{Distribution of expression of DE genes compared to the background}
\usage{
degMB(tags, group, counts, pop = 400)
}
\arguments{
\item{tags}{List of genes that are DE.}

\item{group}{Character vector with group name for each sample in the
same order than counts column names.}

\item{counts}{Matrix with counts for each samples and each gene
Should be same length than pvalues vector.}

\item{pop}{number of random samples taken for background comparison}
}
\value{
ggplot2 object
}
\description{
Distribution of expression of DE genes compared to the background
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dds <- DESeq(dds)
res <- results(dds)
degMB(row.names(res)[1:20], colData(dds)[["group"]],
  counts(dds, normalized = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/AllMethods.R
\name{deg}
\alias{deg}
\alias{deg,DEGSet-method}
\title{Method to get all table stored for an specific comparison}
\usage{
deg(object, value = NULL, tidy = NULL, top = NULL, ...)

\S4method{deg}{DEGSet}(object, value = NULL, tidy = NULL, top = NULL, ...)
}
\arguments{
\item{object}{\link{DEGSet}}

\item{value}{Character to specify which table to use.}

\item{tidy}{Return data.frame, tibble or original class.}

\item{top}{Limit number of rows to return. Default: All.}

\item{...}{Other parameters to pass for other methods.}
}
\description{
Method to get all table stored for an specific comparison
}
\references{
\itemize{
\item Testing if \code{top} is whole number or not comes from:
https://stackoverflow.com/a/3477158
}
}
\author{
Lorena Pantano
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{degResults}
\alias{degResults}
\title{Complete report from DESeq2 analysis}
\usage{
degResults(
  res = NULL,
  dds,
  rlogMat = NULL,
  name,
  org = NULL,
  FDR = 0.05,
  do_go = FALSE,
  FC = 0.1,
  group = "condition",
  xs = "time",
  path_results = ".",
  contrast = NULL
)
}
\arguments{
\item{res}{output from \code{\link[DESeq2:results]{DESeq2::results()}} function.}

\item{dds}{\code{\link[DESeq2:DESeqDataSet]{DESeq2::DESeqDataSet()}} object.}

\item{rlogMat}{matrix from \code{\link[DESeq2:rlog]{DESeq2::rlog()}} function.}

\item{name}{string to identify results}

\item{org}{an organism annotation object, like org.Mm.eg.db.
NULL if you want to skip this step.}

\item{FDR}{int cutoff for false discovery rate.}

\item{do_go}{boolean if GO enrichment is done.}

\item{FC}{int cutoff for log2 fold change.}

\item{group}{string column name in colData(dds) that
separates samples in meaninful groups.}

\item{xs}{string column name in colData(dss)
that will be used as X axes in plots (i.e time)}

\item{path_results}{character path where files are stored.
NULL if you don't want to save any file.}

\item{contrast}{list with character vector indicating the
fold change values from different comparisons to add to the output table.}
}
\value{
ggplot2 object
}
\description{
Complete report from DESeq2 analysis
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dse <- DESeq(dse)
res <- degResults(dds = dse, name = "test", org = NULL,
  do_go = FALSE, group = "group", xs = "group", path_results = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/signature.R
\name{degSignature}
\alias{degSignature}
\title{Plot gene signature for each group and signature}
\usage{
degSignature(
  counts,
  signature,
  group = NULL,
  metadata = NULL,
  slot = 1,
  scale = FALSE
)
}
\arguments{
\item{counts}{expression data. It accepts bcbioRNASeq, DESeqDataSet and
SummarizedExperiment. As well, data.frame or matrix is supported, but
it requires metadata in that case.}

\item{signature}{data.frame with two columns: a) genes that match
row.names of counts, b) label to classify the gene inside a group.
Normally, cell tissue name.}

\item{group}{character in metadata used to split data into different
groups.}

\item{metadata}{data frame with sample information. Rownames
should match \code{ma} column names
row number should be the same length than p-values vector.}

\item{slot}{slotName in the case of SummarizedExperiment objects.}

\item{scale}{Whether to scale or not the expression.}
}
\value{
ggplot plot.
}
\description{
Given a list of genes beloging to a different classes, like
markers, plot for each group, the expression values for all the samples.
}
\examples{
data(humanGender)
data(geneInfo)
degSignature(humanGender, geneInfo, group = "group")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{degMerge}
\alias{degMerge}
\title{Integrate data comming from degPattern into one data object}
\usage{
degMerge(
  matrix_list,
  cluster_list,
  metadata_list,
  summarize = "group",
  time = "time",
  col = "condition",
  scale = TRUE,
  mapping = NULL
)
}
\arguments{
\item{matrix_list}{list expression data for each element}

\item{cluster_list}{list df item from degPattern output}

\item{metadata_list}{list data.frames from each element
with design experiment. Normally \code{colData} output}

\item{summarize}{character column to use to group samples}

\item{time}{character column to use as x-axes in figures}

\item{col}{character column to color samples in figures}

\item{scale}{boolean scale by row expression matrix}

\item{mapping}{data.frame mapping table in case elements use
different ID in the row.names of expression matrix. For instance,
when integrating miRNA/mRNA.}
}
\value{
A data.frame with information on what genes are in each cluster in
all data set, and the correlation value for each pair cluster comparison.
}
\description{
The simplest case is if you want to conbine the pattern profile
for gene expression data and proteomic data. It will use the first element
as the base for the integration. Then, it will loop through clusters
and run \link{degPatterns} in the second data set to detect patterns that match
this one.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DEGreport-package.R
\docType{package}
\name{DEGreport-package}
\alias{DEGreport}
\alias{DEGreport-package}
\title{Deprecated functions in package DEGreport}
\description{
These functions are provided for compatibility with older versions
of DEGreport only and will be defunct at the next release.
}
\details{
The following functions are deprecated and will be made defunct;
use the replacement indicated below:
\itemize{
\item degRank, degPR, degBIcmd, degBI, degFC, degComb, degNcomb: DESeq2::lcfShrink.
This function was trying to avoid big FoldChange
in variable genes. There are other methods nowadays like lcfShrink function.
DEGreport
}
}
\seealso{
Useful links:
\itemize{
  \item \url{http://lpantano.github.io/DEGreport/}
  \item Report bugs at \url{https://github.com/lpantano/DEGreport/issues}
}

}
\author{
\strong{Maintainer}: Lorena Pantano \email{lorena.pantano@gmail.com}

Other contributors:
\itemize{
  \item John Hutchinson [contributor]
  \item Victor Barrera [contributor]
  \item Mary Piper [contributor]
  \item Radhika Khetani [contributor]
  \item Kenneth Daily [contributor]
  \item Thanneer Malai Perumal [contributor]
  \item Rory Kirchner [contributor]
  \item Michael Steinbaugh [contributor]
  \item Ivo Zeller [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degMV}
\alias{degMV}
\title{Correlation of the standard desviation and the mean of the abundance of a
set of genes.}
\usage{
degMV(group, pvalues, counts, sign = 0.01)
}
\arguments{
\item{group}{Character vector with group name for each sample in the
same order than counts column names.}

\item{pvalues}{pvalues of DEG analysis.}

\item{counts}{Matrix with counts for each samples and each gene.}

\item{sign}{Defining the cutoff to label significant features.
row number should be the same length than pvalues vector.}
}
\value{
ggplot2 object
}
\description{
Correlation of the standard desviation and the mean of the abundance of a
set of genes.
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dds <- DESeq(dds)
res <- results(dds)
degMV(colData(dds)[["group"]],
      res[, 4],
      counts(dds, normalized = TRUE))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degVar}
\alias{degVar}
\title{Distribution of pvalues by standard desviation range}
\usage{
degVar(pvalues, counts)
}
\arguments{
\item{pvalues}{pvalues of DEG analysis}

\item{counts}{Matrix with counts for each samples and each gene.
row number should be the same length than pvalues vector.}
}
\value{
ggplot2 object
}
\description{
This function pot the p-valyes distribution colored by
the quantiles of the standard desviation of count data.
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dds <- DESeq(dds)
res <- results(dds)
degVar(res[, 4], counts(dds))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{degPlotCluster}
\alias{degPlotCluster}
\title{Plot clusters from degPattern function output}
\usage{
degPlotCluster(
  table,
  time,
  color = NULL,
  min_genes = 10,
  process = FALSE,
  points = TRUE,
  boxes = TRUE,
  smooth = TRUE,
  lines = TRUE,
  facet = TRUE,
  cluster_column = "cluster",
  prefix_title = "Group: "
)
}
\arguments{
\item{table}{\code{normalized} element from \code{\link[=degPatterns]{degPatterns()}} output.
It can be a data.frame with the following columns in there:
\verb{genes, sample, expression, cluster, xaxis_column, color_column}.}

\item{time}{column name to use in the x-axis.}

\item{color}{column name to use to color and divide the samples.}

\item{min_genes}{minimum number of genes to be added to the plot.}

\item{process}{whether to process the table if it is not
ready for plotting.}

\item{points}{Add points to the plot.}

\item{boxes}{Add boxplot to the plot.}

\item{smooth}{Add regression line to the plot.}

\item{lines}{Add gene lines to the plot.}

\item{facet}{Split figures based on cluster ID.}

\item{cluster_column}{column name if cluster is in a column
with a different name. Usefull, to plot cluster with different
cutoffs used when grouping genes from the clustering step.}

\item{prefix_title}{text to add before the cluster ID in the figure title.}
}
\value{
\link{ggplot2} object.
}
\description{
This function helps to format the cluster plots from \code{\link[=degPatterns]{degPatterns()}}.
It allows to control the layers and it returns a ggplot object that
can accept more ggplot functions to allow customization.
}
\examples{
data(humanGender)
library(SummarizedExperiment)
library(ggplot2)
ma <- assays(humanGender)[[1]][1:100,]
des <- colData(humanGender)
des[["other"]] <- sample(c("a", "b"), 85, replace = TRUE)
res <- degPatterns(ma, des, time="group", col = "other", plot = FALSE)
degPlotCluster(res$normalized, "group", "other")
degPlotCluster(res$normalized, "group", "other", lines = FALSE)

library(dplyr)
library(tidyr)
library(tibble)
table <- rownames_to_column(as.data.frame(ma), "genes") \%>\%
    gather("sample", "expression", -genes) \%>\%
    right_join(distinct(res$df[,c("genes", "cluster")]),
               by = "genes") \%>\%
    left_join(rownames_to_column(as.data.frame(des), "sample"),
              by = "sample") \%>\% 
              as.data.frame()
degPlotCluster(table, "group", "other", process = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/humanGender-data.R
\docType{data}
\name{humanGender}
\alias{humanGender}
\title{DGEList object for DE genes betwen Male and Females}
\format{
DGEList
}
\source{
gEUvadis
}
\usage{
data(humanGender)
}
\description{
DGEList object for DE genes betwen Male and Females
}
\author{
Lorena Pantano, 2017-08-37
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clustering.R
\name{degPCA}
\alias{degPCA}
\title{smart PCA from count matrix data}
\usage{
degPCA(
  counts,
  metadata = NULL,
  condition = NULL,
  pc1 = "PC1",
  pc2 = "PC2",
  name = NULL,
  shape = NULL,
  data = FALSE
)
}
\arguments{
\item{counts}{matrix with count data}

\item{metadata}{dara.frame with sample information}

\item{condition}{character column in metadata to use to color samples}

\item{pc1}{character PC to plot on x-axis}

\item{pc2}{character PC to plot on y-axis}

\item{name}{character if given, column in metadata to print label}

\item{shape}{character if given, column in metadata to shape points}

\item{data}{Whether return PCA data or just plot the PCA.}
}
\value{
if \verb{results <-} used, the function return the output
of \code{\link[=prcomp]{prcomp()}}.
}
\description{
nice plot using ggplot2 from prcomp function
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dse <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
colData(humanGender)[idx,], design=~group)
degPCA(log2(counts(dse)+0.5), colData(dse),
  condition="group", name="group", shape="group")
}
\author{
Lorena Pantano, Rory Kirchner, Michael Steinbaugh
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{degQC}
\alias{degQC}
\title{Plot main figures showing p-values distribution and mean-variance correlation}
\usage{
degQC(counts, groups, object = NULL, pvalue = NULL)
}
\arguments{
\item{counts}{Matrix with counts for each samples and each gene.}

\item{groups}{Character vector with group name for each sample in the
same order than counts column names.}

\item{object}{\link{DEGSet} oobject.}

\item{pvalue}{pvalues of DEG analysis.}
}
\value{
ggplot2 object
}
\description{
This function joins the output of \code{\link[DEGreport]{degMean}},
\code{\link[DEGreport]{degVar}} and \code{\link[DEGreport]{degMV}} in a
single plot. See these functions for further information.
}
\examples{
data(humanGender)
library(DESeq2)
idx <- c(1:10, 75:85)
dds <- DESeqDataSetFromMatrix(assays(humanGender)[[1]][1:1000, idx],
  colData(humanGender)[idx,], design=~group)
dds <- DESeq(dds)
res <- results(dds)
degQC(counts(dds, normalized=TRUE), colData(dds)[["group"]],
  pvalue = res[["pvalue"]])
}
